// Bent-Straight waveguide coupler example

#include "mr.h"

int main()
{
    double R = 15.0;       // Radius of bent waveguide

    int layer = 1;         //  no. of internal layer
    double g = 0.4;        // coupler separation distance
    double d1 = 0.5;       // width of bend waveguide
    double d2 = 0.4;       // width of straight waveguide

    double n_b   = 1.0;    // background R. I.
    double n_bwg = 1.5;    // bend core R. I.
    double n_swg = 1.5;    // straight waveguide core R. I.

    double lambda = 1.05;  // vacuum wavelength

    //--------------Define bent waveguide---------------------------------
    Dvector Bhx(layer+1);
    Dvector Bn(layer+2);

    Bhx(0) = -d1;
    Bhx(1) = 0.0;

    Bn(0) = n_b;
    Bn(1) = n_bwg;
    Bn(2) = n_b;

    BSLAMS_Parameters bpar;

    BSlabWaveguide bwg(layer, Bhx, lambda, Bn, R);

    // defining container to hold mode definition of bent waveguide
    BSlabModeArray bma;

    // Finding modes of bent waveguide. Roots are searched on the domain
    // [b0, b1]x[a0, a1] in the complex plain

    double b0 = 5.0;       // Guess for beta
    double b1 = 8.5;       // Guess for beta

    double a0 = -6e-5;     // Guess for alpha
    double a1 = -3e-5;     // Guess for alpha
	
    int Nb = 10;           // [b0, b1] is divided in to Nb points
    int Na = 8;            // [a0, a1] is divided in to Na points

    
    // finding modes of bent waveguide
    int Nbm = bend_mode_analysis( bwg, TE, b0, b1, Nb, a0, a1,Na, bpar, bma);

    int mc = 1; //number of bent modes considered

    if (Nbm < mc)
    {
	cout << "Number of bent modes is less than " << mc << ".\n";
	exit(1);
    }
    else
    {
	while (Nbm > mc)
	{
	    cout << "Bent modes : "  << Nbm << "\t";
	    bma.remove(mc);
	    Nbm = bma.num;
	    cout << " 1 mode removed. --> Now no mode : " << Nbm << ".\n";
	}
    }


    // Power normalize the modes
    Interval Bdisplay(R-5.0, R + 5.0);
    double tmp_totpower;


    for (int j= 0; j < Nbm; j++)
    {
        // Note that instead of analytic power, power contained in interval
	// is used.
  	tmp_totpower =bma(j).totpower(Bdisplay, 0.0, 3000); 
	bma(j).normalize(tmp_totpower); 
	
	// To check mode profiles
	// rep + img + abs
//	bma(j).mfile(EY, MOD, 1.0, Bdisplay, 1000, '0', '0' + j, 'r');
    }


    //--------------Define straight waveguide---------------------------------
	
    Dvector Shx(layer+1);
    Dvector Sn(layer+2);
	

    Shx(0) = R + g;
    Shx(1) = R + g + d2;
	
    Sn(0) = n_b;
    Sn(1) = n_swg;
    Sn(2) = n_b;

    Waveguide swg(layer, Shx, lambda, Sn);
	
    // defining container to hold mode definition of SWG
    ModeArray sma;

    // finding modes of SWG
    int Nsm = modeanalysis( swg, TE, sma, 0);

    //-----Define bent straight waveguide coupler--------------------------
	
    // setting of computational window [x0, x1]x[z0, z1]:
    // hx is step size for interval in the x-direction [x0, x1].
    // hz is step size for interval in the z-direction [z0, z1].
    // Nx and Nz are number of discretization points in x and z direction.

    int Nx, Nz;            
    
    double hx = 0.005;
    double hz = 0.1;
    double z0 = -8.0;
    double z1 =  8.0;
    double x0 =  10.0;
    double x1 =  20.0;
    
    Nx = (int)((x1 - x0)/hx);
    Nz = (int)((z1 - z0)/hz);

    // Initial amplitudes for couplers.
    // Note that the convention is: 
    // First initial amplitudes for "bent" modes and then for "straight" modes.

    Cvector initial_Amp(bma.num + sma.num);
    
    // Only the fundamental mode of straight waveguide is excited with a 
    // unit power.
    for(int i =0; i < bma.num + sma.num; i++) initial_Amp(i)=CC0;
    initial_Amp(bma.num + sma.num -1) = CC1;


    // Define a coupler
    BSCoupler BS(bma, sma, g, x0, x1, Nx, z0, z1, Nz);

    // Solve coupler mode equation over [x0, x1]x[z0, z1] with initial
    // amplitudes initial_Amp and get transfer matrix T.
    BS.get_transfer_matrix(initial_Amp,0);

    // After getting transfer matrix, get scattering matrix S, and display it.
    BS.get_scattering_matrix(1);

    // Coupler field
    double x, z;
    x=13.0;
    z=7.0;

    Complex fc=BS.field(EY, x, z);
    double  fr=BS.field(EY, MOD, x, z);

    cout << "\nComplex valued coupler field at (x,z)=("
	 << x <<", " << z <<"): (" << fc.re <<", " 
	 << fc.im <<").\n"
	 << "Absolute value of coupler field at (x,z)=("
	 << x <<", " << z <<"): " << fr <<"\n\n";

    // Plot real part of EY field.
    BS.plot(EY, REP, 13.0, 17.0, -8.0, 8.0, 200, 500, '0', '0');

    return 0;
}
