// Example of microdisk resonator field plots.

#include "mr.h"

int main()
{
    cout.setf(ios::scientific, ios::floatfield);

    double R = 5.0;         // Radius of ring

    int layer = 1;          // no. of internal layer
    double d0 = 0.2;        // separation distance
    double d1 = 0.0;        // width of bend waveguide: for cavity = 0
    double d2 = 0.4;        // width of straight waveguide

    double n_b =1.0;        // background R. I.
    double n_bwg=1.5;       // bend core R. I.
    double n_swg = 1.5;     // straight waveguide core R. I.

    double lambda;          // vacuum wavelength

    // Enter lambda = 1.055  for off resonance plots
    //       lambda = 1.0483 for resonance corresponding to TE1 cavity mode
    //       lambda = 1.043  for resonance corresponding to TE0 cavity mode

    cout << "wavelength=";  
    cin >> lambda;


    //--------------Define bent waveguide---------------------------------

    Dvector Bhx(layer+1);
    Dvector Bn(layer+2);
	
    Bhx(0) = -d1;
    Bhx(1) = 0.0;
    
    Bn(0) = n_bwg; //Note: For disk, substrate R.I= core R.I.
    Bn(1) = n_bwg;
    Bn(2) = n_b;
    
    BSLAMS_Parameters bpar;
	
    BSlabWaveguide bwg(layer, Bhx, lambda, Bn, R);
	
    // defining container to hold mode definition of bent waveguide
    BSlabModeArray bma;
	
    double b0 = 6.0;
    double b1 = 9.5;

    double a0 = -6e-5;
    double a1 = -1e-4;

    int Nb = 5;
    int Na = 8;

    // finding modes of bent waveguide
    int Nbm=bend_mode_analysis(bwg, TE, b0, b1, Nb, a0, a1,Na, bpar, bma);

    int mc = 3; //no. of cavity modes considered

    if (Nbm < mc)
    {
	cout << "No. of bent modes is less than " << mc << "\n";
	exit(1);
    }
    else
    {
	while (Nbm > mc)
	{
	    cout << "Bent modes : "  << Nbm << "\t";
	    bma.remove(mc);
	    Nbm = bma.num;
	    cout << " 1 mode removed. --> Now no mode : " << Nbm << "\n";
	}
    }
	
    Interval Bdisplay(bwg.R - 4.0, bwg.R+ 5.0);
    double tmp_totpower;
	
    char index[10] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9'};
	
    for (int j= 0; j < Nbm; j++)
    {
	tmp_totpower =bma(j).totpower(0.0); 
	bma(j).normalize(tmp_totpower); // to normalize modal power
	    
	//plot of only Ey/Hy field
//       bma(j).mfile(HY, 1.0, Bdisplay, 600, '0', '0'+j, 'r'); //real+im
//	 bma(j).mfile(HY, MOD, 1.0, Bdisplay, 1000, '1', '0'+j, 'L'); //abs
	    
	// plot of Ey field and phase profile
//	 bma(j).mfile(EY, 1.0, Bdisplay, 800, '0', '0'+j, 'a');	 
	    
    }

    //------------------Define straight waveguide-------------------------

    Dvector Shx(layer+1);
    Dvector Sn(layer+2);
	
	
    Shx(0) = R + d0;
    Shx(1) = R + d0 + d2; 
	
    Sn(0) = n_b;
    Sn(1) = n_swg;
    Sn(2) = n_b;
	
	
    // Defining Straight waveguide
    Waveguide swg(layer, Shx, lambda, Sn);
	
    // defining container to hold mode definition of SWG
    ModeArray sma;

    // finding modes of SWG
    int no_of_modes = modeanalysis( swg, TE, sma, 0);


    //------Defining bent-straight waveguide coupler----------

    int Nx, Nz;

    double hx = 0.005;
    double hz = 0.1;

    double z0 = -4.0;
    double z1 =  8.0;   // Note that for plotting purpose coupler is 
                        // extended till z1= 8.0
    double x0 =   0.0;
    double x1 =  15.0;

    Nx = (int)((x1 - x0)/hx);
    Nz = (int)((z1 - z0)/hz);

    // For coupled field with bend amp = 0 and SWG amp = 1
    Cvector initial_Amp(bma.num + sma.num);
    for(int i=0; i < bma.num + sma.num; i++) initial_Amp(i) = CC0;
    initial_Amp(bma.num) = CC1;

    BSCoupler BSi(bma, sma, d0, x0, x1, Nx, z0, z1, Nz);
    BSi.get_transfer_matrix( initial_Amp,0);
    BSi.get_scattering_matrix(1);


    BSCoupler BSo;
    BSo= BSi;

    Cvector iAmp(1), aAmp(1);
    iAmp(0) = CC1;
    aAmp(0) = CC0;
	
    MR mr(BSi, BSo, -10, 10, 250, -10, 10, 250, iAmp, aAmp);
    
    // Compute cavity mode amplitudes for a computational window
    // with z1 < R.
    mr.set_amp(1, 4.0);

//	mr.viewer(-7.0, 7.0, -7.0, 7.0, 300, 300, 'x', 'y');
    mr.plot(EY, MOD, -7.0, 7.0, -7.0, 7.0, 400, 400, 'x', 'y');
//	mr.plot(HX, MOD, -7.0, 7.0, -7.0, 7.0, 400, 400, 'x', 'y');
//	mr.plot(HZ, MOD, -7.0, 7.0, -7.0, 7.0, 400, 400, 'x', 'y');

//    mr.plot(EY, REP, -7.0, 7.0, -7.0, 7.0, 400, 400, 'x', 'y');
//    mr.movie(-7.0, 7.0, -7.0, 7.0, 600, 600, 25, '0', '0');

//	mr.plot(HX, REP, -7.0, 7.0, -7.0, 7.0, 400, 400, 'x', 'y');
//	mr.plot(HZ, REP, -7.0, 7.0, -7.0, 7.0, 400, 400, 'x', 'y');


    
    return 0;
}
