// Example of microring resonator

#include "mr.h"

int main()
{
    double R = 5.0;        // Radius of ring

    int layer = 1;         // no. of internal layer
    double g = 0.2;        // separation distance

    double d1 = 0.5;       // width of bent waveguide
    double d2 = 0.4;       // width of straight waveguide

    double n_b   = 1.0;    // background R. I.
    double n_bwg = 1.5;    // bend core R. I.
    double n_swg = 1.5;    // straight waveguide core R. I.

    double lambda=1.05;         // vacuum wavelength

    int Nbmc, Nsmc; 

    Nbmc=1; //no. of BWG modes considered
    Nsmc=1; //no. of SWG modes considered

    Cvector ISAmp(Nsmc);
    ISAmp(0) = CC1; //only SWG fundamental mode with unit amp. launched

    //--------------Define bent waveguide---------------------------------

    Dvector Bhx(layer+1);
    Dvector Bn(layer+2);
	
    Bhx(0) = -d1;
    Bhx(1) = 0.0;
    
    Bn(0) = n_b;
    Bn(1) = n_bwg;
    Bn(2) = n_b;
    
    BSLAMS_Parameters bpar;
    
    
    BSlabWaveguide bwg(layer, Bhx, lambda, Bn, R);
	
    // defining container to hold mode definition of bent waveguide
    BSlabModeArray bma;
    
    double b0 = 6.0;
    double b1 = 9.0;
    
    double a0 = -1e-5;
    double a1 = -2e-5;
    
    int Nb = 10;
    int Na = 10;
    
    // finding modes of bent waveguide
    int Nbm=bend_mode_analysis( bwg, TE, b0, b1, Nb, a0, a1,Na, bpar, bma);
    
    if (Nbm < Nbmc)
    {
	cout << "No. of bent modes is less than " << Nbmc << "\n";
	exit(1);
    }
    else
    {
	while (Nbm > Nbmc)
	{
	    cout << "Bent modes : "  << Nbm << "\t";
	    bma.remove(Nbmc);
	    Nbm = bma.num;
	    cout << " 1 mode removed. --> Now no mode : " << Nbm << "\n";
	}
    }
    
    
    Interval Bdisplay(0.01, R+2.0);
    double tmp_totpower;
    
    for (int j= 0; j < Nbm; j++)
    {
  	tmp_totpower =bma(j).totpower(Bdisplay, 0.0, 5000); 
	bma(j).normalize(tmp_totpower); // to normalize modal power
	
	// rep + img + abs
//	bma(j).mfile(EY, 1.0, Bdisplay, 1000, '0', '0'+ j, 'r');  
//	bma(j).mfile(EY, MOD, 1.0, Bdisplay, 1000, '0', '0' + j, 'r');
    }
    
    
    //------------------Define straight waveguide-------------------------
    
    Dvector Shx(layer+1);
    Dvector Sn(layer+2);
    
    
    Shx(0) = R + g;
    Shx(1) = R + g + d2;
    
    Sn(0) = n_b;
    Sn(1) = n_swg;
    Sn(2) = n_b;
    
    
    // Defining Straight waveguide
    Waveguide swg(layer, Shx, lambda, Sn);
    
    // defining container to hold mode definition of SWG
    ModeArray sma;
    
    // finding modes of SWG
    int Nsm = modeanalysis( swg, TE, sma, 0);
    
    if (Nsm < Nsmc)
    {
	cout << "No. of straight waveguide modes is less than " 
	     << Nsmc << "\n";
	exit(1);
    }
    else
    {
	while (Nsm > Nsmc)
	{
	    cout << "Straight waveguide modes : "  << Nsm << "\t";
	    sma.remove(Nsmc);
	    Nsm = sma.num;
	    cout << " 1 mode removed. --> Now no mode : " << Nsm << "\n";
	}
    }
    
    
    //-----Defining bent-straight waveguide coupler----------

    // setting coupler computational window

    int Nx, Nz;
    
    double hx = 0.005;
    double hz = 0.1;

    double z0 = -4.0;
    double z1 =  4.0;
    double x0 =  0.0;
    double x1 =  10.0; 
    
    Nx = (int)((x1 - x0)/ hx);
    Nz = (int)((z1 - z0)/ hz);

    // For coupled field with bent mode amp = 0 and straight mode amp = 1

    Cvector initial_Amp(bma.num + sma.num);
    for(int i=0; i < bma.num + sma.num; i++) initial_Amp(i) = CC0;
    initial_Amp(bma.num) = CC1;

    // In-port-Through-port coupler
    BSCoupler BSi(bma, sma, g, x0, x1, Nx, z0, z1, Nz);
    BSi.get_transfer_matrix( initial_Amp,0);
    BSi.get_scattering_matrix(1);


    // Add-port-Drop-port coupler
    BSCoupler BSo;
    BSo= BSi;

    //--------Defining resonator----------
    Cvector iAmp(1), aAmp(1);
    iAmp(0) = CC1;
    aAmp(0) = CC0;
	
    MR mr(BSi, BSo, -10, 10, 250, -10, 10, 250, iAmp, aAmp);


    // Absolute value of various mode amplitudes (see the figure of 
    // functional decomposition of resonator.
    cout <<"Absolute value of various mode amplitudes:\n"
	 <<"A=" << mr.Ap(0).abs()
	 <<"\nB=" << mr.Bp(0).abs()
	 <<"\na=" << mr.ap(0).abs()
	 <<"\nb=" << mr.bp(0).abs()
	 <<"\ntilde(A)=" << mr.Cm(0).abs()
	 <<"\ntilde(B)=" << mr.Dm(0).abs()
	 <<"\ntilde(a)=" << mr.cm(0).abs()
	 <<"\ntilde(b)=" << mr.dm(0).abs()
	 <<"\n";

    //------Accessing resonator fields---------
    double x, z;
    x=6.0;
    z=4.0;

    Complex fc=mr.field(EY, x, z);
    double fr=mr.field(EY, MOD, x, z);

    cout <<"\ny component of Microresonator field at "
	 <<"(x,z)=(" << x <<", " <<z <<"):\n"
	 <<"Complex valued field: (" << fc.re <<", " <<fc.im <<")\n"
	 <<"Absolute value of field: " << fr <<"\n";

    return 0;
}
