// Example of microring resonator

#include "mr.h"

int main()
{
    cout.setf(ios::scientific, ios::floatfield);

   
    double R = 5.0;        // Radius of ring

    int layer = 1;         // no. of internal layer
    double g = 0.2;        // separation distance

    double d1 = 0.5;       // width of bent waveguide
    double d2 = 0.4;       // width of straight waveguide

    double n_b   = 1.0;    // background R. I.
    double n_bwg = 1.5;    // bend core R. I.
    double n_swg = 1.5;    // straight waveguide core R. I.

    double lambda;         // vacuum wavelength

    // setting coupler computational window

    int Nx, Nz;
    
    double hx = 0.005;
    double hz = 0.1;

    double z0 = -4.0;
    double z1 =  4.0;
    double x0 =  0.0;
    double x1 =  10.0; 
    
    Nx = (int)((x1 - x0)/ hx);
    Nz = (int)((z1 - z0)/ hz);


    // nodal wavelengths. Bent mode propagation constants and coupler 
    // scattering matrix (defined at a point) at these nodal
    // wavelengths is used for interpolation purpose.

    double ls[]={1.0, 1.05, 1.1}; 

    // File to store bent mode propagation constants
    ofstream gout("gamma_in.data", ios::out);

    // File to store straight waveguide  mode propagation constants for 
    // In-port-Through-port straight waveguide.
    ofstream bout("beta_in.data", ios::out);

    // File to store straight waveguide  mode propagation constants for 
    // Add-port-Drop-port-straight waveguide. 
    ofstream bout1("beta1_in.data", ios::out);

    // File to store coupler scattering matrix (defined at a point).
    ofstream Srout("Sr_in.data", ios::out);
    
    int Nbmc, Nsmc; 

    Nbmc=1; //no. of BWG modes considered
    Nsmc=1; //no. of SWG modes considered

    Cvector ISAmp(Nsmc);
    ISAmp(0) = CC1; //only SWG fundamental mode with unit amp. launched

    gout << R << "\n" << Nbmc <<"\n";

    bout << Nsmc <<"\n";
    bout1 << Nsmc <<"\n";

    for(int i=0; i < Nsmc; i++)
    {
	bout <<ISAmp(i).re << "\t" << ISAmp(i).im <<"\t";
	bout1 << 0.0 << "\t" << 0.0 <<"\t"; // no input at add port
    }

    bout <<"\n";
    bout1 <<"\n";

    Srout << Nbmc <<"\t" << Nsmc <<"\n"
	  << z0 << "\t" << z1 <<"\n";

    for (int l=0; l < (int)(sizeof(ls)/sizeof(double)); l++)
    {
	lambda = ls[l];

	//--------------Define bent waveguide---------------------------------
	Dvector Bhx(layer+1);
	Dvector Bn(layer+2);
	
	Bhx(0) = -d1;
	Bhx(1) = 0.0;
	
	Bn(0) = n_b;
	Bn(1) = n_bwg;
	Bn(2) = n_b;
	
	BSLAMS_Parameters bpar;
    

	BSlabWaveguide bwg(layer, Bhx, lambda, Bn, R);
	
	// defining container to hold mode definition of bent waveguide
	BSlabModeArray bma;
	
	double b0 = 6.0;
	double b1 = 9.0;
	
	double a0 = -1e-5;
	double a1 = -2e-5;
	
	int Nb = 10;
	int Na = 10;
    
	// finding modes of bent waveguide
	int Nbm=bend_mode_analysis( bwg, TE, b0, b1, Nb, a0, a1,Na, bpar, bma);

	if (Nbm < Nbmc)
	{
	    cout << "No. of bent modes is less than " << Nbmc << "\n";
	    exit(1);
	}
	else
	{
	    while (Nbm > Nbmc)
	    {
		cout << "Bent modes : "  << Nbm << "\t";
		bma.remove(Nbmc);
		Nbm = bma.num;
		cout << " 1 mode removed. --> Now no mode : " << Nbm << "\n";
	    }
	}
   

	Interval Bdisplay(0.01, R+2.0);
	double tmp_totpower;
	
	for (int j= 0; j < Nbm; j++)
	{
  	tmp_totpower =bma(j).totpower(Bdisplay, 0.0, 5000); 
	bma(j).normalize(tmp_totpower); // to normalize modal power
	
	// rep + img + abs
//	bma(j).mfile(EY, 1.0, Bdisplay, 1000, '0', '0'+ j, 'r');  
//	bma(j).mfile(EY, MOD, 1.0, Bdisplay, 1000, '0', '0' + j, 'r');
	}

	
    //------------------Define straight waveguide-------------------------
	
	Dvector Shx(layer+1);
	Dvector Sn(layer+2);
	
	
	Shx(0) = R + g;
	Shx(1) = R + g + d2;
	
	Sn(0) = n_b;
	Sn(1) = n_swg;
	Sn(2) = n_b;
	
	
    // Defining Straight waveguide
	Waveguide swg(layer, Shx, lambda, Sn);
	
    // defining container to hold mode definition of SWG
	ModeArray sma;
	
	// finding modes of SWG
	int Nsm = modeanalysis( swg, TE, sma, 0);

	if (Nsm < Nsmc)
	{
	    cout << "No. of straight waveguide modes is less than " 
		 << Nsmc << "\n";
	    exit(1);
	}
	else
	{
	    while (Nsm > Nsmc)
	    {
		cout << "Straight waveguide modes : "  << Nsm << "\t";
		sma.remove(Nsmc);
		Nsm = sma.num;
		cout << " 1 mode removed. --> Now no mode : " << Nsm << "\n";
	    }
	}


	//-----Defining bent-straight waveguide coupler----------
	
	
	Cvector initial_Amp(bma.num + sma.num);
	
	for(int i =0; i < bma.num + sma.num; i++) initial_Amp(i)=CC0;
	initial_Amp(bma.num + sma.num -1) = CC1;
	
	BSCoupler BS(bma, sma, g, x0, x1, Nx, z0, z1, Nz);
	BS.get_transfer_matrix(initial_Amp,1);
	BS.get_scattering_matrix(2);
	
	// get a scattering matrix defined at a point for interpolation
	Cmatrix Sr;
	Sr=BS.reduced_S();

	// writing data to files
	
	gout << lambda;
	for (int i=0; i < Nbm; i++)
	    gout << "\t" << bma(i).beta << "\t" << -bma(i).alpha;
	gout <<"\n";
    
	bout << lambda;
	bout1 << lambda;

	for (int i=0; i < Nsm; i++)
	{
	    bout << "\t" << sma(i).beta;
	    bout1 << "\t" << sma(i).beta;
	}

	bout <<"\n";
	bout1 <<"\n";

	Srout << lambda;
	for (int i=0; i < Sr.r; i++)
	{
	    for (int j=0; j < Sr.c; j++)
		Srout << "\t" << Sr(i,j).re << "\t" << Sr(i,j).im;
	}
	Srout << "\n";

    }
    
    gout.close();
    bout.close();
    bout1.close();
    Srout.close();


    // To get a resonator spectral response.
    // Wavelength interval [1.0, 1.1] is divided into hl points.

    int hl = 501;
    
    // get quadratic interpolated bent mode propagation constants 
    // for wavelength interval [1.0, 1.1].
    interpolated_gamma("gamma_in.data", 
		       "gamma_out.data", 'Q',hl, 1);

    // get quadratic interpolated In-port-Through-port straight waveguide 
    // mode propagation constants for wavelength interval [1.0, 1.1].
    interpolated_beta("beta_in.data", 
		      "beta_out.data", 'Q',hl, 1);

    // get quadratic interpolated Add-port-Drop-port straight waveguide 
    // mode propagation constants for wavelength interval [1.0, 1.1].
    interpolated_beta("beta1_in.data", 
		      "beta1_out.data", 'Q',hl, 1);

    // get quadratic interpolated In-port-Through-port coupler scattering 
    // matrix (reduced at a point) for wavelength interval [1.0, 1.1].
    interpolated_reduced_S("Sr_in.data", 
			   "Sr_i_out.data", 'Q', hl, 1);

    // get quadratic interpolated In-port-Through-port coupler scattering 
    // matrix (reduced at a point) for wavelength interval [1.0, 1.1].
    // Note: In this case, the two couplers are identical.
    interpolated_reduced_S("Sr_in.data", 
			   "Sr_a_out.data", 'Q', hl, 1);

    // get quadratic interpolated spectral response. See file "ps.data".
    interpolated_power_spectrum("gamma_out.data",
			  "beta_out.data",
			  "beta1_out.data",
			  "Sr_i_out.data",
			  "Sr_a_out.data",
			  "ps.data");


    return 0;
}
