/*
 * Circurs - Circular optical microresonators simulation tools
 * Kirankumar R. Hiremath (k.r.hiremath@ieee.org)
 * University of Twente, Department of Applied Mathematics 
 * P.O. Box 217, 7500AE Enschede, The Netherlands
 * (2005)
 */

/*
 * bessel.h
 * Computation of complex order Bessel and Hankel functions.
 */

/*
 * KODE = 1 => without scaling
 * KODE = 2 => with scaling

  Error codes:

  IERR =  0 => :-) Success
  IERR =  1 => inout error
  IERR =  2 => overflow. no computation. Re(\zeta) too large for KODE = 1
  IERR =  3 => |z| too large. ans may wrong.
  IERR =  4 => |z| too large. no computation.
  IERR =  5 => no computation, algorithm termination condition not met
  IERR = 11 => situation beyond scope of routines
  IERR = 12 => division by zero
*/

#include <iostream>
#include <complex>
#include <iomanip>
#include <cmath>



// macro definition for Pi
#define Pi M_PI   

// Not-a-Number
#define nan sqrt(-1.0)   

using namespace std;

// Exception handeling function.
void bessel_error(char *s // Error message to be displayed
    );

// Compute Bessel function of first kind.
extern complex<double> BesselJ(
    complex<double> order, // Order of Bessel function
    complex<double> argument, // Argument of Bessel function
    int KODE, // Scaling flag. =1 (without scaling), =2 (with scaling)
    int *NZ, // Underflow indicator. =0 (Normal return), =1(underflow)
    int *IERR // Output error code
    );

// Compute Bessel function of second kind.
extern complex<double> BesselY(
    complex<double> order, // Order of Bessel function
    complex<double> argument, // Argument of Bessel function
    int KODE, // Scaling flag. =1 (without scaling), =2 (with scaling)
    int *NZ, // Underflow indicator. =0 (Normal return), =1(underflow)
    int *IERR // Output error code
    );

// Compute derivative of Bessel function of first kind.
extern complex<double> DBesselJ(
    complex<double> order, // Order of Bessel function
    complex<double> argument, // Argument of Bessel function
    int KODE, // Scaling flag. =1 (without scaling), =2 (with scaling)
    int *NZ, // Underflow indicator. =0 (Normal return), =1(underflow)
    int *IERR // Output error code
    );

// Compute derivative of Bessel function of second kind.
extern complex<double> DBesselY(
    complex<double> order, // Order of Bessel function
    complex<double> argument, // Argument of Bessel function
    int KODE, // Scaling flag. =1 (without scaling), =2 (with scaling)
    int *NZ, // Underflow indicator. =0 (Normal return), =1(underflow)
    int *IERR // Output error code
    );

// Compute Hankel function of first kind.
extern complex<double> HankelH1( 
    complex<double> order, // Order of Bessel function
    complex<double> argument, // Argument of Bessel function
    int KODE, // Scaling flag. =1 (without scaling), =2 (with scaling)
    int *NZ, // Underflow indicator. =0 (Normal return), =1(underflow)
    int *IERR // Output error code
    );

// Compute Hankel function of second kind.
extern complex<double> HankelH2( 
    complex<double> order, // Order of Bessel function
    complex<double> argument, // Argument of Bessel function
    int KODE, // Scaling flag. =1 (without scaling), =2 (with scaling)
    int *NZ, // Underflow indicator. =0 (Normal return), =1(underflow)
    int *IERR // Output error code
    );

// Compute derivative of Hankel function of first kind.
extern complex<double> DHankelH1( 
    complex<double> order, // Order of Bessel function
    complex<double> argument, // Argument of Bessel function
    int KODE, // Scaling flag. =1 (without scaling), =2 (with scaling)
    int *NZ, // Underflow indicator. =0 (Normal return), =1(underflow)
    int *IERR // Output error code
    );

// Compute derivative of Hankel function of second kind.
extern complex<double> DHankelH2( 
    complex<double> order, // Order of Bessel function
    complex<double> argument, // Argument of Bessel function
    int KODE, // Scaling flag. =1 (without scaling), =2 (with scaling)
    int *NZ, // Underflow indicator. =0 (Normal return), =1(underflow)
    int *IERR // Output error code
    );
