/*
 * Circurs - Circular optical microresonators simulation tools
 * Kirankumar R. Hiremath (k.r.hiremath@ieee.org)
 * University of Twente, Department of Applied Mathematics 
 * P.O. Box 217, 7500AE Enschede, The Netherlands
 * (2005)
 */

/*
 * bscoupler.c
 * Bend-straight waveguide coupler.
 */


#include "bscoupler.h"
#define EPS_MIN 1.0e-10

// error message
void BSCouplererror(char *s)
{
    cerr << "Error:BScoupler: " << s << "\n"; 
    exit(1);
};


//constructor
BSCoupler::BSCoupler(
    BSlabModeArray bma_,
    ModeArray  sma_,
    double g_,
    double x0_, double x1_, int Nx_,
    double z0_, double z1_, int Nz_)
{
    bma = bma_;
    sma = sma_;
    g = g_;

    inspect_flag = inspect();

    if (  inspect_flag == false )
    {
	cerr << "--*-- BSCoupler::BSCoupler() :"
	     << "Improper object creation. Exiting program. --*-- \n";
	exit(1);
   }

    bspol = bma(0).pol;

    x0  = x0_;
    x1 = x1_;
    Nx = Nx_;
    X.x0 = x0;
    X.x1 = x1;
    hx = ( X.x1 - X.x0)/((double)(Nx -1));

    z0  = z0_;
    z1 = z1_;
    Nz = Nz_;
    Z.x0 = z0;
    Z.x1 = z1;
    hz = ( Z.x1 - Z.x0)/((double)(Nz -1));

    set_amp_flag = false;

    // write geometry details to screen.
    fwrite("/dev/stdout", 2);

    //    T = Cmatrix(bma.num + sma.num, bma.num + sma.num );
    T.unit(bma.num + sma.num);
    Oi = T;
    Oo = T;
    S = T;

    Tz = Cmatrix(Nz, (bma.num + sma.num)*(bma.num + sma.num));
    Sz = Cmatrix(Nz, (bma.num + sma.num)*(bma.num + sma.num));


    amp = Cmatrix(Nz, bma.num + sma.num);


    
    // maximum radial distance on which we evaluate bend fields
    double r_max = sqrt( X.x1*X.x1 + max(abs(Z.x0), abs(Z.x1))*max(abs(Z.x0), abs(Z.x1)));

    // Interval on which we need discretised bend field
    Interval rd( X.x0-0.1, r_max);

    // make discretised bend field available 
    for (int i=0; i< bma.num; i++) bma(i).set_d_field(rd, Nx,0);

    cerr << "--*-- Leaving BSCoupler::BSCoupler(). --*--\n\n";

};

// copy constructor
BSCoupler::BSCoupler(BSCoupler &bsc_)
{
    // make sure that all data members are copied.
    cerr << "--*-- From copy constructor BSCoupler(). --*-- \n";
    bma = bsc_.bma;
    sma = bsc_.sma;
    bspol = bsc_.bspol;
    g = bsc_.g;
    inspect_flag = bsc_.inspect_flag;

    x0 = bsc_.x0;
    x1 = bsc_.x1;
    X  = bsc_.X ;
    Nx = bsc_.Nx;
    hx = bsc_.hx;

    z0 = bsc_.z0;
    z1 = bsc_.z1;
    Z  = bsc_.Z ;
    Nz = bsc_.Nz;
    hz = bsc_.hz;

    S = bsc_.S;
    Sz = bsc_.Sz;
    T = bsc_.T;
    Tz = bsc_.Tz;
    Oi = bsc_.Oi;
    Oo = bsc_.Oo;

    set_amp_flag = bsc_.set_amp_flag;
    IBSAmp = bsc_.IBSAmp;

    amp = bsc_.amp;
    p_amp = bsc_.p_amp;

    inspect_flag = bsc_.inspect_flag;

};

// assignment operator
BSCoupler& BSCoupler::operator=(const BSCoupler& bsc_)
{
    if (this != &bsc_)
    {
	bma = bsc_.bma;
	sma = bsc_.sma;
	bspol = bsc_.bspol;
	g = bsc_.g;
	inspect_flag = bsc_.inspect_flag;

	x0 = bsc_.x0;
	x1 = bsc_.x1;
	X  = bsc_.X ;
	Nx = bsc_.Nx;
	hx = bsc_.hx;
	
	z0 = bsc_.z0;
	z1 = bsc_.z1;
	Z  = bsc_.Z ;
	Nz = bsc_.Nz;
	hz = bsc_.hz;
	
	S = bsc_.S;
	Sz = bsc_.Sz;
	T = bsc_.T;
	Tz = bsc_.Tz;
	Oi = bsc_.Oi;
	Oo = bsc_.Oo;
	
	set_amp_flag = bsc_.set_amp_flag;
	amp = bsc_.amp;
	p_amp = bsc_.p_amp;

	IBSAmp = bsc_.IBSAmp;
	inspect_flag = bsc_.inspect_flag;
    }
    return *this;
};


// default constructor
BSCoupler::BSCoupler()
{ 
//    cerr << "calling default constructor \n";
}



// To compute overlap integral between two fields
// Integral is evaluated using Simpson's method.
// conjugation is on field_2
Complex BSCoupler::coverlap(
    Complex *field_1, // ptr to the row of field values at a certain z.
    Complex *field_2,
    Complex *field_1_mp,
    Complex *field_2_mp) const
{
/*    
    Complex sum(0.0, 0.0), summid(0.0,0.0);

    sum = ((*(field_2)).conj()) * (*(field_1)) * 0.5;
    
    for (int i_x=1; i_x < Nx - 1; i_x++)
	sum = sum + (*(field_2 + i_x)).conj() * (*(field_1 + i_x));

    sum = sum + (*(field_2 + Nx -1 )).conj() * (*(field_1 + Nx - 1)) * 0.5;

//    cerr << "Sum : " << sum.re << "\t" << sum.im << "\n";

    for (int i_x=0; i_x< Nx - 1; i_x++) 
	summid = summid + (*(field_2_mp + i_x)).conj() * (*(field_1_mp + i_x));

//    cerr << "Summid : " << summid.re << "\t" << summid.im << "\n";

    return  (sum + 2.0*summid)*hx/3.0;
*/


// trying Trapezoid method 
    
    Complex  sum = ((*(field_2)).conj()) * (*(field_1)) * 0.5;
    for (int i_x=1; i_x < Nx - 1; i_x++)
	sum = sum + (*(field_2 + i_x)).conj() * (*(field_1 + i_x));
    sum = sum + (*(field_2 + Nx -1 )).conj() * (*(field_1 + Nx - 1)) * 0.5;
    
    return sum*hx;


};

Complex BSCoupler::coverlap_eps(
    double *eps,
    double *eps_mp,
    Complex *field_1, // ptr to the row of field values at a certain z.
    Complex *field_2,
    Complex *field_1_mp,
    Complex *field_2_mp) const
{
/*
    Complex sum(0.0, 0.0), summid(0.0, 0.0);
    
    sum = (*(eps))*((*(field_2)).conj()) * (*(field_1)) * 0.5;

    for (int i_x=1; i_x < Nx - 1; i_x++)
	sum = sum + (*(eps + i_x))*((*(field_2 + i_x)).conj()) * (*(field_1 + i_x));

    sum = sum + (*(eps + Nx -1))*((*(field_2 + Nx - 1)).conj()) * (*(field_1 + Nx - 1)) * 0.5;

    for (int i_x=0; i_x< Nx - 1; i_x++)
	summid = summid + (*(eps_mp + i_x))*((*(field_2_mp + i_x)).conj() )* (*(field_1_mp + i_x));

    return  (sum + 2.0*summid)*hx/3.0;
*/
// trying Trapezoid method 
    
    Complex  sum = (*(eps))*((*(field_2)).conj()) * (*(field_1)) * 0.5;
    for (int i_x=1; i_x < Nx - 1; i_x++)
	sum = sum + (*(eps + i_x))*((*(field_2 + i_x)).conj()) * (*(field_1 + i_x));
    sum = sum + (*(eps + Nx -1))*((*(field_2 + Nx - 1)).conj()) * (*(field_1 + Nx - 1)) * 0.5;
    
    return sum*hx;

};

//------For TE case-----defining f(x,z) in RK4
Cmatrix BSCoupler::f_TE(
    double z_n,
    int message)  // 1 means write L and R matrices to file LR_TE.data
{
// populate the required field values at z= z_n

// field values at nodal points
    Complex bEY[bma.num][Nx], bHX[bma.num][Nx]; 
    Complex sEY[sma.num][Nx], sHX[sma.num][Nx];

// field values at mid points
    Complex bEY_mp[bma.num][Nx], bHX_mp[bma.num][Nx];
    Complex sEY_mp[sma.num][Nx], sHX_mp[sma.num][Nx];

    double Deps_b[Nx], Deps_s[Nx];
    double Deps_b_mp[Nx], Deps_s_mp[Nx];

    double x_n, x_n_mp;
    double eps_b, eps_s,  eps_b_mp, eps_s_mp; 


    for( int i_x=0; i_x< Nx; i_x++)
    {
		    
	x_n = X.x0 + i_x*hx;
	x_n_mp = x_n +  0.5*hx;

	for(int q=0;  q < bma.num;  q++)
	{
	    bEY[q][i_x] = bma(q).field_cart( EY, x_n, z_n);
	    bHX[q][i_x] = bma(q).field_cart( HX, x_n, z_n);

	    bEY_mp[q][i_x] = bma(q).field_cart( EY, x_n_mp, z_n);
	    bHX_mp[q][i_x] = bma(q).field_cart( HX, x_n_mp, z_n);
	}


	for(int p=0;  p < sma.num;  p++)
	{
	    sEY[p][i_x] = sma(p).cfield(EY, FORW, x_n, z_n);
	    sHX[p][i_x] = sma(p).cfield(HX, FORW, x_n, z_n);
	    
	    sEY_mp[p][i_x] = sma(p).cfield(EY, FORW, x_n_mp, z_n);
	    sHX_mp[p][i_x] = sma(p).cfield(HX, FORW, x_n_mp, z_n);
	}


	eps_b = bma(0).bend.eps(x_n,z_n);
	eps_s = sma(0).wg.eps(x_n);
	
	eps_b_mp = bma(0).bend.eps(x_n_mp,z_n);
	eps_s_mp = sma(0).wg.eps(x_n_mp);
	
	Deps_b[i_x] = (( eps_b >= eps_s ) ? 0.0 : (eps_s - eps_b)); 
	Deps_s[i_x] = (( eps_b >= eps_s ) ? (eps_b - eps_s) : 0.0); 

	Deps_b_mp[i_x] = (( eps_b_mp >= eps_s_mp ) ? 0.0 : (eps_s_mp - eps_b_mp)); 
	Deps_s_mp[i_x] = (( eps_b_mp >= eps_s_mp ) ? (eps_b_mp - eps_s_mp) : 0.0); 

    }

// defining matrix in CMEs

    Cmatrix L(bma.num + sma.num, bma.num + sma.num), 
	    R(bma.num + sma.num, bma.num + sma.num); 


// Direct implementation of L
    for (int i =0; i < (bma.num + sma.num); i++)
    {
	for (int j =0; j < (bma.num + sma.num); j++)
	{
	    if ( i < bma.num )
	    {
		if (j < bma.num)
		{
		    L(i,j) = -coverlap(&bEY[j][0], &bHX[i][0], 
				       &bEY_mp[j][0], &bHX_mp[i][0])
			      -coverlap(&bHX[j][0], &bEY[i][0], 
					&bHX_mp[j][0], &bEY_mp[i][0]);
		}
		else
		{
		    L(i,j) = -coverlap(&sEY[ j - bma.num ][0], &bHX[i][0], 
				       &sEY_mp[j - bma.num][0], &bHX_mp[i][0])
			      -coverlap(&sHX[j - bma.num][0], &bEY[i][0], 
					&sHX_mp[j - bma.num][0], &bEY_mp[i][0]);
		}
	    }
	    else
	    {
		if (j < bma.num)
		{
		    L(i,j) = -coverlap(&bEY[j][0], &sHX[i - bma.num ][0], 
				       &bEY_mp[j][0], &sHX_mp[i - bma.num][0])
			      -coverlap(&bHX[j][0], &sEY[i - bma.num][0], 
					&bHX_mp[j][0], &sEY_mp[i - bma.num][0]);
		}
		else
		{
		    L(i,j) = -coverlap(&sEY[j - bma.num][0], &sHX[i - bma.num][0], 
				       &sEY_mp[j - bma.num][0], &sHX_mp[i - bma.num][0])
			      -coverlap(&sHX[j - bma.num][0], &sEY[i - bma.num][0], 
					&sHX_mp[j - bma.num][0], &sEY_mp[i - bma.num][0]);
		}

	    }
	}
    }


// direct implementation of R

  for (int i =0; i < (bma.num + sma.num); i++)
  {
      for (int j =0; j < (bma.num + sma.num); j++)
      {
	  if ( i < bma.num)
	  {
	      if ( j < bma.num)
	      {
		  R(i,j) = coverlap_eps(&Deps_b[0], &Deps_b_mp[0],
				        &bEY[j][0], &bEY[i][0],
				        &bEY_mp[j][0], &bEY_mp[i][0]);
	      }
	      else
	      {
		  R(i,j) = coverlap_eps(&Deps_s[0], &Deps_s_mp[0],
				        &sEY[j - bma.num ][0], &bEY[i][0],
				        &sEY_mp[j - bma.num][0], &bEY_mp[i][0]);
	      }
	  }
	  else
	  {
	      if ( j < bma.num)
	      {
		  R(i,j) = coverlap_eps(&Deps_b[0], &Deps_b_mp[0],
					&bEY[j][0], &sEY[ i - bma.num][0],
				        &bEY_mp[j][0], &sEY_mp[i - bma.num][0]);
	      }
	      else
	      {
		  R(i,j) = coverlap_eps(&Deps_s[0], &Deps_s_mp[0],
					&sEY[j - bma.num][0], &sEY[i - bma.num][0],
				        &sEY_mp[j - bma.num][0], &sEY_mp[i - bma.num][0]);
	      }
	  }
      } 
  }

    double my_omega = (2.0*PI*SOLC0/bma(0).bend.wg.lambda);
    
    R = mult(R, -1.0*CCI*my_omega*EP0*1e-12); // multuply 1e-12 for EP0


    if (message == 1)
    {// write matrics L and R to file

	ofstream outfile("LR_TE.data", ios::out | ios::app);
	outfile.setf(ios::scientific, ios::floatfield);
	outfile.setf(ios::showpoint);

	outfile << z_n;

	for( int i = 0; i < (bma.num + sma.num); i++)
	    for( int j = 0; j < (bma.num + sma.num); j++)
		outfile << "\t" << L(i,j).abs();
//		outfile << "\t" << L(i,j).re << "\t" << L(i,j).im;
	
	for( int i = 0; i < (bma.num + sma.num); i++)
	    for( int j = 0; j < (bma.num + sma.num); j++)
		outfile << "\t" << R(i,j).abs();
//		outfile << "\t" << R(i,j).re << "\t" << R(i,j).im;
	outfile << "\n";

	outfile.close();
	

    }

    L.inverse();

    return  mult (L, R);
};

//------------------------------------------------------


//-------for TM case -----defining f(x,z) in RK4
Cmatrix BSCoupler::f_TM( double z_n, int message)  // 1 means write output to file
{
// populate the required field values at z= z_n

// field values at nodal points
    Complex bEX[bma.num][Nx], bEZ[bma.num][Nx], bHY[bma.num][Nx]; 
    Complex sEX[sma.num][Nx], sEZ[bma.num][Nx], sHY[sma.num][Nx];

// field values at mid points
    Complex bEX_mp[bma.num][Nx],bEZ_mp[bma.num][Nx], bHY_mp[bma.num][Nx];
    Complex sEX_mp[sma.num][Nx],sEZ_mp[bma.num][Nx], sHY_mp[sma.num][Nx];

    double Deps_b[Nx], Deps_s[Nx];
    double Deps_b_mp[Nx], Deps_s_mp[Nx];

    double x_n, x_n_mp;
    double eps_b, eps_s,  eps_b_mp, eps_s_mp; 


    for( int i_x=0; i_x< Nx; i_x++)
    {
		    
	x_n = X.x0 + i_x*hx;
	x_n_mp = x_n + 0.5*hx;

	for(int q=0;  q < bma.num;  q++)
	{
	    bEX[q][i_x] = bma(q).field_cart( EX, x_n, z_n);
	    bEZ[q][i_x] = bma(q).field_cart( EZ, x_n, z_n);
	    bHY[q][i_x] = bma(q).field_cart( HY, x_n, z_n);

	    bEX_mp[q][i_x] = bma(q).field_cart( EX, x_n_mp, z_n);
	    bEZ_mp[q][i_x] = bma(q).field_cart( EZ, x_n_mp, z_n);
	    bHY_mp[q][i_x] = bma(q).field_cart( HY, x_n_mp, z_n);
	}


	for(int p=0;  p < sma.num;  p++)
	{
	    sEX[p][i_x] = sma(p).cfield(EX, FORW, x_n, z_n);
	    sEZ[p][i_x] = sma(p).cfield(EZ, FORW, x_n, z_n);
	    sHY[p][i_x] = sma(p).cfield(HY, FORW, x_n, z_n);
	    
	    sEX_mp[p][i_x] = sma(p).cfield(EX, FORW, x_n_mp, z_n);
	    sEZ_mp[p][i_x] = sma(p).cfield(EZ, FORW, x_n_mp, z_n);
	    sHY_mp[p][i_x] = sma(p).cfield(HY, FORW, x_n_mp, z_n);
	}


	eps_b = bma(0).bend.eps(x_n,z_n);
	eps_s = sma(0).wg.eps(x_n);
	
	eps_b_mp = bma(0).bend.eps(x_n_mp,z_n);
	eps_s_mp = sma(0).wg.eps(x_n_mp);
	
	Deps_b[i_x] = (( eps_b >= eps_s ) ? 0.0 : (eps_s - eps_b)); 
	Deps_s[i_x] = (( eps_b >= eps_s ) ? (eps_b - eps_s) : 0.0); 

	Deps_b_mp[i_x] = (( eps_b_mp >= eps_s_mp ) ? 0.0 : (eps_s_mp - eps_b_mp)); 
	Deps_s_mp[i_x] = (( eps_b_mp >= eps_s_mp ) ? (eps_b_mp - eps_s_mp) : 0.0); 

    }

// defining matrix in CMEs

    Cmatrix L(bma.num + sma.num, bma.num + sma.num), 
	    R(bma.num + sma.num, bma.num + sma.num); 


// Direct implementation of L
    for (int i =0; i < (bma.num + sma.num); i++)
	for (int j =0; j < (bma.num + sma.num); j++)
	{
	    if ( i < bma.num )
	    {
		if (j < bma.num)
		{
		    L(i,j) = coverlap(&bHY[j][0], &bEX[i][0], 
				      &bHY_mp[j][0], &bEX_mp[i][0])
			   + coverlap(&bEX[j][0], &bHY[i][0], 
				      &bEX_mp[j][0], &bHY_mp[i][0]);
		}
		else
		{
		    L(i,j) = coverlap(&sHY[bma.num - j][0], &bEX[i][0], 
				      &sHY_mp[j - bma.num][0], &bEX_mp[i][0])
			    +coverlap(&sEX[j - bma.num][0], &bHY[i][0], 
				      &sEX_mp[j - bma.num][0], &bHY_mp[i][0]);
		}
	    }
	    else
	    {
		if (j < bma.num)
		{
		    L(i,j) = coverlap(&bHY[j][0], &sEX[i - bma.num][0], 
				      &bHY_mp[j][0], &sEX_mp[i - bma.num][0])
			    +coverlap(&bEX[j][0], &sHY[i - bma.num][0], 
				      &bEX_mp[j][0], &sHY_mp[i - bma.num][0]);
		}
		else
		{
		    L(i,j) = coverlap(&sHY[j - bma.num][0], &sEX[i - bma.num][0],
				      &sHY_mp[j - bma.num][0], &sEX_mp[i - bma.num][0])
			     +coverlap(&sEX[j - bma.num][0], &sHY[i - bma.num][0], 
				       &sEX_mp[j - bma.num][0], &sHY_mp[i - bma.num][0]);
		}

	    }
	}


// direct implementation of R

  for (int i =0; i < (bma.num + sma.num); i++)
  {
      for (int j =0; j < (bma.num + sma.num); j++)
      {
	  if ( i < bma.num)
	  {
	      if ( j < bma.num)
	      {
		  R(i,j) = coverlap_eps(&Deps_b[0], &Deps_b_mp[0],
				        &bEX[j][0], &bEX[i][0],
				        &bEX_mp[j][0], &bEX_mp[i][0]) + 
		           coverlap_eps(&Deps_b[0], &Deps_b_mp[0],
				        &bEZ[j][0], &bEZ[i][0],
				        &bEZ_mp[j][0], &bEZ_mp[i][0]);
	      }
	      else
	      {
		  R(i,j) = coverlap_eps(&Deps_s[0], &Deps_s_mp[0],
				        &sEX[j - bma.num ][0], &bEX[i][0],
				        &sEX_mp[j - bma.num][0], &bEX_mp[i][0])+
		           coverlap_eps(&Deps_s[0], &Deps_s_mp[0],
					&sEZ[j - bma.num ][0], &bEZ[i][0],
				        &sEZ_mp[j - bma.num][0], &bEZ_mp[i][0]);
	      }
	  }
	  else
	  {
	      if ( j < bma.num)
	      {
		  R(i,j) = coverlap_eps(&Deps_b[0], &Deps_b_mp[0],
					&bEX[j][0], &sEX[ i - bma.num][0],
				        &bEX_mp[j][0], &sEX_mp[i - bma.num][0])+
		           coverlap_eps(&Deps_b[0], &Deps_b_mp[0],
					&bEZ[j][0], &sEZ[ i - bma.num][0],
				        &bEZ_mp[j][0], &sEZ_mp[i - bma.num][0]);
	      }
	      else
	      {
		  R(i,j) = coverlap_eps(&Deps_s[0], &Deps_s_mp[0],
					&sEX[j - bma.num][0], &sEX[i - bma.num][0],
				        &sEX_mp[j - bma.num][0], &sEX_mp[i - bma.num][0])+
		           coverlap_eps(&Deps_s[0], &Deps_s_mp[0],
					&sEZ[j - bma.num][0], &sEZ[i - bma.num][0],
				        &sEZ_mp[j - bma.num][0], &sEZ_mp[i - bma.num][0]);
	      }
	  }
      } 
  }

    double my_omega = (2.0*PI*SOLC0/bma(0).bend.wg.lambda);
    
    R = mult(R, -1.0*CCI*my_omega*EP0*1e-12); // multuply 1e-12 for EP0


    if (message == 1)
    {// write matrics L and R to file

	ofstream outfile("LR_TM.data", ios::out | ios::app);
	outfile.setf(ios::scientific, ios::floatfield);
	outfile.setf(ios::showpoint);

	outfile << z_n;

	for( int i = 0; i < (bma.num + sma.num); i++)
	    for( int j = 0; j < (bma.num + sma.num); j++)
		outfile << "\t" << L(i,j).abs();
//		outfile << "\t" << L(i,j).re << "\t" << L(i,j).im;
	
	for( int i = 0; i < (bma.num + sma.num); i++)
	    for( int j = 0; j < (bma.num + sma.num); j++)
		outfile << "\t" << R(i,j).abs();
//		outfile << "\t" << R(i,j).re << "\t" << R(i,j).im;
	outfile << "\n";

	outfile.close();
	

    }

    L.inverse();

    return  mult (L, R);
};

// solves transfer matrix
// initial_Amp : first entries are for bend modes and then for SWG modes.
// info = 1 means write coupled amplitudes evalution "coupled_amp.data".
// info = 2 means Tz matrix data is written to file Tmatrix.data
//                Sz matrix data is written to file Smatrix.data
Cmatrix BSCoupler::get_transfer_matrix(Cvector initial_Amp, int info)
{

// Check that initial_Amp has (bma.num + sma.num) row entries
    if ( initial_Amp.nel != (bma.num + sma.num) )
    {
	cerr <<"--*-- From BSCoupler::get_transfer_matrix() -->"
	     <<"Error:No. of rows of intial_Amp is not compatiable with"
	     <<"no. of required rows.\n"
	     <<"No. of rows of intial_Amp= " << initial_Amp.nel << "\n"
	     <<"No. of required rows (no. of BWG modes + no. of SWG modes) ="
	     << (bma.num + sma.num) << "--*-- \n";
	exit(1);
    }

// initialised IBSAmp. This is used in the field evaluation.
    IBSAmp = initial_Amp;
    cerr << "--*-- From BSCoupler::get_transfer_matrix(): "
	 << "IBSAmp is initialised.--*-- \n";

    // Using function pointer approch, depending upon the polarization 
    // type either f_TE() or f_TM() is called.

    //define a pointer to function
    Cmatrix (BSCoupler::*fptr)(double, int);

    //assign it depending upon polarization type
    

    if ( info == 99 )
    {
	// do nothing.

    }
    else
    {
	cerr << "--*-- Conventional CMT --*--\t";
	if ( bspol == TE )
	{
	    fptr = &BSCoupler::f_TE;
	    cerr << " f_TE() is called.--*--\n";
	}
	else if (bspol == TM )
	{
	    fptr = &BSCoupler::f_TM;
	    cerr << " f_TM() is called.--*--\n";
	}
    }

    Cmatrix I;
    Cmatrix k1(bma.num + sma.num, bma.num + sma.num), 
	    k2(bma.num + sma.num, bma.num + sma.num), 
	    k3(bma.num + sma.num, bma.num + sma.num), 
	    k4(bma.num + sma.num, bma.num + sma.num), 
	    k_tmp(bma.num + sma.num, bma.num + sma.num); 	

    I.unit(bma.num + sma.num);  //    I is a unit matrix

    Cvector amp_n(bma.num + sma.num); // amp. at n'th level

    double z_n;

// store initial values of the amplitude in 'amp' data member
// initially amp at n'th level = amp at zero'th level

    cerr << 0 << "\t" << Z.x0;

    for(int i=0; i < bma.num + sma.num; i++) 
    {
	amp(0,i) = initial_Amp(i);
	amp_n(i) = initial_Amp(i);
	cerr << "\t" << amp(0,i).abs();

    }
    

    cerr <<"\n";

// fill the Sz, Tz matrix at firat level
    int ic = 0;

    for( int rc=0; rc < bma.num + sma.num; rc++)
    {
	for( int cc=0; cc < bma.num + sma.num; cc++)
	{
	    Tz(0, ic) = (rc == cc) ? 1.0 : 0.0;
	    Sz(0, ic) = (rc == cc) ? 1.0 : 0.0;
	    ic++;
	}
    }

// solving CMT for next z-step
    for(int n = 1; n < Nz; n++)
    {
//  knowing amplitudes at n'th step and want to find out ampl at (n+1) step
	z_n = Z.x0 + ((double)(n -1))*hz;

	if ( n == 1)
	{
         // set last argument = 1 for CM matrix analysis
	    k1 = (*this.*fptr)(z_n, 0);

	    k1 = mult(k1, hz);
	}
	else
	    k1 = k_tmp;

	k2 = (*this.*fptr)(z_n + hz/2.0, 0);
	k2 = mult(k2, hz);
	k3 = k2; //To save computation time.
	k2 = mult(k2, add(I, mult(k1, 0.5)));

	

	k3 = mult(k3, add(I, mult(k2, 0.5)));
	
	// set last argument = 1 for CM matrix analysis
	k4 = (*this.*fptr)(z_n + hz, 0);

	k4 = mult(k4, hz);
	k_tmp = k4 ; // To save computation time. k_tmp is k1 in next step.
	k4 = mult(k4, add(I, k3));

	k1 = mult(k1, 1.0/6.0);
	k2 = mult(k2, 1.0/3.0);
	k3 = mult(k3, 1.0/3.0);
	k4 = mult(k4, 1.0/6.0);

	Cmatrix T_n(bma.num + sma.num, bma.num + sma.num);

	T_n = add(T_n, k1);
	T_n = add(T_n, k2);
	T_n = add(T_n, k3);
	T_n = add(T_n, k4);
	T_n = add(T_n, I);

	T = mult(T_n, T);

// fill the Tz, Sz matrix

	Cmatrix Stmp( bma.num + sma.num, bma.num + sma.num); 


// to get Stmp ---------Start

	correction_matrix(Z.x0, Oi);
	correction_matrix(z_n + hz, Oo);

	Oi.inverse();
	Stmp = mult(T, Oi);
	Stmp = mult(Oo, Stmp);

	
	ic = 0;
	
 	for( int rc=0; rc < bma.num + sma.num; rc++)
	{
	    for( int cc=0; cc < bma.num + sma.num; cc++)
	     {
		 Tz(n, ic) = T(rc, cc);
		 Sz(n, ic) = Stmp(rc, cc);
		 ic++;
	     }
	 }


// amplitudes at (n+1)'th level
	amp_n = mult(T, initial_Amp);

// fill amp values at (n) level to data member amp
	for(int j = 0; j < bma.num + sma.num; j++) amp(n, j) = amp_n(j);

//display amplitudes to screen
	cerr << n << "\t" << (Z.x0 + (n)*hz) <<"\t";

	for (int j = 0; j < bma.num + sma.num; j++) 
	    cerr  << amp(n, j).abs() << "\t";
	
	cerr <<"\n";
    }
    
//now amplitudes are set :-)
    set_amp_flag = true;

    if (info == 1)
    { // write amplitude data to file 'coupled_amp.data'
	ofstream outfile("coupled_amp.data", ios::out);
	outfile.setf(ios::scientific, ios::floatfield);
	outfile.setf(ios::showpoint);

	for(int i=0; i < Nz; i++)
	{
	    outfile << Z.x0 + i*hz;
	    for(int n=0; n < (bma.num + sma.num); n++)
		outfile << "\t" << amp(i,n).re << "\t" << amp(i,n).im;
	    outfile <<"\n";
	}
	
	outfile.close();
	cerr << "--*-- Coupled amplitude data is written to file coupled_amp.data.\n --*--";
    }

    if (info == 2)
    { 
      // write Tz to file 'Tmatrix.data'
	ofstream Tfile("Tmatrix.data", ios::out);
	Tfile.setf(ios::scientific, ios::floatfield);
	Tfile.setf(ios::showpoint);

      // write Sz to file 'Smatrix.data'
	ofstream Sfile("Smatrix.data", ios::out);
	Sfile.setf(ios::scientific, ios::floatfield);
	Sfile.setf(ios::showpoint);



	for(int rc=0; rc < Nz; rc++)
	{
	    Tfile << Z.x0 + rc*hz << "\t";
	    Sfile << Z.x0 + rc*hz << "\t";

	    for (int cc=0; cc < (bma.num + sma.num)*(bma.num + sma.num); cc++)
	    {
		Tfile << Tz(rc, cc).abs() << "\t";
		Sfile << Sz(rc, cc).abs() << "\t";
	    }
	    
	    Tfile <<"\n";
	    Sfile <<"\n";
	}
	
	Tfile.close();
	Sfile.close();

	cerr << "--*-- T matrix data is written to file Tmatrix.data. --*--\n";	
	cerr << "--*-- S matrix data is written to file Smatrix.data. --*--\n";


    }

    cerr << "---------------------------------------------------------\n"
	 << "--*-- From BSCoupler::get_transfer_matrix(): Displaying abs(T).--*--\n";
    
    for(int i = 0; i < (bma.num + sma.num); i++)
    {
	for(int j = 0; j < (bma.num + sma.num); j++)
	{
	    cerr << "( " << T(i,j).abs() << " )\t";
	}
	cerr << "\n";
    }	    
    
    cerr << "--------------------------------------------------------\n";

    return T;
};



// gets scattering matrix
// info == 1 display S matrix on screen.

Cmatrix BSCoupler::get_scattering_matrix(int info)
{
// get (projection) corrections for transfter matrix to get scattering matix
    correction_matrix(Z.x0, Oi);
    correction_matrix(Z.x1, Oo);

// with projection corrections
    Oi.inverse();
    S = mult(T, Oi);
    S = mult(Oo, S);


    if (info == 1 )
    {
	cerr << "---------------------------------------------------------\n"
	     << "From BSCoupler::get_scattering_matrix(): Displaying complex valued S.\n\n";

	for(int i = 0; i < (bma.num + sma.num); i++)
	{
	    
	    for(int j = 0; j < (bma.num + sma.num); j++)
	    {
		cerr << "( " << S(i,j).re << ", " << S(i,j).im << " )\t";
	    }
	    cerr << "\n";
	}
	cerr << "--------------------------------------------------------\n";

    }	    
    else if ( info == 2) // show on screen abs values of S matrix elements
    {
	cerr << "---------------------------------------------------------\n"
	     << "From BSCoupler::get_scattering_matrix(): "
	     << "Displaying abs(S).\n\n";
	
	    for(int i = 0; i < (bma.num + sma.num); i++)
	    {
		for(int j = 0; j < (bma.num + sma.num); j++) 
		    cerr << S(i,j).abs() << " \t";
		cerr << "\n";
	    }
    	    
	    cerr << "--------------------------------------------------------\n";
	    
    }

    return S;
}


// coupled field evaluation at point(x,z)
Complex BSCoupler::field(Fcomp cp, double x, double z)const
{
    // to evaluate field, first check coupled mode amplitudes are set.

    Complex ans;
    
    // find out the position of input z.

    double i_z = (z - Z.x0)/hz;
    
    int nc = (int)(ceil(i_z));
    int nf = (int)(floor(i_z));

    if (nc == nf)
    { // means input z is a nodal value

	for (int i=0; i< bma.num; i++)
	    ans = ans + amp((int)(i_z), i)*bma(i).field_cart(cp, x,z);
	for (int i= 0; i <  sma.num; i++)
	    ans = ans + amp( (int)(i_z), i + bma.num)*sma(i).cfield(cp, FORW, x,z);
    }
    else
    { // approximate the amplitudes by linear interpolation

	double zf, zc; //  input z lies between zf < z < zc
	zf = Z.x0 + nf*hz;
	zc = Z.x0 + nc*hz;

	Complex tAmp;

	for (int i=0; i< bma.num; i++)
	{
	    tAmp = (z - zf) * amp(nc, i) / ( zc - zf) - 
		   (z - zc) * amp(nf, i) / ( zc - zf);
	    ans = ans + tAmp * bma(i).field_cart(cp, x,z);
	}

	for (int i= 0; i < sma.num; i++)
	{
	    tAmp = (z - zf) * amp(nc, i + bma.num) / ( zc - zf) - 
		   (z - zc) * amp(nf, i+ bma.num) / ( zc - zf);
	    ans = ans + tAmp * sma(i).cfield(cp, FORW,x,z);
	}


    }
	

    return ans;
};

//------------------------------------------------------------------------
// coupled field evaluation at point(-x,-z), x>0, z> 0
Complex BSCoupler::mirror_field(Fcomp cp, double x, double z)const
{

    Complex ans;
    
    // find out the position of input z.

    double i_z = (z - Z.x0)/hz;
    
    int nc = (int)(ceil(i_z));
    int nf = (int)(floor(i_z));

    if (nc == nf)
    { // means input z is a nodal value

	for (int i=0; i< bma.num; i++)
	    ans = ans - amp((int)(i_z), i)*bma(i).field_cart(cp, x,z);
	for (int i= 0; i <  sma.num; i++)
	    ans = ans + amp( (int)(i_z), i + bma.num)*sma(i).cfield(cp, FORW, x,z);
    }
    else
    { // approximate the amplitudes by linear interpolation

	double zf, zc; //  input z lies between zf < z < zc
	zf = Z.x0 + nf*hz;
	zc = Z.x0 + nc*hz;

	Complex tAmp;

	for (int i=0; i< bma.num; i++)
	{
	    tAmp = (z - zf) * amp(nc, i) / ( zc - zf) - 
		   (z - zc) * amp(nf, i) / ( zc - zf);
	    ans = ans - tAmp * bma(i).field_cart(cp, x,z);
	}

	for (int i= 0; i < sma.num; i++)
	{
	    tAmp = (z - zf) * amp(nc, i + bma.num) / ( zc - zf) - 
		   (z - zc) * amp(nf, i+ bma.num) / ( zc - zf);
	    ans = ans + tAmp * sma(i).cfield(cp, FORW,x,z);
	}


    }
	

    return ans;
};



//---------------------------------------------------------------------

// to check various consistant conditions and other constratints
bool BSCoupler::inspect(void)
{

// checking the wavelength and polarization of SWG modes

    for( int i=1; i < sma.num ; i++)
    {
	if ( sma(0).wg.lambda != sma(i).wg.lambda )
	{
	    cerr << "--*-- BSCoupler::inspect() :"
		 << " All straight waveguide modes are not of same wavelength.\n";
	    return false;
	};

	if ( sma(0).pol != sma(i).pol )
	{
	    cerr << "--*-- BSCoupler::inspect() :" 
		 << "All straight waveguide modes are not of same polarisation.\n";
	    return false;
	};

    }

// If control reached to this point means, all SWG modes have
// same wavelength and polarization. 
// Now check if the wavelength and polarization of bend modes
// is same as above wavelength and polarization

    for( int i=0; i < bma.num ; i++)
    {
	if ( sma(0).wg.lambda != bma(i).bend.wg.lambda )
	{
	    cerr << "--*-- BSCoupler::inspect() :"
		 << " SWG modes and BWG modes are not wavelength compatible.\n";
	    return false;
	};

	if ( sma(0).pol != bma(i).pol )
	{
	    cerr << "--*-- BSCoupler::inspect() :"
		 << " SWG modes and BWG modes are not polarization compatible.\n";
	    return false;
	};
	
    }
    
    return true;
    
};

//-------------------------------------------------------------------------

void BSCoupler::fwrite(char *filename, int spec)
{
    // creating proper files
    ofstream outfile(filename, ios::out | ios::app);
    outfile.setf(ios::scientific, ios::floatfield);
    outfile.setf(ios::showpoint);

    switch (spec) {
	case 1 :
	{ // write seperation distance (g) and S
	    outfile << g << "\t";
	    for(int p = 0; p < bma.num + sma.num; p++)
	    {
		for(int q = 0; q < bma.num + sma.num; q++)
		{
		    outfile << S(p,q).re << "\t" << S(p,q).im << "\t";
		}
		outfile <<"\n";
		break;
	    }
	}
	case 2 :
	{ // write geometry details
	    outfile << "# x0=" <<x0 << "\n"
		    << "# x1=" <<x1 << "\n"
		    << "# Nx=" <<Nx << "\n"
		    << "# hx=" <<hx << "\n"
		    << "# z0=" <<z0 << "\n"
		    << "# z1=" <<z1 << "\n"
		    << "# Nz=" <<Nz << "\n"
		    << "# hz=" <<hz << "\n\n";
	    break;
	}
    }
    outfile.close();

}



//-------------------------------------------------------
/* field profile -> MATLAB .m file  
   cp: EX - HZ, SZ
   foa: MOD, ORG, SQR
   fac: output factor (typically 1.0 or -1.0) 
   disp: output region on the x axis
   np: number of output points
   ext0, ext1: filename id characters
   pltype: 'L': geometry information + plot commands (default)
	   'V': field, mesh, and plot command, to be included into *L.m */

void BSCoupler::mfile(Fcomp cp, Afo foa, Interval disp, double z,
		      int np, char ext0, char ext1, char pltype)const
{

	char filename[14] = "BSt_______.m";
	filename[3] = polchr(bma(0).pol);
	switch(foa)
	{
		case MOD: filename[4] = 'm'; break;
		case ORG: filename[4] = 'f'; break;
		case SQR: filename[4] = 'q'; break;
		case REP: filename[4] = 'r'; break;
		case IMP: filename[4] = 'i'; break;
	}
	filename[5] = fldchr(cp);
	filename[6] = cpchr(cp);
	filename[7] = ext0;
	filename[8] = ext1;
	if(pltype != 'V') pltype = 'L';
	filename[9] = pltype;


	ofstream outfile(filename, ios::out);
	outfile.setf(ios::scientific, ios::floatfield);
	outfile.setf(ios::showpoint);

	outfile << "%Coupled field values at z=" << z
		<< "in the x-interval ["<< disp.x0 << ", "
		<< disp.x1 << "].\n\n";

    
	outfile << "\n\n field = [...\n";

	double dx = (disp.x1 - disp.x0)/(np -1); // discretisation step
	double x;
	Complex f;

	for (int i_x = 0; i_x < np; i_x++)
	{
	    x = disp.x0 + i_x * dx;
	    
	    f = field(cp, x, z);  

	    outfile << x << "\t" << f.re << "\t" << f.im << ";\n";
	}
    
	outfile <<"];\n\n";
//plotting command for coupled field

	switch (foa)
	{
	    case REP:
	    {
		outfile << "plot( field(:,1), field(:,2),"
			<< "\'LineWidth\', 2);\n\n"
			<< "xlabel(\' x position [\\mum]\') \n"
			<< "ylabel(\'Re( " << fldchr(cp) 
			<< cpchr(cp) << ")\')\n\n";
		break;
	    }
	    case IMP:
	    {
		outfile << "plot( field(:,1), field(:,3),"
			<< "\'LineWidth\', 2);\n\n"
			<< "xlabel(\' x position [\\mum]\') \n"
			<< "ylabel(\'Im( " << fldchr(cp) 
			    << cpchr(cp) << ")\')\n\n";
		break;
	    }
	    case MOD: case ORG:
	    {
		outfile << "plot( field(:,1), abs(field(:,2)"
			<< " + sqrt(-1) * field(:,3)),"
			<< "\'LineWidth\', 2);\n\n"
			<< "xlabel(\' x position [\\mum]\') \n"
			<< "ylabel(\'abs( " << fldchr(cp) 
			<< cpchr(cp) << ")\')\n\n";
		break;
	    }
	    case SQR:
	    {
		outfile << "plot( field(:,1), (field(:,2).* field(:,2)"
			<< " + field(:,3) .* field(:,3)),"
			<< "\'LineWidth\', 2);\n\n"
			<< "xlabel(\' x position [\\mum]\') \n"
			<< "ylabel(\'( " << fldchr(cp) 
			<< cpchr(cp) << ")^2\')\n\n";
		break;
	    }
	    
	}

    outfile.close();

    cerr << filename << " created.\n";
    return;

}

//----------------------------------------------------------------

    // complete field profile on rectangular domain[x0, x1]x[z0,z1]
    // -> MATLAB .m file  
    //	cp: EX - HZ, SZ
    //	foa: MOD, ORG, SQR  
    //	fac: output factor (typically 1.0 or -1.0)  
    //	Nx, Nz number of points in X and Z direction
    //	ext0, ext1: filename id characters
    //      pltype: 'L': geometry information + plot commands (default)
    //              'V': field, mesh, and plot comand, 
    //                   to be included into *L.m 
void BSCoupler::mfile(Fcomp cp, Afo foa, 
		      double x0_, double x1_, int Nx_, 
		      double z0_, double z1_, int Nz_,
		      char ext0, char ext1, char pltype)const
{

    char filename[15] = "CBSt_______.m";
	filename[4] = polchr(bma(0).pol);
	switch(foa)
	{
		case MOD: filename[5] = 'm'; break;
		case ORG: filename[5] = 'f'; break;
		case SQR: filename[5] = 'q'; break;
		case REP: filename[5] = 'r'; break;
		case IMP: filename[5] = 'i'; break;
	}
	filename[6] = fldchr(cp);
	filename[7] = cpchr(cp);
	filename[8] = ext0;
	filename[9] = ext1;
	if(pltype != 'V') pltype = 'L';
	filename[10] = pltype;


	ofstream outfile(filename, ios::out);
	outfile.setf(ios::scientific, ios::floatfield);
	outfile.setf(ios::showpoint);
	outfile.setf(ios::showpos);

	outfile << "%Coupled field values on [=" 
		<< x0_ <<", " << x1_ << "] with " << Nx_ << "points and"
		<< "on [" << z0_ <<", " << z1_ << "] with " << Nz_ << "points.\n";
    
	outfile << "\n\n field = [...\n";

	double dx_ = (x1_ - x0_)/(double)(Nx_ -1); // discretisation step in x
	double dz_ = (z1_ - z0_)/(double)(Nz_ -1); // discretisation step in z
	Complex f;

	for (int i_z = 0; i_z < Nz_; i_z++)
	{
	    for (int i_x = 0; i_x < Nx_; i_x++)
	    {
		f = field(cp, x0_ + i_x * dx_, z0_ + i_z * dz_);
		outfile	<< f.re << f.im << "*i\t";
	    }

	    cerr << "Done for iz=" << i_z << "\n";
	    outfile <<";\n";

	}

	outfile <<"];\n\n";

	outfile << "z= [ ";
	for (int i_z = 0; i_z < Nz_; i_z++) outfile << z0_ + i_z * dz_ << "\t";
	outfile <<"];\n\n";

	outfile << "x= [ ";
	for (int i_x = 0; i_x < Nx_; i_x++) outfile << x0_ + i_x * dx_ << "\t";
	outfile <<"];\n\n";

/*
//plotting command for coupled field
......
*/
	outfile.close();
	
	cerr << filename << " created.\n";
}

//==================================================================
// S is a scatterin matrix for z= [z0, z1]. reduced_S() reduces this 
// scattering matrix along line of symmetry z=0.
Cmatrix BSCoupler::reduced_S(void)const
{
    Cmatrix tCi(bma.num + sma.num, bma.num + sma.num);
    Cmatrix tDi(bma.num + sma.num, bma.num + sma.num);

// fill input coupler corrections
    for (int r=0; r < bma.num; r++) 
    {
	tCi(r,r) = exp(CCI*bma(r).gamma* bma(0).bend.R*(-theta(Z.x0))); 
	tDi(r,r) = exp(CCI*bma(r).gamma* bma(0).bend.R*(-theta(Z.x1)));
    }

    for (int r=bma.num; r < (bma.num + sma.num); r++) 
    {
	tCi(r,r) = exp(CCI*sma(r - bma.num).beta*(-Z.x0));
	tDi(r,r) = exp(CCI*sma(r - bma.num).beta*(-Z.x1));     
    }

// Get reduced Scattering matrices
    tDi.inverse();
    Cmatrix SR = mult(tDi, S);
    SR = mult(SR, tCi);

    return SR;
}

//=========================================================================
double BSCoupler::theta(double z)const
{

    if ( fabs(z) <= bma(0).bend.R)
	return (atan2( z, sqrt(bma(0).bend.R * bma(0).bend.R - z*z)));
    else
	return (atan2(sqrt(bma(0).bend.R * bma(0).bend.R - X.x0*X.x0), X.x0));  
}

//=======================================================================
// set `amp' corresponding to initial amplitudes given by iIBSAmp
void BSCoupler::set_amp(Cvector iIBSAmp)
{
    cerr << "--*-- BSCoupler::set_amp() --> Setting amplitudes at nodal"
	 << " points.--*-- \n";

    //check if IBSAmp is correct
    if (IBSAmp.nel != (bma.num + sma.num))
    {
	cerr <<"--*-- BSCoupler::set_amp() --> Input amplitudes are not correct."
	     << "\nRequired no. of amplitudes =" << (bma.num + sma.num)
	     <<"\tInput no. of amplitudes =" << IBSAmp.nel <<"--*-- \n";
	exit(1);
    }
    else
	IBSAmp = iIBSAmp;

    Cmatrix tT(bma.num + sma.num, bma.num + sma.num);
    Cvector tamp;

    // calculating new amps
    for(int iz=0; iz < Nz; iz++)
    {
	int ic = 0;
	
	for(int i=0; i < tT.r; i++)
	   for(int j=0; j < tT.c; j++)  tT(i,j) = Tz(iz, ic++);
	
	tamp = mult(tT, IBSAmp);

	amp.row(tamp, iz); // store new amplitude
    }

}
//----------------------------------------------------------------------------
// get amplitudes at given z position
Cvector BSCoupler::get_amp(double z)
{

    Cvector ans(bma.num + sma.num);
    
    // find out the position of input z.

    double i_z = (z - Z.x0)/hz;
    
    int nc = (int)(ceil(i_z));
    int nf = (int)(floor(i_z));

    if (nc == nf)
    { // means input z is a nodal value

	for (int i=0; i< bma.num + sma.num; i++)  ans(i) =  amp(nc, i);
    }
    else
    { // approximate the amplitudes by linear interpolation

	double zf, zc; //  input z lies between zf < z < zc
	zf = Z.x0 + nf*hz;
	zc = Z.x0 + nc*hz;


	for (int i=0; i< bma.num + sma.num; i++)
	{
	    ans(i) = (z - zf) * amp(nc, i) / ( zc - zf) - 
		   (z - zc) * amp(nf, i) / ( zc - zf);
	}
    }
	
    return ans;
}

//----------------------------------------------------------------------------
// Gives correction matrix at level `z'.
void BSCoupler::correction_matrix(double z, Cmatrix &O) 
{
    double theta_z = theta(z);
   
    double x_n, x_n_mp;
    Complex num, den;

    O.unit(bma.num + sma.num);

// for row <= bma.num, only non digonal entries are nonzero. Fill it.
	for(int q = 0; q < bma.num; q++)
	{
	    O(q,q) = exp(-CCI*bma(q).gamma*bma(q).bend.R* theta_z);
	}

// for row > bma.num, fill the matrix.	    
	for(int p = 0; p < sma.num; p++) // sort of row counter
	{
	    if (bspol == TE)
	    {
		Complex sEY[Nx], sHX[Nx];
		Complex sEY_mp[Nx], sHX_mp[Nx];
		Complex bEY[bma.num][Nx], bHX[bma.num][Nx];
		Complex bEY_mp[bma.num][Nx], bHX_mp[bma.num][Nx];

		// fill all bend fields only once
		for (int q = 0; q < bma.num; q++)
		{
		    for(int i_x = 0; i_x < Nx; i_x++)
		    {
			x_n = X.x0 + i_x*hx;
			x_n_mp = x_n  + 0.5*hx;
			
			bEY[q][i_x] = bma(q).field_cart( EY, x_n, z);
			bHX[q][i_x] = bma(q).field_cart( HX, x_n, z);
			
			bEY_mp[q][i_x] = bma(q).field_cart( EY, x_n_mp, z);
			bHX_mp[q][i_x] = bma(q).field_cart( HX, x_n_mp, z);	
		    }
		}
		for(int i_x = 0; i_x < Nx; i_x++)
		{
		    x_n = X.x0 + i_x*hx;
		    x_n_mp = x_n + 0.5*hx;
		
		    sEY[i_x] = sma(p).cfield(EY, FORW, x_n, z);
		    sHX[i_x] = sma(p).cfield(HX, FORW, x_n, z);
		    
		    sEY_mp[i_x] = sma(p).cfield(EY, FORW, x_n_mp, z);
		    sHX_mp[i_x] = sma(p).cfield(HX, FORW, x_n_mp, z);
		}
		
		den = -coverlap(&sEY[0], &sHX[0], &sEY_mp[0], &sHX_mp[0]);
		den = den + den.conj();
		
		// fill column entries from 0 to (bma.num -1)
		for(int q = 0; q < bma.num; q++) // sort of column counter
		{
		    num = -coverlap(&bEY[q][0], &sHX[0], &bEY_mp[q][0], &sHX_mp[0]);
		    num = num -coverlap(&bHX[q][0], &sEY[0], &bHX_mp[q][0], &sEY_mp[0]);
		    O( p + bma.num, q) = exp(-CCI*sma(p).beta*z)* num/den;
		}
	    }
	    else
	    { // pol = TM
		Complex sHY[Nx], sEX[Nx];
		Complex sHY_mp[Nx], sEX_mp[Nx];
		Complex bHY[bma.num][Nx], bEX[bma.num][Nx];
		Complex bHY_mp[bma.num][Nx], bEX_mp[bma.num][Nx];
		
		// fill all bend fields only once
		for (int q = 0; q < bma.num; q++)
		{
		    for(int i_x = 0; i_x < Nx; i_x++)
		    {
			x_n = X.x0 + i_x*hx;
			x_n_mp = x_n +  0.5*hx;
			
			bHY[q][i_x] = bma(q).field_cart( HY, x_n, z);
			bEX[q][i_x] = bma(q).field_cart( EX, x_n, z);
			
			bHY_mp[q][i_x] = bma(q).field_cart( HY, x_n_mp, z);
			bEX_mp[q][i_x] = bma(q).field_cart( EX, x_n_mp, z);	
		    }
		}

		for(int i_x = 0; i_x < Nx; i_x++)
		{
		    x_n = X.x0 + i_x*hx;
		    x_n_mp = x_n + 0.5*hx;
		
		    sHY[i_x] = sma(p).cfield(HY, FORW, x_n, z);
		    sEX[i_x] = sma(p).cfield(EX, FORW, x_n, z);
		    
		    sHY_mp[i_x] = sma(p).cfield(HY, FORW, x_n_mp, z);
		    sEX_mp[i_x] = sma(p).cfield(EX, FORW, x_n_mp, z);
		}
		
		den =  coverlap(&sHY[0], &sEX[0], &sHY_mp[0], &sEX_mp[0]);
		den = den + den.conj();
		
		// fill column entries from 0 to (bma.num -1)
		for(int q = 0; q < bma.num; q++) // sort of column counter
		{
		    num = coverlap(&bHY[q][0], &sEX[0], &bHY_mp[q][0], &sEX_mp[0]);
		    num = num + coverlap(&bEX[q][0], &sHY[0], &bEX_mp[q][0], &sHY_mp[0]);
		    O( p + bma.num, q) = exp(-CCI*sma(p).beta*z)* num/den;
		}

	    }
// fill column entries from (bma.num) to (bma.num + sma.num)	
// here only digonal entries are nonzero.	

	    O(p + bma.num,p + bma.num) = exp(-CCI*sma(p).beta*z);
	}

}

//-------------------------------------------------------------
/* visualization of the light propagation, output: MATLAB scripts */
	
/* image of component cp
   cp: one of E?, H?, S? ... ?
   foa: MOD, SQR, REP, IMP, ORG = REP 
   xbeg, xend, zbeg, zend: x resp. z extensions of the plot window   
   npx, npz: number of plot points in the x and z directions
   ext0, ext1: filename id characters for the m.file */
void BSCoupler::plot(Fcomp cp, Afo foa, 
              double xbeg, double xend, double zbeg, double zend,
              int npx, int npz, char ext0, char ext1) const
{

	FILE *dat;
	char name[13] = "bs_____I.m";
	name[2] = afochr(foa);
        name[3] = fldchr(cp);
        name[4] = cpchr(cp);      
	name[5] = ext0;
	name[6] = ext1;
	fprintf(stderr, "plot([%g, %g] x [%g, %g]) >> %s\n", 
	                xbeg, xend, zbeg, zend, name);
	dat = fopen(name, "w+");
	mlout_title(dat, name, "Circurs, BS coupler field");
	mlout_bscoupgeo(dat, bma(0).bend, sma(0).wg);
	mlout_meshxz(dat, xbeg, xend, npx, zbeg, zend, npz);
	Dmatrix fld = field(cp, foa, Interval(xbeg, xend), npx, 
	                             Interval(zbeg, zend), npz);
	mlout_fld(dat, npx, npz, cp, fld);
	name[8] = 0; 
	char desc[10];
	afocpdesc(foa, cp, desc);
	mlout_coupgenimage(cp, name, desc, dat); 
	if(foa == MOD || foa == SQR) mlout_lincolormap(dat);
	else                         mlout_magcolormap(dat);
	mlout_print(dat, name, 'e'); 
	fclose(dat);
	return;
}

//-----------------------------
/* the optical field, discretized on a regular rectangular mesh,  
   npx x npz points on a display window dwx x dwz
   cp: one of E?, H?, S? ... ?
   foa: ORG, MOD, SQR, REP, IMP (ORG = REP) */
Dmatrix BSCoupler::field(Fcomp cp, Afo foa, Interval dwx, int npx, 
                            Interval dwz, int npz) const
{
	Dmatrix fld(npx, npz);
	double dx = dwx.len()/((double)(npx-1));
	double dz = dwz.len()/((double)(npz-1));
	double x, z;

	z = dwz.x0;
	for(int j=0; j<=npz-1; ++j)
	{
		x = dwx.x0;
		for(int i=0; i<=npx-1; ++i) 
		{
			fld(i, j) = field(cp, foa, x, z);
			x += dx;
		}
		z += dz;
	}
	return fld;
}


//--------------------------------------------------------
/* 'realized' field values 
   cp:  ER, EY, EP, HR, HY, HP, SR, SY, SP 
   foa: MOD, ORG, REP, IMP, SQR */
double BSCoupler::field(Fcomp cp, Afo foa, double x, double z) const
{
    Complex fc;
    fc = field(cp, x, z);
    switch(foa)
    {
	case REP: 
	    return fc.re;
	case IMP: 
	    return fc.im;
	case MOD: case ORG:
	    return fc.abs();
	case SQR:
	    return fc.sqabs();
    }
    return fc.re;
}
