/*
 * Circurs - Circular optical microresonators simulation tools
 * Kirankumar R. Hiremath (k.r.hiremath@ieee.org)
 * University of Twente, Department of Applied Mathematics 
 * P.O. Box 217, 7500AE Enschede, The Netherlands
 * (2005)
 */

/*
 * bscoupler.h
 * Bend-straight waveguide coupler.
 */

#ifndef BSCOUPLER_H
#define BSCOUPLER_H

#include "complex"
#include "bmsinc.h"
#include "metric.h"
#include <iostream>
#include <fstream>
#include <algorithm>

using namespace std;

class BSCoupler
{
 public:
    // the bend waveguide mode array
    BSlabModeArray bma;
    
    // the straight waveguide mode array
    ModeArray sma;

    // Polarization 
    Polarization bspol;

    // minimum distance of seperation
    double g;

    // interval along x axis [x0, x1]
    double x0, x1;

    // interval along z axis [z0, z1]
    double z0, z1;

    // no. of discertization points along x direction
    int Nx;
    
    // no. of discertization points along z direction
    int Nz;
    
    // scattering matrix for [z0, z1]
    Cmatrix S;

    // initial amplitude for coupler T matrix computation
    // it is initialised in get_transfer_matrix().
    Cvector IBSAmp; 

    // Corrections matrices: 
    // Oi = correction at input, Oo = correction at output
    // Using these corrections, S = Oo * T * inv(Oi)
    // These matrices also required for field of complete MR.
    Cmatrix Oi, Oo; 


        // X = [x0, x1]
    Interval X;
    
    // Z= [z0, z1]
    Interval Z;

    // discretization step in x direction
    double hx;
 
    // discretization step in z direction
    double hz;

    // chcecks whether Cmatrix amp is filled or not
    bool set_amp_flag;

    // It stores amplitudes at z levels. 
    // It is (bsa.num + sa.num) row and (Nz) cols complex number matrix. 
    // each row corresponds to one mode and each col. corresponds to one 
    // z point. The memory is alloted by the constructor.
    Cmatrix amp;

    // projected amplitude - detail same as variable 'amp'. 
    // p_amp(z) := Oo(z) * amp(z)
    Cmatrix p_amp;
    // final propagation matrix elements
    Cmatrix T;

    // collection of propagation matrix at each z
    Cmatrix Tz;

    // collection of scattering matrix at each z
    Cmatrix Sz;


    // To check whether all modes have same wavelength and polarization
    bool inspect_flag;

    // default constructor
    BSCoupler();

    //constructor
    BSCoupler(BSlabModeArray bma_,
	      ModeArray  sma_,
	      double g_,
	      double x0_, double x1_, int Nx_,
	      double z0_, double z1_, int Nz_);
    
    // copy constructor 
    BSCoupler( BSCoupler &bsc_);

    // assignment for BSCoupler
    BSCoupler& operator=(const BSCoupler& bsc_);

    // field_2 is conjugated
    Complex coverlap(
	Complex *field_1, // ptr to the row of field values at a certain z.
	Complex *field_2, 
	Complex *field_1_mp,
	Complex *field_2_mp
	) const;

    // field_2 is conjugated
    Complex coverlap_eps(
	double *eps,
	double *eps_mp,
	Complex *field_1, // ptr to the row of field values at a certain z.
	Complex *field_2,
	Complex *field_1_mp,
	Complex *field_2_mp
	) const;

    // -----defining f(x,z) in RK41 ( d_z(C(z)) = f(x,z). C(z) )
    // message = 1 => write L and R matrices to file LR_TE.data
    Cmatrix f_TE(double z, int message);  // for TE case 
    Cmatrix f_TM(double z, int message ); // for TM case
    
    // solves transfer matrix
    // initial_Amp : first entries are for bend modes and then for SWG modes.
    // info = 1 means write coupled amplitudes evalution "coupled_amp.data".
    // info = 2 means Tz matrix data is written to file Tmatrix.data
    //                Sz matrix data is written to file Smatrix.data
    Cmatrix get_transfer_matrix(Cvector initial_Amp, int info); 

    // gets scattering matrix
    // info = 0 => don't display scattering matrix
    // info = 1 => display complex S matrix
    // info = 2 => display absolute value of S
    Cmatrix get_scattering_matrix(int info);

    // set `amp' corresponding to initial amplitudes given by iIBSAmp
    void set_amp(Cvector iIBSAmp);

    // get amplitudes at given z position
    Cvector get_amp(double z);

    // to check various consistant conditions and other constratints
    bool inspect(void);

    // field values at position (x, z). complex mode profiles
    // cp: EX - HZ, ER, EP, HR, HP, SX, SY, SZ, SP, SR */
    // note the coordinate system convention!

    Complex field(Fcomp cp, double x, double z)const;
    double field(Fcomp cp, Afo foa, double x, double z)const;

    Complex mirror_field(Fcomp cp, double x, double z)const;

    /* the optical field, discretized on a regular rectangular mesh,  
       npx x npz points on a display window dwx x dwz
       cp: one of E?, H?, S? ... ?
       foa: ORG, MOD, SQR, REP, IMP (ORG = REP) 
    */
    Dmatrix field(Fcomp cp, Afo foa, Interval dwx, int npx, 
		  Interval dwz, int npz) const;


    // writes coupler specifications of type <spec> to file 
    void fwrite( char *filename, int spec);

    // field profile -> MATLAB .m file  
    //	cp: EX - HZ, SZ
    //	foa: MOD, ORG, SQR  
    //	fac: output factor (typically 1.0 or -1.0)  
    //	disp: output region on the x axis
    //	npx: number of points in output mesh
    //	ext0, ext1: filename id characters
    //      pltype: 'L': geometry information + plot commands (default)
    //              'V': field, mesh, and plot comand, 
    //                   to be included into *L.m 
    void mfile(Fcomp cp, Afo foa, Interval disp, double z,
	       int np, char ext0, char ext1, char pltype) const;

    // complete field profile on rectangular domain[x0, x1]x[z0,z1]
    // -> MATLAB .m file  
    //	cp: EX - HZ, SZ
    //	foa: MOD, ORG, SQR  
    //	fac: output factor (typically 1.0 or -1.0)  
    //	Nx, Nz number of points in X and Z direction
    //	ext0, ext1: filename id characters
    //      pltype: 'L': geometry information + plot commands (default)
    //              'V': field, mesh, and plot comand, 
    //                   to be included into *L.m 
    void mfile(Fcomp cp, Afo foa, 
	       double x0, double x1, int Nx, 
	       double z0, double z1, int Nz,
	       char ext0, char ext1, char pltype) const;

    // S is a scatterin matrix for z= [z0, z1]. reduced_S() reduces this 
    // scattering matrix along line of symmetry z=0.
    Cmatrix reduced_S(void)const;

    /* image of component cp
       cp: one of E?, H?, S? ... ?
       foa: MOD, SQR, REP, IMP, ORG = REP 
       xbeg, xend, zbeg, zend: x resp. z extensions of the plot window   
       npx, npz: number of plot points in the x and z directions
       ext0, ext1: filename id characters for the m.file 
    */
    void plot(Fcomp cp, Afo foa, 
              double xbeg, double xend, double zbeg, double zend,
	      int npx, int npz, char ext0, char ext1) const;

    // given angle theta (anticlockwise dir) of a 'z' position 
    // w.r.t. coupler window.
    double theta(double z)const;

    // gives correction matrix at level z
    void correction_matrix(double z, Cmatrix &O);
};


#endif //BSCOUPLER_H
