/*
 * Circurs - Circular optical microresonators simulation tools
 * Kirankumar R. Hiremath (k.r.hiremath@ieee.org)
 * University of Twente, Department of Applied Mathematics 
 * P.O. Box 217, 7500AE Enschede, The Netherlands
 * (2005)
 */

/*
 * bslamarr.c
 * arrays of BSlabModes, mode interference evaluation
 */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#include"bslawaveg.h"
#include"bslamvis.h"
#include"bslamode.h"
#include"bslamarr.h"

/* For all the required stuff corresponding to straight waveguide */
#include "metric.h"

/* BSlabModeArray: Array of BSlabModes */

/* error message */
void bslamarrerror(char *msg)
{
	fprintf(stderr, "BSlabModeArray: Error: %s.\n", msg);
	exit(1);
}

/* initialize */
BSlabModeArray::BSlabModeArray()
              : num(0)
{
	int i;
	for(i=0; i<=MAXNUMSLABMODES-1; ++i) mvec[i] = NULL;
}

/* destroy */
BSlabModeArray::~BSlabModeArray()
{
	int i;
	for(i=0; i<=MAXNUMSLABMODES-1; ++i) 
	{
		if(mvec[i] != NULL)
		{
			delete mvec[i];
			mvec[i] = NULL;
		}
	}
	num = 0; 
}

/* copy constructor */
BSlabModeArray::BSlabModeArray(const BSlabModeArray& ma)
	      : num(ma.num)
{
	int i;
	for(i=0; i<=num-1; ++i)
	{
		mvec[i] = new BSlabMode;
		*mvec[i] = *(ma.mvec[i]);
	}
}
	
/* assignment */
BSlabModeArray& BSlabModeArray::operator=(const BSlabModeArray& ma)
{
	int i;
	if(this != &ma)
	{
		for(i=0; i<=MAXNUMSLABMODES-1; ++i) 
		{
			if(mvec[i] != NULL)
			{
				delete mvec[i];
				mvec[i] = NULL;
			}
		}
		num = ma.num; 
		for(i=0; i<=num-1; ++i)
		{
			mvec[i]  = new BSlabMode;
			*mvec[i] = *(ma.mvec[i]);
		}
	}
	return *this;
}

/* delete all Mode entries */
void BSlabModeArray::clear()
{
	int i;
	for(i=0; i<=MAXNUMSLABMODES-1; ++i) 
	{
		if(mvec[i] != NULL)
		{
			delete mvec[i];
			mvec[i] = NULL;
		}
	}
	num = 0; 
}

/* input from FILE dat */
void BSlabModeArray::read(FILE *dat)
{
	int i;
	if(dat == stderr || dat == stdout)
	{
		bslamarrerror("BSlabModeArray: read");
	}
	else
	{
		for(i=0; i<=MAXNUMSLABMODES-1; ++i) 
		{
			if(mvec[i] != NULL)
			{
				delete mvec[i];
				mvec[i] = NULL;
			}
		}
		num = 0; 
		num = fgetint(dat);
		if(num >= MAXNUMSLABMODES)
		{
			fprintf(stderr, 
				"BSlabModeArray: read: NUM reduced to %d.\n",
				MAXNUMSLABMODES);
			num = MAXNUMSLABMODES;
		}
		for(i=0; i<=num-1; ++i)
		{
			mvec[i] = new BSlabMode;
			(*mvec[i]).read(dat); 
		}
	}
	return;
}

/* output to FILE dat */
void BSlabModeArray::write(FILE *dat)
{
	int i;
	comment("BSlabModeArray", dat);
	comment("num", dat); fputint(num, dat);
	for(i=0; i<=num-1; ++i)
	{
		comment("*** -------- ***", dat);
		(*mvec[i]).write(dat); 
	}
	return;
}

/* input from default file */
void BSlabModeArray::read_def(char ext0, char ext1)
{
	char name[10];
	FILE *dat;
	int i;
	name[0] = 's';
	name[1] = 'l';
	name[2] = 'a';
	name[3] = ext0;
	name[4] = ext1;
	name[5] = '.';
	name[6] = 'b';
	name[7] = 'm';
	name[8] = 'a';
	name[9] = 0;

	fprintf(stderr, "<< %s ", name);
	
	dat = fopen(name, "r");
	read(dat);
	fclose(dat);

	fprintf(stderr, "#%d\n", num);
	for(i=0; i<=num-1; ++i)
		fprintf(stderr, 
		  "   (%d) t%c,  gamma: %.10g +i %.10g,  neff: %.10g -i %.10g\n",
			i,
			polchr((*mvec[i]).pol),
			(*mvec[i]).beta, 
			(*mvec[i]).alpha, 
			(*mvec[i]).neff.re,
			(*mvec[i]).neff.im); 
	return;
}

/* output to default file */
void BSlabModeArray::write_def(char ext0, char ext1)
{
	char name[10];
	FILE *dat;
	int i;
	name[0] = 's';
	name[1] = 'l';
	name[2] = 'a';
	name[3] = ext0;
	name[4] = ext1;
	name[5] = '.';
	name[6] = 'b';
	name[7] = 'm';
	name[8] = 'a';
	name[9] = 0;

	fprintf(stderr, ">> %s #%d\n", name, num);
	for(i=0; i<=num-1; ++i)
		fprintf(stderr, 
		  "   (%d) t%c,  gamma: %.10g + i %.10g  neff: %.10g- i %.10g\n",
			i,
			polchr((*mvec[i]).pol),
			(*mvec[i]).beta, 
			(*mvec[i]).alpha, 
			(*mvec[i]).neff.re,
			(*mvec[i]).neff.im); 
	dat = fopen(name, "w+");
	write(dat);
	fclose(dat);

	return;
}

/* subscripting */
BSlabMode& BSlabModeArray::operator() (int i)  
{ 
	if(i<0 || i>=num) bslamarrerror("BSlabModeArray: () out of range");
	return *mvec[i]; 
}
BSlabMode BSlabModeArray::operator() (int i) const 
{ 
	if(i<0 || i>=num) bslamarrerror("BSlabModeArray: () out of range");
	return *mvec[i]; 
}

/* add a mode */
void BSlabModeArray::add(BSlabMode m)
{
	if(num >= MAXNUMSLABMODES)
		bslamarrerror("BSlabModeArray: add: MAXNUMSLABMODES exceeded");
	if(mvec[num] != NULL)
		bslamarrerror("BSlabModeArray: add: mvec corrupted");
	mvec[num] = new BSlabMode;
	*mvec[num] = m;
	++num;
	return;
}

/* delete a mode entry */
void BSlabModeArray::remove(int i)
{
	int j;
	if(i<0 || i>=num) 
		bslamarrerror("BSlabModeArray: remove: argument out of range");
	if(mvec[i] == NULL)
		bslamarrerror("BSlabModeArray: remove: mvec corrupted");
	delete mvec[i];
	mvec[i] = NULL;
	if(i==num-1) 
	{
		--num;
		return;
	}
	for(j=i; j<=num-2; ++j) mvec[j] = mvec[j+1];
	mvec[num-1] = NULL;
	--num;
	return;
}

/* add an entire BSlabModeArray nma, nma is cleared ! */ 
void BSlabModeArray::merge(BSlabModeArray& ma)
{
	int j;
	if(num+ma.num >= MAXNUMSLABMODES)
		bslamarrerror("BSlabModeArray: merge: MAXNUMSLABMODES exceeded");
	if(mvec[num] != NULL)
		bslamarrerror("BSlabModeArray: merge: mvec corrupted");
	for(j=0; j<=ma.num-1; ++j)
	{
		if(mvec[num] != NULL)
			bslamarrerror("BSlabModeArray: merge: mvec corrupted");
		mvec[num] = ma.mvec[j];
		++num;
		ma.mvec[j] = NULL;
	}
	ma.num = 0;
	return;
}

/* sort the array by (real) effective mode indices, highest first */
void BSlabModeArray::sort()
{
	int j, k, maxi;
	double maxn;
	BSlabModeP t;
	if(num<=1) return;
	for(j=0; j<=num-2; ++j)
	{
		maxi = j; 
		maxn = (*mvec[j]).neff.re;
		for(k=j+1; k<=num-1; ++k)
		{
			if(maxn<(*mvec[k]).neff.re)
			{
				maxn = (*mvec[k]).neff.re;
				maxi = k; 
			}
		}
		t = mvec[j]; mvec[j] = mvec[maxi]; mvec[maxi] = t;
	}
	return;
}

/* ................................................................... */

/* 
 * Mode interference evaluation,
 * all modes are assumed to belong to the same bend waveguide,
 * note the coordinate system convention.
 */

/* field superposition at point (r, phi), 
   amp:  complex amplitudes at phi=0
	 amplitudes evolve according to 
         amp_j(phi) = amp_j(0)*exp(-i gamma_j*R*phi)
   cp:   ER, EY, EP, HR, HY, HP, SR, SY, SP; EX, EZ, HX, HZ, SX, SZ */
Complex BSlabModeArray::field(const Cvector& amp, Fcomp cp, 
                              double r, double phi) const
{
	if(amp.nel  < num) bslamarrerror("field: array length mismatch");
	Complex s = CC0;
	Complex er=CC0, ey=CC0, ep=CC0, hr=CC0, hy=CC0, hp=CC0;
	int j;
	switch(cp)
	{
	case ER: case EY: case EP: case EX: case EZ: 
	case HR: case HY: case HP: case HX: case HZ:
		for(j=0; j<=num-1; ++j)
			s = s+amp(j)*(*mvec[j]).field(cp, r, phi);  
		return s;
	case SR:
		for(j=0; j<=num-1; ++j)
		{
			ey = ey+amp(j)*(*mvec[j]).field(EY, r, phi);
			ep = ep+amp(j)*(*mvec[j]).field(EP, r, phi);
			hy = hy+amp(j)*(*mvec[j]).field(HY, r, phi);
			hp = hp+amp(j)*(*mvec[j]).field(HP, r, phi);
		}
		s = ey*hp.conj()-ep*hy.conj();
		return Complex(0.5*s.re, 0.0);
	case SY:
		for(j=0; j<=num-1; ++j)
		{
			er = er+amp(j)*(*mvec[j]).field(ER, r, phi);
			ep = ep+amp(j)*(*mvec[j]).field(EP, r, phi);
			hr = hr+amp(j)*(*mvec[j]).field(HR, r, phi);
			hp = hp+amp(j)*(*mvec[j]).field(HP, r, phi);
		}
		s = ep*hr.conj()-er*hp.conj();
		return Complex(0.5*s.re, 0.0);
		break;
	case SP:
		for(j=0; j<=num-1; ++j)
		{
			er = er+amp(j)*(*mvec[j]).field(ER, r, phi);
			ey = ey+amp(j)*(*mvec[j]).field(EY, r, phi);
			hr = hr+amp(j)*(*mvec[j]).field(HR, r, phi);
			hy = hy+amp(j)*(*mvec[j]).field(HY, r, phi);
		}
		s = er*hy.conj()-ey*hr.conj();
		return Complex(0.5*s.re, 0.0);
		break;
	case SX: 
		return cos(phi)*field(amp, SR, r, phi) 
		      -sin(phi)*field(amp, SP, r, phi); 
	case SZ: 
		return sin(phi)*field(amp, SR, r, phi) 
		      +cos(phi)*field(amp, SP, r, phi); 
	default: bslamarrerror("field: mysterious cp");
	}
	return CC0;
}

/* field superposition at point (x, z), 
   amp:  complex amplitudes at phi=0
	 amplitudes evolve according to 
         amp_j(phi) = amp_j(0)*exp(-i gamma_j*R*phi)
   cp:   ER, EY, EP, HR, HY, HP, SR, SY, SP; EX, EZ, HX, HZ, SX, SZ */
Complex BSlabModeArray::field_cart(const Cvector& amp, Fcomp cp, 
                                   double x, double z) const
{
        double r, phi;

        r = sqrt(x*x+z*z);
        if(x>0.0) phi = atan(z/x);
        else
        {
                if(x<0.0)
                {
                        if(z>0.0) phi = PI-atan(fabs(z/x));
                        else
                        {
                                if(z<0.0) phi = -PI+atan(fabs(z/x));
                                else phi = PI;
                        }
                }
                else
                {
                        if(z>0.0) phi = PI/2.0;
                        else
                        {
                                if(z<0.0) phi = -PI/2.0;
                                else phi = 0.0;
                        }
                }
        }
        return field(amp, cp, r, phi);
}


/* - Output to MATLAB .m files ---------------------------------------- */

/* interference pattern on the 
   plane xbeg <= x <= xend, zbeg <= z <= zend,
   image plot corresponding to field component cp
   cp: ER, EY, EP, HR, HY, HP, SR, SY, SP; EX, EZ, HX, HZ, SX, SZ 
   foa: MOD, SQR, REP, IMP
   amp: mode amplitudes at phi=0, respectively z=0 
   npx, npz: number of points in output mesh
   ext0, ext1: filename id characters */
void BSlabModeArray::plot(const Cvector& amp, Fcomp cp, Afo foa,
                         double xbeg, double xend, int npx,
                         double zbeg, double zend, int npz,
                         char ext0, char ext1) const
{
	if(num < 1) bslamarrerror("plot: num < 1");
	BSlabWaveguide bend = (*mvec[0]).bend;
	FILE *dat;
	char name[14] = "bpl_____I.m";
	double x, dx, z, dz;
	Complex f; 
	int i, j;
	switch(foa)
        {
                case MOD: name[3] = 'm'; break;
                case ORG: name[3] = 'f'; break;
                case SQR: name[3] = 'q'; break;
                case REP: name[3] = 'r'; break;
                case IMP: name[3] = 'i'; break;
        }
        name[4] = fldchr(cp);
        name[5] = cpchr(cp);      
	name[6] = ext0;
	name[7] = ext1;
	fprintf(stderr, "plot_b(x=%g -> %g, z=%g -> %g) >> %s\n", 
	                xbeg, xend, zbeg, zend, name);
	dat = fopen(name, "w+");
	mlout_title(dat, name, "BSLAMS interference field");
	mlout_bendgeoxz(dat, bend, xbeg, xend, zbeg, zend);
	mlout_meshxz(dat, xbeg, xend, npx, zbeg, zend, npz);

	Dmatrix fld(npx, npz);
	dx = (xend-xbeg)/(npx-1);
	dz = (zend-zbeg)/(npz-1);

	for(j=0; j<=npz-1; ++j)
	{
		z = zbeg+j*dz;
		for(i=0; i<=npx-1; ++i) 
		{
			x = xbeg+i*dx;
			f = field_cart(amp, cp, x, z);
			if(foa == MOD) fld(i, j) = f.abs();
			if(foa == SQR) fld(i, j) = f.sqabs();
			if(foa == ORG) fld(i, j) = f.abs();
			if(foa == REP) fld(i, j) = f.re;
			if(foa == IMP) fld(i, j) = f.im;
		}
	}
	mlout_fld(dat, npx, npz, cp, fld);
	name[9] = 0; 
	mlout_bendimage(cp, name, dat); 
	mlout_print(dat, name, 'p'); 
	fclose(dat);
	return;
}

/* animation of the light propagation along the waveguide bend,
   the frames show the EY component (TE) resp. HY component (TM), at
   ntfr equidistant times over one time period
   amp: mode amplitude at phi=0, respectively z=0 
   xbeg, xend, zbeg, zend: x resp. z extensions of the plot window   
   npx, npz: number of plot points in the x and z directions
   ntfr: number of frames per time period
   ext0, ext1: filename id characters for the m.file */
void BSlabModeArray::movie(const Cvector& amp, 
			   double xbeg, double xend,int npx, 
			   double zbeg, double zend,int npz,
			   int ntfr, char ext0, char ext1) const 
{
	if(num < 1) bslamarrerror("plot: num < 1");
	BSlabWaveguide bend = (*mvec[0]).bend;
	FILE *dat;
	char name[14] = "ban____M.m";
	double x, dx, z, dz;
	double tmax, dt;
	Complex f; 
	int i, j;
	Polarization p = (*mvec[0]).pol;
	double lambda = (*mvec[0]).bend.wg.lambda;
	Fcomp cp = principalcomp(p);
        name[3] = fldchr(cp);
        name[4] = cpchr(cp);      
	name[5] = ext0;
	name[6] = ext1;

	tmax = period_T(lambda);
	dt = tmax/ntfr;

	fprintf(stderr, 
	   "movie(z=%g -> %g), %d frames, T=%g fs, dt = %g fs >> %s\n", 
	   zbeg, zend, ntfr, tmax, dt, name);
	dat = fopen(name, "w+");
	mlout_title(dat, name, "BSLAMS interference animation");
	mlout_bendgeoxz(dat, bend, xbeg, xend, zbeg, zend);
	mlout_meshxz(dat, xbeg, xend, npx, zbeg, zend, npz);

	Cmatrix fld(npx, npz);
	double famp = 0.0;

	dx = (xend-xbeg)/(npx-1);
	dz = (zend-zbeg)/(npz-1);

	for(j=0; j<=npz-1; ++j)
	{
		z = zbeg+j*dz;
		for(i=0; i<=npx-1; ++i) 
		{
			x = xbeg+i*dx;
			f = field_cart(amp, cp, x, z);
			fld(i, j) = f;
			if(f.abs() > famp) famp = f.abs();
		}
	}
	mlout_fld(dat, npx, npz, cp, fld.re());
	mlout_fldtore(dat, cp);
	mlout_fld(dat, npx, npz, cp, fld.im());
	mlout_fldtoim(dat, cp);
	name[8] = 0; 
	mlout_bendmovie(cp, name, dat, ntfr, dt, omega(lambda), famp); 
	fclose(dat);
	return;
}

//-----------------------------------------------------------
Complex BSlabModeArray::orthogonality(int mode_p, int mode_q, 
				       double r0, double r1,
				      int Nr) const
{
    Complex fac = ((*mvec[mode_p]).gamma +  (*mvec[mode_q]).gamma)* (*mvec[mode_p]).bend.R;

    double dr = (r1 - r0)/((double)(Nr));

    Complex sum;
    if (r0 == 0.0)
	sum = CC0;
    else
	sum = orthofun(mode_p,mode_q,r0) * 0.5;

    for(int i= 1; i < Nr; i++) sum +=  orthofun(mode_p,mode_q,r0 + i*dr);
    
    sum += orthofun(mode_p,mode_q,r1)*0.5;

    Complex summid = CC0;
    
    for(int i= 1; i <= Nr; i++) summid +=  orthofun(mode_p, mode_q,r0 + (i - 0.5)*dr);
    
    return (sum + 2*summid)*dr*fac/3.0;
}

Complex BSlabModeArray::orthofun(int mode_p, int mode_q, double r)const
{
    Complex tmp1, tmp2;

    tmp1 = (*mvec[mode_p]).field(EY, r)*(*mvec[mode_q]).field(EY, r)/r;
    tmp1 = tmp1 * val_invommu0((*mvec[mode_p]).bend.wg.lambda);
    
    tmp2 = (*mvec[mode_p]).field(HY, r)*(*mvec[mode_q]).field(EY, r)/r;
    tmp2 = tmp2 * val_invomep0((*mvec[mode_p]).bend.wg.lambda);
    tmp2 = tmp2 / (*mvec[mode_p]).bend.eps(r);

    return tmp1 + tmp2;
}

