/*
 * Circurs - Circular optical microresonators simulation tools
 * Kirankumar R. Hiremath (k.r.hiremath@ieee.org)
 * University of Twente, Department of Applied Mathematics 
 * P.O. Box 217, 7500AE Enschede, The Netherlands
 * (2005)
 */

/*
 * bslamarr.h
 * arrays of BSlabModes, (mode interference evaluation ...)
 */


#ifndef BSLAMARR_H
#define BSLAMARR_H

class BSlabMode;

/* mode pointer */
typedef BSlabMode* BSlabModeP;


/* array of BSlabModes */
#define MAXNUMSLABMODES 1000   // maximum number of modes in array

class BSlabModeArray 
{
public:
	// number of modes included
	int num;
	// initialize
	BSlabModeArray();
	// destroy
	~BSlabModeArray();
	// copy constructor 
	BSlabModeArray(const BSlabModeArray& ma);
	// assignment 
	BSlabModeArray& operator=(const BSlabModeArray& s);

	// free allocated memory
	void clear();

	// input from FILE dat
	void read(FILE *dat);
	// input from file with default name
	void read_def(char ext0, char ext1);
	// output to FILE dat
	void write(FILE *dat);
	// output to file with default name
	void write_def(char ext0, char ext1);

	// subscripting
	BSlabMode& operator()(int i); 
	BSlabMode operator()(int i) const; 

	// add a mode
	void add(BSlabMode m);
	// delete a mode entry
	void remove(int i);
	// add an entire BSlabModeArray nma, nma is cleared ! 
	void merge(BSlabModeArray& nma);

	// sort the array by effective mode indices, highest first
	void sort();

	// -----------------------------------------------------------
	// Mode interference evaluation,
	// all modes are assumed to belong to the same bend waveguide, 
	// note the coordinate system convention.

	// field superposition at point (r, phi),
	// amp: complex amplitudes at phi=0
	//      amplitudes evolve according to
	//      amp_j(phi) = amp_j(0)*exp(-i gamma_j*phi)
	// cp: ER, EY, EP, HR, HY, HP, SR, SY, SP; EX, EZ, HX, HZ, SX, SZ 
	Complex field(const Cvector& amp, Fcomp cp, 
	              double r, double phi) const;

	// field superposition at point (x, z), 
	// amp: complex amplitudes at phi=0 
	//      amplitudes evolve according to 
	//      amp_j(phi) = amp_j(0)*exp(-i gamma_j*R*phi)
	// cp: ER, EY, EP, HR, HY, HP, SR, SY, SP; EX, EZ, HX, HZ, SX, SZ 
	Complex field_cart(const Cvector& amp, Fcomp cp, 
                           double x, double z) const;

	// ----------------------------------------------------------- 
	// Visualization: output to MATLAB .m files  

	// interference pattern on the
	// plane xbeg <= x <= xend, zbeg <= z <= zend,
	// image plot corresponding to field component cp
	// cp: ER, EY, EP, HR, HY, HP, SR, SY, SP; EX, EZ, HX, HZ, SX, SZ
	// foa: MOD, SQR, REP, IMP
	// amp: mode amplitudes at phi=0, respectively z=0
	// npx, npz: number of points in output mesh
	// ext0, ext1: filename id characters 
	void plot(const Cvector& amp, Fcomp cp, Afo foa,
                  double xbeg, double xend, int npx,
                         double zbeg, double zend, int npz,
                         char ext0, char ext1) const;

	// animation of the light propagation along the waveguide bend,
	// the frames show the EY component (TE) resp. HY component (TM), at
	// ntfr equidistant times over one time period
	// amp: mode amplitude at phi=0, respectively z=0
	// xbeg, xend, zbeg, zend: x resp. z extensions of the plot window
	// npx, npz: number of plot points in the x and z directions
	// ntfr: number of frames per time period
	// ext0, ext1: filename id characters for the m.file 
	void movie(const Cvector& amp,
                   double xbeg, double xend, int npx,
		   double zbeg, double zend, int npz, 
		   int ntfr, char ext0, char ext1) const;

	// check the orthogonality on the interval L
	// which is divided into dl points.

	Complex orthogonality(int mode_p, int mode_q, 
			       double r0, double r1, int dl) const;

	// function used in orthogonality()
	Complex orthofun(int mode_p, int mode_q, double r)const;

private:
	BSlabModeP mvec[MAXNUMSLABMODES];
};

#endif // BSLAMARR_H
