/*
 * Circurs - Circular optical microresonators simulation tools
 * Kirankumar R. Hiremath (k.r.hiremath@ieee.org)
 * University of Twente, Department of Applied Mathematics 
 * P.O. Box 217, 7500AE Enschede, The Netherlands
 * (2005)
 */

/*
 * bslamode.c
 * bend slab mode definitions
 */

#include<stdio.h>
#include<fstream>


#include<math.h>
#include"bslawaveg.h"

#include"bslamode.h"

#define MY_MU0 1.2566370614E-12
#define MY_EP0 8.92062058E-18  //in microns
#define RTOL 1.0E-15    

/* error message */
void bslabmodeerror(char *s)
{
    fprintf(stderr, " ERROR: BSlabMode: %s.\n", s);
    exit(1);
}
// power on infinite interval at an angle theta
double BSlabMode::totpower(double theta)
{
    if (set_field_coefficients_flag != 1)
    {
	set_field_coefficients();
    }

    double tmp;
    tmp = abs(D_c)*abs(D_c)  * exp( alpha * bend.R * (PI - 2.0 * theta));
    tmp = tmp / ( 2.0 * PI * alpha * bend.R);

    if ( alpha <= 0.0)
    {
	cerr << "From BSlabMode::totpower(double theta): Error in alpha. Quitting.\n";
	exit(1);
    }
    
    if (pol == 0)
    { // TE
	tmp = tmp * val_invommu0(bend.wg.lambda);
    }
    else
    { // TM
	tmp = tmp * val_invomep0(bend.wg.lambda) / bend.wg.eps( bend.wg.nx + 1);
    }

    return tmp;

}

// trying analytic definition of totpower()
double BSlabMode::totpower(Interval xinterval)
{

    if (set_field_coefficients_flag != 1)
    {
	set_field_coefficients();
    }

    // power in substrat, film and cover resp. au upper and lower limits
    complex <double> P_s_l, P_s_u, P_f_l, P_f_u, P_c_l, P_c_u ;
    // check it should be real !

    int NZ, IERR;
    double a; // argument of bessel fun.
    Complex nu = gamma * bend.R;

    complex <double> NU ( nu.re, nu.im);
    complex<double> b[2];

    complex <double> D1, D2;
    D1 = (NU + conj(NU))*(NU - conj(NU));
    D2 = NU + conj(NU);

// in substrate 
    
    // lower limit = xinterval.x0
    a = bend.wg.n(0) * k0 * xinterval.x0; 
    b[0] = C_s * BesselJ(NU, a, 1, &NZ, &IERR);
    b[1] = C_s * BesselJ(NU + 1.0, a, 1, &NZ, &IERR);

    P_s_l =  (conj(b[1]) * b[0] - b[1] * conj(b[0])) * a / D1 +
	     b[0] * conj(b[0]) / D2;
    
///*

    // uppper limit = R^{-}
    a = bend.wg.n(0) * k0 * ( bend.R + bend.wg.hx(0) );
    b[0] = C_s * BesselJ(NU, a, 1, &NZ, &IERR);
    b[1] = C_s * BesselJ(NU + 1.0, a, 1, &NZ, &IERR);
    
    P_s_u =  (conj(b[1]) * b[0] - b[1] * conj(b[0])) * a / D1 +
	b[0] * conj(b[0]) / D2;

// in film

    // lower limit 
    a = bend.wg.n(1) * k0 * ( bend.R + bend.wg.hx(0) );
    b[0] = C_f * BesselJ(NU, a, 1, &NZ, &IERR) + 
	   D_f * BesselY(NU, a, 1, &NZ, &IERR) ;
    b[1] = C_f * BesselJ(NU + 1.0, a, 1, &NZ, &IERR) + 
           D_f * BesselY(NU + 1.0, a, 1, &NZ, &IERR) ;

    P_f_l =  (conj(b[1]) * b[0] - b[1] * conj(b[0])) * a / D1 +
	b[0] * conj(b[0]) / D2;

    // upper limit 
    a = bend.wg.n(1) * k0 * ( bend.R + bend.wg.hx(1) );
    b[0] = C_f * BesselJ(NU, a, 1, &NZ, &IERR) + 
	   D_f * BesselY(NU, a, 1, &NZ, &IERR) ;
    b[1] = C_f * BesselJ(NU + 1.0, a, 1, &NZ, &IERR) + 
           D_f * BesselY(NU + 1.0, a, 1, &NZ, &IERR) ;

    P_f_u =  (conj(b[1]) * b[0] - b[1] * conj(b[0])) * a / D1 +
	     b[0] * conj(b[0]) / D2;
    
// in cover 

    // lower limit
    a = bend.wg.n(2) * k0 * ( bend.R + bend.wg.hx(1) );
    b[0] = D_c * HankelH2(NU, a, 1, &NZ, &IERR);
    b[1] = D_c * HankelH2(NU + 1.0, a, 1, &NZ, &IERR);
    
    P_c_l =  (conj(b[1]) * b[0] - b[1] * conj(b[0])) * a / D1 +
	     b[0] * conj(b[0]) / D2;
//*/

    // upper limit
    a = bend.wg.n(2) * k0 * xinterval.x1;
    b[0] = D_c * HankelH2(NU, a, 2, &NZ, &IERR);
    b[1] = D_c * HankelH2(NU + 1.0, a, 2, &NZ, &IERR);
    
    P_c_u =  (conj(b[1]) * b[0] - b[1] * conj(b[0])) * a / D1 +
	     b[0] * conj(b[0]) / D2;

// At theta = 0; factor in front of power expression
    
    double f;

    if ( pol == 0 )
    { // TE
	f = bend.R * beta * val_invommu0(bend.wg.lambda) / (2.0);
    }
    else
    {
	f = bend.R * beta * val_invomep0(bend.wg.lambda) / (2.0);

	P_s_l = P_s_l / bend.wg.eps(0);
	P_s_u = P_s_u / bend.wg.eps(0);
	P_f_l = P_f_l / bend.wg.eps(1);
	P_f_u = P_f_u / bend.wg.eps(1);
	P_c_l = P_c_l / bend.wg.eps(2);
	P_c_u = P_c_u / bend.wg.eps(2);

    }


/*

    cerr << "---------------from BSlabMode::totpower() ---------\n"  
	 << "P_s_l :" << P_s_l << "\n"
	 << "P_s_u :" << P_s_u << "\n"
	 << "P_f_l :" << P_f_l << "\n"
	 << "P_f_u :" << P_f_u << "\n"
	 << "P_c_l :" << P_c_l << "\n"
	 << "P_c_u :" << P_c_u << "\n";

*/

    complex <double> tmp = ((P_s_u + P_f_u + P_c_u) - (P_s_l + P_f_l + P_c_l))*f ;
    return real(tmp);
}

//--------------------------------------------------------------------

/* minimum distance between boundaries to be considered separated */
#define S_HDIST 1.0e-14


void BSlabMode::set_field_coefficients()
{
    cerr << "--*-- set_field_coefficients() called. --*--\n";
    complex<double> order(beta*bend.R, -alpha*bend.R), del;
  
    int NZ, IERR;
    double R_plus, R_min, a1, a2, a3, a4;
  
    R_plus = bend.R + bend.wg.hx(1);
    R_min  = bend.R + bend.wg.hx(0);
  
    a1 = bend.wg.n(0)*k0*R_min;      // ns*k0*R_min
    a2 = bend.wg.n(1)*k0*R_min;      // nf*k0*R_min 
    a3 = bend.wg.n(1)*k0*R_plus;     // nf*k0*R_plus
    a4 = bend.wg.n(2)*k0*R_plus;     // nc*k0*R_plus
  

    C_s = 1.0e+1;
  

    if (pol == TE)
    {
	del =  bend.wg.n(1)*
	    (  BesselJ(order, a2, 1, &NZ, &IERR) * DBesselY(order, a2, 1, &NZ, &IERR) - 
	       DBesselJ(order, a2, 1, &NZ, &IERR) *  BesselY(order, a2, 1, &NZ, &IERR) );
  
	C_f = (bend.wg.n(1)*BesselJ(order, a1, 1, &NZ, &IERR)
	       *DBesselY(order, a2, 1, &NZ, &IERR))/del  -
	    (bend.wg.n(0)*DBesselJ(order, a1, 1, &NZ, &IERR)
	     *BesselY(order, a2, 1, &NZ, &IERR))/del;
  
	C_f  = C_f * C_s;

	D_f = (-bend.wg.n(1)*BesselJ(order, a1, 1, &NZ, &IERR)
	       *DBesselJ(order, a2, 1, &NZ, &IERR))/del + 
	    (bend.wg.n(0)*DBesselJ(order, a1, 1, &NZ, &IERR)
	     *BesselJ(order, a2, 1, &NZ, &IERR))/del;
	D_f  = D_f * C_s;

	D_c =  (BesselJ(order, a3, 1, &NZ, &IERR)*C_f 
		+ BesselY(order, a3, 1, &NZ, &IERR)*D_f) / 

            HankelH2(order, a4, 1, &NZ, &IERR);
    }
    else if (pol == TM)
    {
	del =  (1.0/bend.wg.n(1))*
	    (  BesselJ(order, a2, 1, &NZ, &IERR) * DBesselY(order, a2, 1, &NZ, &IERR) -
	       DBesselJ(order, a2, 1, &NZ, &IERR) *  BesselY(order, a2, 1, &NZ, &IERR) );

	C_f = ((1.0/bend.wg.n(1))*BesselJ(order, a1, 1, &NZ, &IERR)
	       *DBesselY(order, a2, 1, &NZ, &IERR))/del  -
	    ((1.0/bend.wg.n(0))*DBesselJ(order, a1, 1, &NZ, &IERR)
	     *BesselY(order, a2, 1, &NZ, &IERR))/del;
	C_f  = C_f * C_s;

	D_f = (-(1.0/bend.wg.n(1))*BesselJ(order, a1, 1, &NZ, &IERR)
	       *DBesselJ(order, a2, 1, &NZ, &IERR))/del +
	    ( (1.0/bend.wg.n(0))*DBesselJ(order, a1, 1, &NZ, &IERR)
	      *BesselJ(order, a2, 1, &NZ, &IERR))/del;
	D_f  = D_f * C_s;
	
	D_c =  (BesselJ(order, a3, 1, &NZ, &IERR)*C_f 
		+ BesselY(order, a3, 1, &NZ, &IERR)*D_f) /
            HankelH2(order, a4, 1, &NZ, &IERR);
    }
    set_field_coefficients_flag = 1;
  
}


            
/* field values at radius r, complex mode profiles
   cp:  ER, EY, EP, HR, HY, HP, SR, SY, SP */
Complex BSlabMode::field(Fcomp cp, double r) //const
{

    if(bend.wg.nx > 1)
	bslabmodeerror("--*-- more than 1 layer: not implemented so far --*--");
    if(cpchr(cp) == 'x' || cpchr(cp) == 'z') 
	bslabmodeerror("--*-- field(cp, r): non cylindrical component --*--");
    Complex ans, mult_fact;

    //--------Speed up by discretisation----start----
    if ( set_d_field_flag == true)
    {
//	cerr << "From field(Cp, r) : using d_field()";
	int ndp = d_field_EY.nel-1;

// finding out two r positions in between given r lies
	double dr = (rinterval.x1 - rinterval.x0)/((double)ndp);
	int n0 = (int)floor((r-rinterval.x0)/dr);
	int n1 = (int)ceil((r-rinterval.x0)/dr);

	Complex e0, e1, r0, r1;
	if ( ( n0 >= 0 ) && ( n1 <= ndp) )
	{
	    if ( n0 == n1)
	    { // return stored values
		switch (cp)
		{
		    case ER:
			return d_field_ER(n0);
		    case EY :
    			return d_field_EY(n0);
		    case EP:
    			return d_field_EP(n0);
		    case HR:
    			return d_field_HR(n0);
		    case HY :
		    	return d_field_HY(n0);
		    case HP:
    			return d_field_HP(n0);
	            default: goto CONT;
		};
	    }
	    else
	    { //return linear interpolated values 
		switch (cp)
		{
		    case ER:
			e0 = d_field_ER(n0);
			e1 = d_field_ER(n1);
			break;
		    case EY :
			e0 = d_field_EY(n0);
			e1 = d_field_EY(n1);
			break;
		    case EP:
			e0 = d_field_EP(n0);
			e1 = d_field_EP(n1);
			break;
		    case HR:
			e0 = d_field_HR(n0);
			e1 = d_field_HR(n1);
			break;
		    case HY :
			e0 = d_field_HY(n0);
			e1 = d_field_HY(n1);
			break;
		    case HP:
			e0 = d_field_HP(n0);
			e1 = d_field_HP(n1);
			break;
	            default: goto CONT;
		} 
		r0 = ((double) n0)*dr+rinterval.x0;
		r1 = ((double) n1)*dr+rinterval.x0;
		// return linearly interpolated values
		// does following formula cause some numerical instability ?
		return e0 + (r-r0)*(e0 - e1)/(r0 - r1);
      CONT: ;
	    }
	}
    }
    //--------Speed up by discretisation----end----
    
    complex<double> order(beta*bend.R, -alpha*bend.R), cpp_ans;
    double R_plus, R_min, a1, a2, a3, a4;
    int NZ, IERR;

    R_plus = bend.R + bend.wg.hx(1);
    R_min  = bend.R + bend.wg.hx(0);

    a1 = bend.wg.n(0)*k0*R_min;      // ns*k0*R_min
    a2 = bend.wg.n(1)*k0*R_min;      // nf*k0*R_min
    a3 = bend.wg.n(1)*k0*R_plus;     // nf*k0*R_plus
    a4 = bend.wg.n(2)*k0*R_plus;     // nc*k0*R_plus

    if (set_field_coefficients_flag != 1)
    {
	set_field_coefficients();
    }


    if(pol == TM) 
    {
	switch(cp)
	{
	    case EY: case HR : case HP: case SY:
		return CC0;
	    case ER:    
	    {
		ans = field(HY,r);
		
		if ( r < RTOL)
		    mult_fact=0.0;
		else if ( (RTOL <= r) && (r <= R_min) )
		    mult_fact = (gamma*bend.R * val_invomep0(bend.wg.lambda))/(bend.wg.eps(0)*r);
		else if ( (R_min < r) && (r < R_plus))
		    mult_fact = (gamma*bend.R * val_invomep0(bend.wg.lambda))/(bend.wg.eps(1)*r);
		else
		    mult_fact = (gamma*bend.R * val_invomep0(bend.wg.lambda))/(bend.wg.eps(2)*r);
                           
		ans = mult_fact*ans;

		return ans;
	    }
	    case EP:
	    {
		if (( 0 <= r) && (r <= R_min))
		{
		    cpp_ans = C_s*bend.wg.n(0)*k0*DBesselJ(order, bend.wg.n(0)*k0*r, 1, &NZ, &IERR);
		    cpp_ans = (complex<double> (0,-1.0))*cpp_ans* normalization_factor * val_invomep0(bend.wg.lambda)/bend.wg.eps(0);
		}
		else if( ( R_min < r ) && ( r < R_plus ) )
		{
		    cpp_ans = C_f*bend.wg.n(1)*k0*DBesselJ(order, bend.wg.n(1)*k0*r, 1, &NZ, &IERR)+
			      D_f*bend.wg.n(1)*k0*DBesselY(order, bend.wg.n(1)*k0*r, 1, &NZ, &IERR);
		    cpp_ans = cpp_ans*(complex<double> (0, -1.0))*normalization_factor * val_invomep0(bend.wg.lambda)/bend.wg.eps(1);
		}                              
		else if ( r >= R_plus)
		{
		    cpp_ans = D_c*bend.wg.n(2)*k0*DHankelH2(order, bend.wg.n(2)*k0*r, 1, &NZ, &IERR);
		    cpp_ans = cpp_ans *(complex<double> (0,-1))*normalization_factor * val_invomep0(bend.wg.lambda)/bend.wg.eps(2);
		}
		
		ans = Complex(cpp_ans.real(), cpp_ans.imag());
		return ans;
		
	    }
	    case HY:        
	    {
		if (( 0 <= r) && (r <= R_min))
		    cpp_ans = C_s*BesselJ(order, bend.wg.n(0)*k0*r, 1, &NZ, &IERR);
		else if( ( R_min < r ) && ( r < R_plus ) )
		    cpp_ans = C_f*BesselJ(order, bend.wg.n(1)*k0*r, 1, &NZ, &IERR) +
			D_f*BesselY(order, bend.wg.n(1)*k0*r, 1, &NZ, &IERR);
		else if ( r >= R_plus)
		    cpp_ans = D_c*HankelH2(order, bend.wg.n(2)*k0*r, 1, &NZ, &IERR);

		cpp_ans = cpp_ans * normalization_factor;
		return Complex(cpp_ans.real(), cpp_ans.imag());
	    }
	    case SR:
	    {
		Complex s;
		s = field(EP, r)*field(HY, r).conj();
		return Complex(0.5*s.re, 0.0);
	    }
	    case SP:
	    {
		Complex s;
		s = field(ER, r)*field(HY, r).conj();
		return Complex(0.5*s.re, 0.0);
	    }
	    case EX : case EZ: case HX: case HZ: case SX: case SZ:
		bslabmodeerror("--*-- From field(): Error. FComp can't be cartesian component.--*--\n");
		//TM case ends  
	}
    }
    else  //TE case starts 
    {

	switch(cp)
	{
	    case ER: case EP: case HY: case SY:
		return CC0; 
	    case EY:
	    {
		if (( 0 <= r) && (r <= R_min))
		    cpp_ans = C_s*BesselJ(order, bend.wg.n(0)*k0*r, 1, &NZ, &IERR);
		else if( ( R_min < r ) && ( r < R_plus ) )
		{                     
		    cpp_ans = C_f*BesselJ(order, bend.wg.n(1)*k0*r, 1, &NZ, &IERR) +
			D_f*BesselY(order, bend.wg.n(1)*k0*r, 1, &NZ, &IERR);
		}
		else if ( r >= R_plus)  
		{
		    cpp_ans = D_c*HankelH2(order, bend.wg.n(2)*k0*r, 1, &NZ, &IERR);
		};
		    
		cpp_ans = cpp_ans * normalization_factor;
		return Complex(cpp_ans.real(), cpp_ans.imag());
	    }
	    case HR: 
	    {

		ans = field(EY,r);
		if ( r < RTOL)  //to avoid 0/0 case at r=0
		    mult_fact = 0;
		else
		    mult_fact = -1.0*gamma*bend.R * val_invommu0(bend.wg.lambda) / r;
		ans = mult_fact*ans;
		    
		return ans;
	    }
	    case HP:
	    {
		if (( 0 <= r) && (r <= R_min))
		{
		    cpp_ans = C_s*bend.wg.n(0)*k0*DBesselJ(order, bend.wg.n(0)*k0*r, 1, &NZ, &IERR);
		}
		else if( ( R_min < r ) && ( r < R_plus ) )
		{
		    cpp_ans = C_f*bend.wg.n(1)*k0*DBesselJ(order, bend.wg.n(1)*k0*r, 1, &NZ, &IERR)+ 
			D_f*bend.wg.n(1)*k0*DBesselY(order, bend.wg.n(1)*k0*r, 1, &NZ, &IERR);
		      
		}
		else if ( r >= R_plus)
		{
		    cpp_ans = D_c*bend.wg.n(2)*k0*DHankelH2(order, bend.wg.n(2)*k0*r, 1, &NZ, &IERR);
		}
  
		cpp_ans = cpp_ans* (complex<double> (0, 1.0)) * normalization_factor * val_invommu0(bend.wg.lambda);
		ans = Complex(cpp_ans.real(), cpp_ans.imag());
		  		  
		return ans;
	    }
	    case SR: 
	    {
		Complex s;
		s = field(EY, r)*field(HP, r).conj()
		    -field(EP, r)*field(HY, r).conj();
		return Complex(0.5*s.re, 0.0);
	    }
	    case SP: 
	    {
		Complex s;
		s = field(ER, r)*field(HY, r).conj()
		    -field(EY, r)*field(HR, r).conj();
		return Complex(0.5*s.re, 0.0);
	    }
	    default: return CC0;
	}	
	return CC0; 
    }
    return ans;
};

/* 'realized' field values at radius r 
   cp:  ER, EY, EP, HR, HY, HP, SR, SY, SP 
   foa: MOD, ORG, REP, IMP, SQR */
double BSlabMode::field(Fcomp cp, Afo foa, double r) //const
{
    Complex fc;
    fc = field(cp, r);
    switch(foa)
    {
	case REP: 
	    return fc.re;
	case IMP: 
	    return fc.im;
	case MOD: case ORG:
	    return fc.abs();
	case SQR:
	    return fc.sqabs();
    }
    return fc.re;
}

//-----------------------***-------------------------------

/* field values at radius r and angle p, complex mode profiles
   cp: EX - HZ, ER, EP, HR, HP, SX, SY, SZ, SP, SR 
   note the coordinate system convention ! */
Complex BSlabMode::field(Fcomp cp, double r, double p) //const
{
    switch(cp)
    {
	case ER: case EY: case EP: case HR: case HY: case HP:
	    return  field(cp, r)*exp(-1.0*CCI*gamma*bend.R*p);
	case SR: case SY: case SP:
	    return  field(cp, r)*exp(-2.0*alpha*bend.R*p);
	case EX:
	    return cos(p)*field(ER, r, p)-sin(p)*field(EP, r, p);
        case HX:
	    return cos(p)*field(HR, r, p)-sin(p)*field(HP, r, p);
        case SX:
	    return cos(p)*field(SR, r, p)-sin(p)*field(SP, r, p);
        case EZ:
	    return sin(p)*field(ER, r, p)+cos(p)*field(EP, r, p);
        case HZ:
	    return sin(p)*field(HR, r, p)+cos(p)*field(HP, r, p);
        case SZ:
	    return sin(p)*field(SR, r, p)+cos(p)*field(SP, r, p);
    }
    return CC0;
}

/* field values at a cartisian position x, z, complex mode profiles
   cp: EX - HZ, ER, EP, HR, HP, SX, SY, SZ, SP, SR 
   note the coordinate system convention ! */
Complex BSlabMode::field_cart(Fcomp cp, double x, double z) //const
{
    double r, phi;

    r = sqrt(x*x+z*z);


    if(x>0.0)
	phi = atan(z/x);	
    else
    {
	if(x<0.0) 
	{
	    if(z>0.0) phi = PI-atan(fabs(z/x));
	    else
	    {
		if(z<0.0) phi = -PI+atan(fabs(z/x));
		else phi = PI;
	    }
	}
	else  
	{
	    if(z>0.0) phi = PI/2.0;
	    else if(z<0.0) phi = -PI/2.0;
	    else phi = 0.0;   // ?
	}
    }

    return field(cp, r, phi); 
}

/* azimuthal component of the Poyntingvector, 
   integrated over the entire cross section up to ... */
/* total power */
double BSlabMode::totpower(Interval xinterval, double z, int xnodes)//const
{
    //computes total power (in cartisian system) in xinterval at position z. 
    // uses Simposon integration method with step size hx

//----Simposon integration method----START----------
//    int xnodes  = (int) (ceil((xinterval.x1 - xinterval.x0)/hx));
    double hx = (xinterval.x1 - xinterval.x0)/((double)xnodes);

    Complex sum, summid, ans;

    if (pol == TE)
    {
	sum = field_cart( EY, xinterval.x0, z).conj()* 
	    field_cart( HX, xinterval.x0, z)* 0.5;

	for (int i=1; i< xnodes; i++) 
	{
	    sum = sum + field_cart( EY, xinterval.x0 + i*hx, z).conj() * 
		field_cart( HX, xinterval.x0 + i*hx, z);
	}
	sum = sum +  field_cart( EY, xinterval.x1, z).conj() * 
	    field_cart( HX, xinterval.x1, z) * 0.5;

	
	for (int i=1; i<= xnodes; i++) 
	    summid = summid + field_cart( EY, xinterval.x0 + ( (double)i - 0.5)*hx, z).conj()* 
		field_cart( HX, xinterval.x0 + ((double)i-0.5)*hx, z);
    }

    if (pol == TM)
    {

	sum = field_cart( HY, xinterval.x0, z).conj()* field_cart( EX, xinterval.x0, z)* 0.5;

	for (int i=1; i< xnodes; i++)
	{
	    sum = sum + field_cart( HY, xinterval.x0 + i*hx, z).conj() * field_cart( EX, xinterval.x0 + i*hx, z);
	}
	sum = sum +  field_cart( HY, xinterval.x1, z).conj() * field_cart( EX, xinterval.x1, z) * 0.5;

	for (int i=0; i<= xnodes; i++)
	    summid = summid +
		field_cart( HY, xinterval.x0 + (i-0.5)*hx, z).conj()* field_cart( EX, xinterval.x0 + (i-0.5)*hx, z);
    }
        
    ans =  (sum + 2.0*summid)*hx/3.0;

    ans = ans + ans.conj(); // IMPORTANT STEP

    return abs(ans.re)/4.0;  

    //----Simposon integration method----END------------

}

// normalize modal power to 1
// nrm = total power in a given interval
// also E^{0}_{y} (R) or H^{0}_{y} (R) should be real and positive.
void BSlabMode::normalize(double nrm)
{
    Complex p;
    
    if (pol == 0 )
	p = field(EY, bend.R);
    else
	p = field(HY, bend.R);

    complex<double> P(p.re, p.im);
    P = P/abs(P); // to make mode profile real and positive at r = R

    normalization_factor = 1.0/ (P*sqrt(nrm));

}

/* data file: num equidistant values of component cp between r0 and r1 
   cp:  ER, EY, EP, HR, HY, HP, SR, SY, SP */
void BSlabMode::profiledata(Fcomp cp, 
			    int num, double r0, double r1, 
			    char ext0, char ext1) //const
{
    if(num <= 0) bslabmodeerror("profiledata: num <= 0");
    char name[11] = "bprt_____";
    name[4] = polchr(pol);
    name[5] = fldchr(cp);
    name[6] = cpchr(cp);
    name[7] = ext0;
    name[8] = ext1;

    double r;
    double dr;
    int j;
    Cvector f(num);
    Dvector pos(num);
    if(num == 1)
    {
	r = 0.5*(r0+r1);
	f(0) = field(cp, r);
	pos(0) = r;
    }
    else 
    {
	dr = (r1-r0)/(num-1);
	for(j=0; j<=num-1; ++j)
	{
	    r = r0+j*dr;
	    f(j) = field(cp, r);
	    pos(j) = r;
	}
    }
    fprintf(stderr, "   %c%c ", fldchr(cp), cpchr(cp));
    toxyf(name, pos, f); 
    return;
}

/* 'realized' field profile -> .xyf - file 
   cp:  ER, EY, EP, HR, HY, HP, SR, SY, SP 
   foa: MOD, ORG, REP, IMP, SQR
   fac: output factor (typically 1.0 or -1.0)
   disp: output region on the radial axis 
   ext0, ext1: filename id characters
   npx: number of points in output mesh */
void BSlabMode::writeprofile(Fcomp cp, Afo foa, double fac, Interval disp,
			     char ext0, char ext1, int num) //const
{
    if(num <= 0) bslabmodeerror("writeprofile: num <= 0");
    char name[11] = "bt______";
    name[2] = polchr(pol);
    switch(foa)
    {
	case MOD: name[3] = 'm'; break;
	case ORG: name[3] = 'f'; break;
	case SQR: name[3] = 'q'; break;
	case REP: name[3] = 'r'; break;
	case IMP: name[3] = 'i'; break;
    }
    name[4] = fldchr(cp);
    name[5] = cpchr(cp);
    name[6] = ext0;
    name[7] = ext1;
    double r;
    double dr;
    int j;
    Dvector rf(num);
    Dvector pos(num);
    if(num == 1)
    {
	r = 0.5*(disp.x0+disp.x1);
	rf(0) = field(cp, foa, r);
	pos(0) = r;
    }
    else 
    {
	dr = (disp.x1-disp.x0)/(num-1);
	for(j=0; j<=num-1; ++j)
	{
	    r = disp.x0+j*dr;
	    rf(j) = field(cp, foa, r);
	    pos(j) = r;
	}
    }
    fprintf(stderr, "   %c%c *(%g) ", fldchr(cp), cpchr(cp), fac);
    toxyf(name, pos, rf); 
    return;
}

/* ------------------------------------------------------------------------ */


/* field profile -> MATLAB .m file  
   cp: EX - HZ, ER, EP, HR, HP, SX, SY, SZ, SP, SR
   foa: MOD, ORG, REP, IMP, SQR
   fac: output factor (typically 1.0 or -1.0) 
   disp: output region on the x axis
   np: number of output points
   ext0, ext1: filename id characters
   pltype: 'L': geometry information + plot commands (default)
   'V': field, mesh, and plot command, to be included into *L.m */
#define SLHDISTF 1.0e-8
void BSlabMode::mfile(Fcomp cp, Afo foa, double fac, Interval disp, int np, 
		      char ext0, char ext1, char pltype) //const
{
    FILE *dat;
    int i, j;
    double x, dx, t;
    double x0, x1;
    double xbd;
    double epsold, epsnew;
    double minf, maxf, f;
    int nsec, nbd;
    char name[14] = "bt_______.m";
    name[2] = polchr(pol);
    switch(foa)
    {
	case MOD: name[3] = 'm'; break;
	case ORG: name[3] = 'f'; break;
	case SQR: name[3] = 'q'; break;
	case REP: name[3] = 'r'; break;
	case IMP: name[3] = 'i'; break;
    }
    name[4] = fldchr(cp);
    name[5] = cpchr(cp);
    name[6] = ext0;
    name[7] = ext1;
    if(pltype != 'V') pltype = 'L';
    name[8] = pltype;
	
    if(foa == MOD || foa == SQR) fac = fabs(fac);

    x0 = disp.x0;
    x1 = disp.x1;
	
    if(x0 > x1) { t = x0; x0 = x1; x1 = t; }

    fprintf(stderr, "   %c%c [%g -> %g] >> %s\n", 
	    fldchr(cp), cpchr(cp), x0, x1, name);

    Dmatrix fld(bend.wg.nx+2, np+1);
    Dmatrix pos(bend.wg.nx+2, np+1);
    Ivector nump(bend.wg.nx+2);
    Dvector of(np+1);
    Dvector op(np+1);
    Dvector bd(bend.wg.nx+1);
    Dvector ri(bend.wg.nx+2);
    nsec = 0;
    nbd  = 0;

    dx = x1-x0;
    nump(nsec) = 0;
    x = x0;
    epsold = bend.wg.eps(x-bend.R);  	
    ri(nbd) = sqrt(epsold);
    pos(nsec, nump(nsec)) = x-bend.R;  
    fld(nsec, nump(nsec)) = field(cp, foa, x)*fac;  
    ++nump(nsec);
    for(i=1; i<=np; ++i)
    {
	x = x0+i*dx/np;
	epsnew = bend.wg.eps(x-bend.R);
	if(fabs(epsnew-epsold)<1.0e-10)
	{
	    pos(nsec, nump(nsec)) = x-bend.R;  
	    fld(nsec, nump(nsec)) = field(cp, foa, x)*fac;  
	    ++nump(nsec);
	}
	else
	{
	    xbd = (bend.wg.layer(x-bend.R)).x0; 
	    pos(nsec, nump(nsec)) = xbd;  
	    fld(nsec, nump(nsec)) = 	
		field(cp, foa, xbd+bend.R-dx*SLHDISTF)*fac;  
	    ++nump(nsec);
	    bd(nbd) = xbd; 
	    ++nbd;
	    epsold = epsnew;
	    ri(nbd) = sqrt(epsold);
	    ++nsec;
	    nump(nsec) = 0;
	    pos(nsec, nump(nsec)) = xbd;  
	    fld(nsec, nump(nsec)) = 	
		field(cp, foa, xbd+bend.R+dx*SLHDISTF)*fac;  
	    ++nump(nsec);
	    pos(nsec, nump(nsec)) = x-bend.R;  
	    fld(nsec, nump(nsec)) = field(cp, foa, x)*fac;  
	    ++nump(nsec);
	}
    }
    ++nsec;

    dat = fopen(name, "w+");
    mlout_title(dat, name, "BSLAMS mode profile");
    name[8] = 0;

    minf = -1.0;
    maxf =  1.0;
    if(pltype == 'L')
    {
	minf = 1.0e+300;
	maxf = -1.0e-300;
	for(j=0; j<=nsec-1; ++j)
	{
	    for(i=0; i<=nump(j)-1; ++i)
	    {
		f = fld(j, i);
		if(f < minf) minf = f;
		if(f > maxf) maxf = f;
	    }
	}
	if(minf >= maxf) {minf=maxf-1.0; maxf=minf+1.0;}
	mlout_geo(dat, bend.wg, minf, maxf); 
	mlout_Lsecgeo(dat, x0-bend.R, x1-bend.R, nbd, bd, ri);    }

    for(j=0; j<=nsec-1; ++j)
    {
	for(i=0; i<=nump(j)-1; ++i)
	{
	    f = fld(j, i);
	    of(i) = f;
	    op(i) = pos(j, i);
	}
	if(pltype == 'L')
	    mlout_sec(dat, cp, dig10(j), dig1(j), ' ', ' ', 
			nump(j), of, op); 
	else
	    mlout_sec(dat, cp, dig10(j), dig1(j), ext0, ext1, 
			nump(j), of, op); 
    }

    if(pltype == 'L') 
    {
	mlout_Lsecplot_rad(name, dat, cp, nbd, nsec); 
	mlout_print(dat, name, 'e');
    }
    else mlout_Vsecplot(dat, cp, nsec, ext0, ext1); 
    fclose(dat);
    return;
}


/* complex mode profile -> MATLAB .m file
   cp: EX - HZ, ER, EP, HR, HP, SX, SY, SZ, SP, SR
   fac: output factor (typically 1.0 or -1.0)
   disp: output region on the x axis
   np: number of points in output mesh
   ext0, ext1: filename id characters
   pltype: 'r': real + imaginary part
   'a': amplitude + phase */
void BSlabMode::mfile(Fcomp cp, double fac, Interval disp, int np, 
		      char ext0, char ext1, char pltype) //const
{
    FILE *dat;
    int i, j;
    double x, dx, t;
    double x0, x1;
    double xbd;
    double epsold, epsnew;
    double minf, maxf, f;
    int nsec, nbd;
    char name[15] = "bt________.m";
    name[2] = polchr(pol);
    if(pltype != 'a') pltype = 'r';
    switch(pltype)
    {
	case 'r': name[3] = 'r'; name[4] = 'i'; break;
	case 'a': name[3] = 'a'; name[4] = 'p'; break;
    }
    name[5] = fldchr(cp);
    name[6] = cpchr(cp);
    name[7] = ext0;
    name[8] = ext1;
    name[9] = 'P';
	
    x0 = disp.x0;
    x1 = disp.x1;
	
    if(x0 > x1) { t = x0; x0 = x1; x1 = t; }

    fprintf(stderr, "   %c%c [%g -> %g] >> %s\n", 
	    fldchr(cp), cpchr(cp), x0, x1, name);

    Cmatrix fld(bend.wg.nx+2, np+1);
    Dmatrix pos(bend.wg.nx+2, np+1);
    Ivector nump(bend.wg.nx+2);
    Dvector of(np+1);
    Dvector op(np+1);
    Dvector bd(bend.wg.nx+1);
    Dvector ri(bend.wg.nx+2);
    nsec = 0;
    nbd  = 0;

    dx = x1-x0;
    nump(nsec) = 0;
    x = x0;
    epsold = bend.wg.eps(x-bend.R);  	
    ri(nbd) = sqrt(epsold);
    pos(nsec, nump(nsec)) = x-bend.R;  
    fld(nsec, nump(nsec)) = field(cp, x)*fac;  
    ++nump(nsec);
    for(i=1; i<=np; ++i)
    {
	x = x0+i*dx/np;
	epsnew = bend.wg.eps(x-bend.R);
	if(fabs(epsnew-epsold)<1.0e-10)
	{
	    pos(nsec, nump(nsec)) = x-bend.R;  
	    fld(nsec, nump(nsec)) = field(cp, x)*fac;  
	    ++nump(nsec);
	}
	else
	{
	    xbd = (bend.wg.layer(x-bend.R)).x0; 
	    pos(nsec, nump(nsec)) = xbd;  
	    fld(nsec, nump(nsec)) = 	
		field(cp, xbd+bend.R-dx*SLHDISTF)*fac;  
	    ++nump(nsec);
	    bd(nbd) = xbd; 
	    ++nbd;
	    epsold = epsnew;
	    ri(nbd) = sqrt(epsold);
	    ++nsec;
	    nump(nsec) = 0;
	    pos(nsec, nump(nsec)) = xbd;  
	    fld(nsec, nump(nsec)) = 	
		field(cp, xbd+bend.R+dx*SLHDISTF)*fac;  
	    ++nump(nsec);
	    pos(nsec, nump(nsec)) = x-bend.R;  
	    fld(nsec, nump(nsec)) = field(cp, x)*fac;  
	    ++nump(nsec);
	}
    }
    ++nsec;

    dat = fopen(name, "w+");
    mlout_title(dat, name, "BSLAMS complex mode profile");
    name[10] = 0;

    minf = -1.0;
    maxf =  1.0;
    if(pltype == 'r')
    {
	minf = 1.0e+300;
	maxf = -1.0e-300;
	for(j=0; j<=nsec-1; ++j)
	{
	    for(i=0; i<=nump(j)-1; ++i)
	    {
		f = fld(j, i).re;
		if(f < minf) minf = f;
		if(f > maxf) maxf = f;
		f = fld(j, i).im;
		if(f < minf) minf = f;
		if(f > maxf) maxf = f;
		f = fld(j, i).abs();
		if(f < minf) minf = f;
		if(f > maxf) maxf = f;
	    }
	}
    }
    else	
    {
	minf = 0.0;
	maxf = -1.0e-300;
	for(j=0; j<=nsec-1; ++j)
	{
	    for(i=0; i<=nump(j)-1; ++i)
	    {
		f = fld(j, i).abs();
		if(f > maxf) maxf = f;
	    }
	}
	minf = -maxf/20.0;
    }
    if(minf >= maxf) {minf=maxf-1.0; maxf=minf+1.0;}
    mlout_geo(dat, bend.wg, minf, maxf); 
    mlout_Lsecgeo(dat, x0-bend.R, x1-bend.R, nbd, bd, ri);

    for(j=0; j<=nsec-1; ++j)
    {
	for(i=0; i<=nump(j)-1; ++i)
	{
	    f = fld(j, i).re;
	    of(i) = f;
	    op(i) = pos(j, i);
	}
	mlout_sec(dat, cp, dig10(j), dig1(j), 'r', 'e', 
		    nump(j), of, op); 

	for(i=0; i<=nump(j)-1; ++i)
	{
	    f = fld(j, i).im;
	    of(i) = f;
	    op(i) = pos(j, i);
	}
	mlout_sec(dat, cp, dig10(j), dig1(j), 'i', 'm', 
		    nump(j), of, op); 
    }

    if(pltype == 'r') 
	mlout_bendriaplot(name, dat, minf, maxf, cp, nbd, nsec); 
    else 
	mlout_bendaphplot(name, dat, minf, maxf, cp, nbd, nsec); 
    mlout_print(dat, name, 'e');
    fclose(dat);
    return;
}

/* ------------------------------------------------------------------------ */

/* output to file */
void BSlabMode::write(FILE *dat)
{
    comment("BSlabMode", dat);	
    comment("pol", dat); fputint(pol, dat);
    comment("gamma", dat); gamma.write(dat);
    comment("beta", dat); fputdouble(beta, dat);
    comment("alpha", dat); fputdouble(alpha, dat);
    comment("k0", dat); fputdouble(k0, dat);
    comment("neff.re", dat); fputdouble(neff.re, dat);
    comment("neff.im", dat); fputdouble(neff.im, dat);
    bend.write(dat);
// write the internal representation ....
    return;
}

/* input from file */
void BSlabMode::read(FILE *dat)
{
    pol = Polarization(fgetint(dat));
    gamma.read(dat);
    beta = fgetdouble(dat);
    alpha = fgetdouble(dat);
    k0 = fgetdouble(dat);
    neff = fgetdouble(dat);
    bend.read(dat);
// read the internal representation ....
    return;
}

/* output to default file */
void BSlabMode::write_def(char ext0, char ext1)
{
    char name[10] = "t___.bsm";
    FILE *dat;
    name[1] = polchr(pol);
    name[2] = ext0;
    name[3] = ext1;

    fprintf(stderr, "gamma: %.10g + i %.10g,  neff: %.10g - i %.10g >> %s\n",
	    beta, alpha, neff.re, neff.im, name); 
    dat = fopen(name, "w+");
    write(dat);
    fclose(dat);
    return;
}

/* input from default file */
void BSlabMode::read_def(Polarization p, char ext0, char ext1)
{
    char name[10] = "t___.bsm";
    FILE *dat;
    name[1] = polchr(p);
    name[2] = ext0;
    name[3] = ext1;

    fprintf(stderr, "<< %s : ", name);
	
    dat = fopen(name, "r");
    read(dat);
    fclose(dat);

    fprintf(stderr, "gamma: %.10g + i %.10g,  neff: %.10g - i %.10g >> %s\n",
	    beta, alpha, neff.re, neff.im, name); 
    return;
}

//-------------------Addition by Kiran--------------------

BSlabMode::BSlabMode()
{
    Waveguide g =Waveguide();
    double R = 0;
    BSlabWaveguide bend(g,R);
    k0 = 2.0*PI / bend.wg.lambda;
    set_d_field_flag = false;
    //    set_field_coefficients_flag = 0;
    norm = 1.0;
    normalization_factor= 1.0;
    //    set_field_coefficients();


}

BSlabMode::BSlabMode(BSlabWaveguide bend_waveguide)
{
    if (bend_waveguide.wg.nx != 1 )
    {
	cerr << " Internal layers of input bent waveguide =  " << bend_waveguide.wg.nx << endl;
	cerr << "Routines are not implemented for more than 1 internal layer. :-( \n";
	exit(1);
    }
    bend = BSlabWaveguide(bend_waveguide.wg, bend_waveguide.R);
    k0 = 2.0*PI / bend.wg.lambda;
    normalization_factor = 1.0;
    set_d_field_flag = false;
    // set_field_coefficients_flag = 0;
    // set_field_coefficients();
    norm = 1.0;
}

BSlabMode::BSlabMode( BSlabWaveguide BS, Polarization bpol)
{
    if (BS.wg.nx != 1 )
    {
	cerr << " Internal layers of input bent waveguide =  " << BS.wg.nx << endl;
	cerr << "Routines are not implemented for more than 1 internal layer. :-( \n";
	exit(1);
    }

    bend = BSlabWaveguide(BS.wg, BS.R);
    pol = bpol;
    k0 = 2.0*PI / bend.wg.lambda;
    normalization_factor = 1.0;
    set_d_field_flag = false;
    // set_field_coefficients_flag = 0;
    // set_field_coefficients();
    norm = 1.0;
}

BSlabMode::BSlabMode( BSlabWaveguide BS, Polarization bpol, Complex bgamma)
{
    if (BS.wg.nx != 1 )
    {
	cerr << " Internal layers of input bent waveguide =  " << BS.wg.nx << endl;
	cerr << "Routines are not implemented for more than 1 internal layer. :-( \n";
	exit(1);
    }
    
    bend = BSlabWaveguide(BS.wg, BS.R);
    pol = bpol;
    k0 = 2.0*PI / bend.wg.lambda;
    gamma = bgamma;
    beta = gamma.re;
    alpha = -1.0*gamma.im;
    neff = beta /k0;

    normalization_factor = 1.0; //?
    set_d_field_flag = false;
    // set_field_coefficients_flag = 0;
    // set_field_coefficients();
    norm = 1.0; //?
}


//-------- Combined dispersion eq. for TE and TM----Start------
complex<double> BSlabMode::dispersion_eq(complex<double> z)
{
    //  Note: dispersion_eq()  are not implemented for more than 1 internal layer. :-( 
    //For 1 internal layer
	
    complex<double> ans;
    int KODE, NZ, LHS_IERR[8], RHS_IERR[8];
    
    double n_f, n_c, n_s, d, k_0;
    double R_f_plus, R_f_minus, R_c_plus, R_s_minus;
    
    complex<double> LHS[8], RHS[8];
	
    n_s = bend.wg.n(0);
    n_f = bend.wg.n(1);
    n_c = bend.wg.n(2);
    d   = abs( bend.wg.hx(1) - bend.wg.hx(0));
    k_0 = 2*Pi/bend.wg.lambda;

/* ... why not a general definition ? (tried by MH)
   Explanition : Let "d" be the width of the single layer bend core.
   1. hx(0) = -d , hx(1)= 0; then outer interfce at R. (so called new definition)
   2. hx(0) = -d/2,hx(1) = d/2, then outer interface is R + d/2. (old definition)
*/

    R_f_minus= n_f*k_0*(bend.R + bend.wg.hx(0));
    R_s_minus= n_s*k_0*(bend.R + bend.wg.hx(0));
    R_f_plus = n_f*k_0*(bend.R + bend.wg.hx(1));
    R_c_plus = n_c*k_0*(bend.R + bend.wg.hx(1));
    
    KODE = 1;
    
    LHS[0] =  BesselJ(z, R_f_minus, KODE, &NZ, &LHS_IERR[0]);
    LHS[1] =  BesselJ(z, R_s_minus, KODE, &NZ, &LHS_IERR[1]);
    LHS[2] = DBesselJ(z, R_f_minus, KODE, &NZ, &LHS_IERR[2]);
    LHS[3] = DBesselJ(z, R_s_minus, KODE, &NZ, &LHS_IERR[3]);
    
    LHS[4] =  BesselY(z, R_f_minus, KODE, &NZ, &LHS_IERR[4]);
    LHS[5] =  BesselJ(z, R_s_minus, KODE, &NZ, &LHS_IERR[5]);
    LHS[6] = DBesselY(z, R_f_minus, KODE, &NZ, &LHS_IERR[6]);
    LHS[7] = DBesselJ(z, R_s_minus, KODE, &NZ, &LHS_IERR[7]);
    
    RHS[0] =   BesselJ(z, R_f_plus, KODE, &NZ, &RHS_IERR[0]);
    RHS[1] =  HankelH2(z, R_c_plus, KODE, &NZ, &RHS_IERR[1]);
    RHS[2] =  DBesselJ(z, R_f_plus, KODE, &NZ, &RHS_IERR[2]);
    RHS[3] = DHankelH2(z, R_c_plus, KODE, &NZ, &RHS_IERR[3]);
    
    RHS[4] =   BesselY(z, R_f_plus, KODE, &NZ, &RHS_IERR[4]);
    RHS[5] =  HankelH2(z, R_c_plus, KODE, &NZ, &RHS_IERR[5]);
    RHS[6] =  DBesselY(z, R_f_plus, KODE, &NZ, &RHS_IERR[6]);
    RHS[7] = DHankelH2(z, R_c_plus, KODE, &NZ, &RHS_IERR[7]);
	
    if ( LHS_IERR[0] != 0 )
    {
	cerr << "--*-- BSlabMode::dispersion_eq(): In LHS[0]  IERR = " 
	     << LHS_IERR[0] << "--*--\n";
	exit(1);
    }
    else if ( LHS_IERR[1] != 0)
    {
	cerr << "--*-- BSlabMode::dispersion_eq(): In LHS[1]  IERR = " 
	     << LHS_IERR[1] << "--*--\n";
	exit(1);
    }
    else if ( LHS_IERR[2] != 0  )
    {
	cerr << "--*-- BSlabMode::dispersion_eq(): In LHS[2]  IERR = " 
	     << LHS_IERR[2] << "--*--\n";
	exit(1);
    }
    else if ( LHS_IERR[3] != 0)
    {
	cerr << "--*-- BSlabMode::dispersion_eq(): In LHS[3]  IERR = " 
	     << LHS_IERR[3] << "--*--\n";
	exit(1);
    }
    else if ( LHS_IERR[4] != 0)
    {
	cerr << "--*-- BSlabMode::dispersion_eq(): In LHS[4]  IERR = " 
	     << LHS_IERR[4] << "--*--\n";
	    exit(1);
    }
    else if ( LHS_IERR[5] != 0 )
    {
	cerr << "--*-- BSlabMode::dispersion_eq(): In LHS[5]  IERR = " 
	     << LHS_IERR[5] << "--*--\n";
	    exit(1);
    }
    else if ( LHS_IERR[6] != 0 )
    {
	cerr << "--*-- BSlabMode::dispersion_eq(): In LHS[6]  IERR = " 
	     << LHS_IERR[6] << "--*--\n";
	exit(1);
    }
    else if ( LHS_IERR[7] != 0 )
    {
	cerr << "--*-- BSlabMode::dispersion_eq(): In LHS[7]  IERR = " 
	     << LHS_IERR[7] << "--*--\n";
	    exit(1);
    }
    
    if ( RHS_IERR[0] != 0 )
    {
	cerr << "--*-- BSlabMode::dispersion_eq(): In RHS[0]  IERR = " 
	     << RHS_IERR[0] << "--*--\n";
	exit(1);
    }
    else if ( RHS_IERR[1]!= 0  )
    {
	cerr << "--*-- BSlabMode::dispersion_eq(): In RHS[1]  IERR = " 
	     << RHS_IERR[1] << "--*--\n";
	exit(1);
    }
    else if ( RHS_IERR[2] != 0 )
    {
	cerr << "--*-- BSlabMode::dispersion_eq(): In RHS[2]  IERR = " 
	     << RHS_IERR[2] << "--*--\n";
	exit(1);
    }
    else if ( RHS_IERR[3] != 0 )
    {
	cerr << "--*-- BSlabMode::dispersion_eq(): In RHS[3]  IERR = " 
	     << RHS_IERR[3] << "--*--\n";
	exit(1);
    }
    else if ( RHS_IERR[4] != 0 )
    {
	cerr << "--*-- BSlabMode::dispersion_eq(): In RHS[4]  IERR = " 
	     << RHS_IERR[4] << "--*--\n";
	exit(1);
    }
    else if ( RHS_IERR[5]!= 0  )
    {
	cerr << "--*-- BSlabMode::dispersion_eq(): In RHS[5]  IERR = " 
	     << RHS_IERR[5] << "--*--\n";
	exit(1);
    }
    else if ( RHS_IERR[6]!= 0  )
    {
	cerr << "--*-- BSlabMode::dispersion_eq(): In RHS[6]  IERR = " 
	     << RHS_IERR[6] << "--*--\n";
	exit(1);
    }
    else if ( RHS_IERR[7]!= 0  )
    {
	cerr << "--*-- BSlabMode::dispersion_eq(): In RHS[7]  IERR = " 
	     << RHS_IERR[7] << "--*--\n";
	exit(1);
    }
    
    if (pol == TE )
    {
	ans = ( LHS[0] * (n_s*LHS[3]) -  LHS[1] * (n_f*LHS[2]) ) *
	    ( RHS[4] * (n_c*RHS[7]) -  RHS[5] * (n_f*RHS[6]) ) -
	    ( LHS[4] * (n_s*LHS[7]) -  LHS[5] * (n_f*LHS[6]) ) *
	    ( RHS[0] * (n_c*RHS[3]) -  RHS[1] * (n_f*RHS[2]) );    
    }
    if (pol == TM )
    {
	ans = ( (n_f)*LHS[0] * LHS[3] -  (n_s)*LHS[1] * LHS[2] ) *
	    ( (n_f)*RHS[4] * RHS[7] -  (n_c)*RHS[5] * RHS[6] ) -
	    ( (n_f)*LHS[4] * LHS[7] -  (n_s)*LHS[5] * LHS[6] ) *
	    ( (n_f)*RHS[0] * RHS[3] -  (n_c)*RHS[1] * RHS[2] );
    }
    
    return ans;
    
};

//-------- combining dispersion eq. for TE and TM----end------

//secant method solver

/*
  Secant method error code:
  output : 0 (success), 1 (failure)
  exit_status code : 1 = rel. error < TOL and abs(f) < TOL
                     2 = f(root)    < TOL
		     3 = MAX_ITER reached
		     4 = Division by zero
*/
int BSlabMode::secant(
    complex<double> guess1, 
    complex<double> guess2,
    double tol, 
    int max_iter,
    complex<double> *root,
    int *iter_counter,
    int *exit_status)
{ 
    complex<double> guess_new ; 
    int counter=0;

    if ( abs(dispersion_eq(guess1)) <= tol )
    {
	*root = guess1;
	*iter_counter = counter;
	*exit_status = 2;
	return 0;
    }
    else if ( abs(dispersion_eq(guess2)) <=  tol )
    {
	*root = guess2;
	*iter_counter = counter;
	*exit_status = 2;
	return 0;
    }
    else
    { // generates next guess in secant fashion

	guess_new = guess1;

	while ( abs(dispersion_eq(guess_new)) > tol)
	{
//	    if (abs((guess1 - guess2)/guess1) <= tol )
	    if ( (abs( real(guess1)- real(guess2)) <= tol) && 
		 (abs( imag(guess1)- imag(guess2)) <= tol))
	    {
		*root= guess_new;
		*iter_counter = counter;
		*exit_status=1;
		return 1;
	    }
	    else if ( (real(dispersion_eq(guess1)) == real(dispersion_eq(guess2)))  && (imag(dispersion_eq(guess1)) == imag(dispersion_eq(guess2))))
	    {
		*root= guess_new;
		*iter_counter = counter;
		*exit_status = 4;
		return 1;
	    }
	    else
	    {

		guess_new = guess1 -
		    ((guess1 - guess2) / (dispersion_eq(guess1) - dispersion_eq(guess2)))*dispersion_eq(guess1);
	    }

	    counter++;

	    if ( counter >= max_iter)
	    {
		*root = guess_new;
		*iter_counter = counter;
		(abs(dispersion_eq(guess_new)) <= tol) ? *exit_status = 1 : *exit_status=3;
		return 1;
	    }

	    guess2 = guess1;
	    guess1= guess_new;

	}

	*root= guess_new;
	*iter_counter = counter;
	*exit_status = 1;
	return 0;

    }
   

};

//discretised version of field(FComp, r) 
// message = 0 => Don't show messages
bool BSlabMode::set_d_field(Interval rinterval_, int Nr_, int message)
{

    if (message != 0)
    {
	cerr <<"--*-- BSlabMode::set_d_field() --*--\n";

	if (set_d_field_flag == true)
	{
	    cerr << "      Currently fields are discretised on  ["
		 << rinterval.x0 <<", " << rinterval.x1 <<"] "
		 << "with "<< Nr << " points. \n";
	}
	
	cerr << "      Now fields will be discretised on  ["
	     << rinterval_.x0 <<", " << rinterval_.x1 <<"] "
	     << "with "<< Nr_ << " points. \n";

    }

   rinterval = rinterval_;
   Nr = Nr_;

   set_d_field_flag = false;
   
   d_field_ER = Cvector(Nr);
   d_field_EY = Cvector(Nr);
   d_field_EP = Cvector(Nr);
   d_field_HR = Cvector(Nr);
   d_field_HY = Cvector(Nr);
   d_field_HP = Cvector(Nr);

   hr = ( rinterval.x1 - rinterval.x0) / ((double)(Nr - 1));

    if (pol == TE)
    {
	for(int i=0; i< Nr; i++)
	{
	    d_field_EY(i) = field(EY, rinterval.x0 + ((double) i)*hr);
	    d_field_HP(i) = field(HP, rinterval.x0 + ((double) i)*hr);
	    d_field_HR(i) = field(HR, rinterval.x0 + ((double) i)*hr);
	    d_field_ER.init(CC0);
	    d_field_EP.init(CC0);
	    d_field_HY.init(CC0);
	}
    }
    else if (pol == TM)
    {
	for(int i=0; i< Nr; i++)
	{
	    d_field_EP(i) = field(EP, rinterval.x0 + ((double) i)*hr);
	    d_field_ER(i) = field(ER, rinterval.x0 + ((double) i)*hr);
	    d_field_HY(i) = field(HY, rinterval.x0 + ((double) i)*hr);
	    d_field_HR.init(CC0);
	    d_field_HP.init(CC0);
	    d_field_EY.init(CC0);
	}
    }

    set_d_field_flag = true;

    if (message != 0)
	cerr << "--*-- BSlabMode::set_d_field() leaving. --*--\n\n";

    return set_d_field_flag;
};

