/*
 * Circurs - Circular optical microresonators simulation tools
 * Kirankumar R. Hiremath (k.r.hiremath@ieee.org)
 * University of Twente, Department of Applied Mathematics 
 * P.O. Box 217, 7500AE Enschede, The Netherlands
 * (2005)
 */

/*
 * bslamode.h
 * bend slab mode definitions 
 * mode overlaps
 */

/*
 secant error codes
 0 => success
 1 => error

 exit status:
 1 => rel. error < TOL
 2 => f(root) < TOL
 3 => MAX_ITER reached
 4 => division by zero
*/

#ifndef BSLAMODE_H
#define BSLAMODE_H

#include"bessel.h"
#include<cmath>
#include<stdlib.h>
#include<cstdlib>
#include<iostream>
#include<complex>
#include<iomanip>
#include"bslamvis.h"

#include "complex.h"

/* To include required code for straight waveguide */
#include "metric.h"

using namespace std;

/* planar bend mode */
class BSlabMode
{
 
public:
	// the corresponding bend waveguide  
	BSlabWaveguide bend;
	// Polarization
	Polarization pol;
	// propagation constant;
	Complex gamma;
	// phase propagation constant;
	double beta;
	// attenuation constant;
	double alpha;
	// vacuum wavenumber
	double k0;
	// effective mode index = gamma/k0
	Complex neff;

	complex <double> normalization_factor;
	double norm; // = tot power

	//constructor
	BSlabMode();
	BSlabMode( BSlabWaveguide BS);
	BSlabMode( BSlabWaveguide BS, Polarization bpol);
	BSlabMode( BSlabWaveguide BS, Polarization bpol, Complex bgamma);
		   

	// sets field coefficients of mode profile (E_y or H_y)
	void set_field_coefficients();

	// field values at radius r, complex mode profiles 
	// cp:  ER, EY, EP, HR, HY, HP, SR, SY, SP 
	Complex field(Fcomp cp, double r);//const;

	// store num equidistant values of component cp between r0 and r1
        // in a vector
	// cp:  ER, EY, EP, HR, HY, HP, SR, SY, SP */
        Cvector fieldvec(Fcomp cp, int num, double r0, double r1);//const;

	// 'realized' field values at radius r 
	// cp:  ER, EY, EP, HR, HY, HP, SR, SY, SP
	// foa: MOD, ORG, REP, IMP, SQR  
	double field(Fcomp cp, Afo foa, double r);//const;

	// field values at radius r and angle p, complex mode profiles 
	// cp: EX - HZ, ER, EP, HR, HP, SX, SY, SZ, SP, SR
	// note the coordinate system convention!
	Complex field(Fcomp cp, double r, double p);//const;

	// field values at position x, z, complex mode profiles
	// cp: EX - HZ, ER, EP, HR, HP, SX, SY, SZ, SP, SR */
	// note the coordinate system convention!
	Complex field_cart(Fcomp cp, double x, double z);//const;


	
	// azimuthal component of the Poynting vector
	double totpower(Interval xinterval, double z, int xnodes);//const;

	//power for modes in cartesian system!
	// analytic power evaluation
	double totpower(Interval xinterval); 

	// power on infinite interval at an angle theta
	double totpower(double theta);
	    
	// normalize mode to totpower() == nrm
       	void normalize(double nrm);

	
	// data file: num equidistant values of component cp between r0 and r1
   	// cp:  ER, EY, EP, HR, HY, HP, SR, SY, SP */
	void profiledata(Fcomp cp, int num, double r0, double r1,
                         char ext0, char ext1);//const;

	// 'realized' field profile -> .xyf - file  
	// cp:  ER, EY, EP, HR, HY, HP, SR, SY, SP
	// foa: MOD, ORG, REP, IMP, SQR  
	// fac: output factor (typically 1.0 or -1.0)
	// disp: output window on the radial axis 
	// ext0, ext1: filename id characters
	// num: number of points in output mesh
	void writeprofile(Fcomp cp, Afo foa, double fac, Interval disp,
			  char ext0, char ext1, int num);//const;

	// field profile -> MATLAB .m file  
	// cp: EX - HZ, ER, EP, HR, HP, SX, SY, SZ, SP, SR
	// foa: MOD, ORG, REP, IMP, SQR  
	// fac: output factor (typically 1.0 or -1.0)  
	// disp: output region on the x axis
	// np: number of points in output mesh
	// ext0, ext1: filename id characters
	// pltype: 'L': geometry information + plot commands (default)
	//         'V': field, mesh, and plot comand, 
        //              to be included into *L.m 
	void mfile(Fcomp cp, Afo foa, double fac, 
                   Interval disp, int np, char ext0, 
                   char ext1, char pltype);//const;

	// complex mode profile -> MATLAB .m file  
	// cp: EX - HZ, ER, EP, HR, HP, SX, SY, SZ, SP, SR
	// fac: output factor (typically 1.0 or -1.0)  
	// disp: output region on the x axis
	// np: number of points in output mesh
	// ext0, ext1: filename id characters
	// pltype: 'r': real + imaginary part  
	//         'a': amplitude + phase 
	void mfile(Fcomp cp, double fac, 
                   Interval disp, int np, char ext0, 
                   char ext1, char pltype);//const;

	// output to file with default name
	void write_def(char ext0, char ext1);
	// input from file with default name
	void read_def(Polarization p, char ext0, char ext1);

	// output to file dat
	void write(FILE *dat);
	// input from file dat
	void read(FILE *dat);

	//dispersion eq.
	complex<double> dispersion_eq(complex<double> z);
  
	// to find complex root with secant method 
	int secant(
	    complex<double> guess1, 
	    complex<double> guess2, 
	    double tol, 
	    int max_iter, 
	    complex<double> *root, 
	    int *iter_counter, 
	    int *exit_status);  
	
	
	complex<double> C_s;  // coeff. of mode profile in substrate region (J)
	complex<double> C_f;  // coeff. of mode profile  in film region  (J)
	complex<double> D_f;  // coeff. of mode profile  in film region  (Y)
	complex<double> D_c;  // coeff. of mode profile  in cover region (H_2)

	//default value is 0 and if function set_field_coefficients() is called
	// then it is set to 1. 
	int set_field_coefficients_flag;  

	// rinterval is interval on which field values are discretised
	Interval rinterval;
	int Nr;
	double hr;

	// This function discretizes the field profile in the radial direction
	// in the Interval "rinterval_" into Nr_.
	// Used to speed up the computation of cme::coverlap() :-)
	// message = 0 => Don't show messages
	bool set_d_field(Interval rinterval_, int Nr_, int message);
  

	// discretised field values
	Cvector d_field_ER;
	Cvector d_field_EY;
	Cvector d_field_EP;
	Cvector d_field_HR;
	Cvector d_field_HY;
	Cvector d_field_HP;

	 private:

	// default value is false meaning no discretised field available,
	// if set_d_field() is called it is set to true.
	bool set_d_field_flag;
    
};

#endif  // BSLAMODE_H

