/*
 * Circurs - Circular optical microresonators simulation tools
 * Kirankumar R. Hiremath (k.r.hiremath@ieee.org)
 * University of Twente, Department of Applied Mathematics 
 * P.O. Box 217, 7500AE Enschede, The Netherlands
 * (2005)
 */

/*
 * bslams.c
 * bend slab mode solver 
 */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include <fstream>

#include"bslams.h"

#define ALPHA_MIN 1e-14
#define GMAX 100 //maximum 

/* default analysis parameters */
BSLAMS_Parameters::BSLAMS_Parameters()
{
    btol = 1e-14;
    iter = 200;
}

/* error message */
void bslamserror(char *msg)
{
	fprintf(stderr, "\nbslams: ERROR: %s\n", msg);
	exit(1);
}

/* ................................................................... */
/* BSLAMS mode analysis procedures */

/* complete mode analysis, returns number of found modes */
int BSLAMS_findfundmode(BSlabWaveguide bend,   // the analysed structure
                      Polarization pol,  // polarization type: TE, TM
		      BSLAMS_Parameters par, // further SLAMS parameters
                      BSlabModeArray& marr)  // found modes 
{
	BSlabMode m(bend, pol);
	
	// Finding neff for corresponding SWG
	ModeArray smode;
	
	int total_modes = modeanalysis( m.bend.wg, m.pol, smode, 1);
    
	if (total_modes == 0)
	{
	    cerr << "Structure doesn't support any modes. \n";
	    exit(1);
	}
	else if ( total_modes > 1)
	{
	    cerr << "Structure is multimode. Bend mode can not handle it.\n";
	    exit(1);
	}


	// Controlling initial guess

	complex<double> g, guess1, guess2;
	double b= smode(0).beta;
	bool correct=false;

	if ( (pol == TE ) || ( pol == TM ))
	{
	    while ( correct == false)
	    {
		if (m.bend.R <= 50)
		  {
		      g = complex<double> (b,-1.0e-1);
		      guess1 = complex<double> (0.99*b, -2.0e-1);
		      guess2 = g;
		  }
		else if (m.bend.R <= 100)
		  {
		      g = complex<double> (b,-1.0e-5);
		      guess1 = complex<double> (0.99*b, -2.0e-5);
		      guess2 = g;
		  }
		else
		{
		    g = complex<double> (b,-1.0e-15);
		    guess1 = complex<double> (0.99*b, -2.0e-15);
		    guess2 = g;
		}

		guess1 = guess1*m.bend.R;
		guess2 = guess2*m.bend.R;

		complex<double> cpp_gamma;
		int  success,iter_counter, exit_status;

		success = m.secant(guess1, guess2, 
				   par.btol, 
				   par.iter, 
				   &cpp_gamma, 
				   &iter_counter, 
				   &exit_status); 

		cpp_gamma = cpp_gamma/m.bend.R;
	

		//transforming results in class variables
		m.gamma = Complex(cpp_gamma.real(), cpp_gamma.imag());
		m.alpha = -m.gamma.im; 
		m.beta  =  m.gamma.re;
		m.neff  =  m.gamma / m.k0;
		
		//checking results
		if ( (m.alpha < 0 ) && (abs(m.alpha) > ALPHA_MIN) )
		{
		    cerr<< "\nWrong alpha.  \n";
		}
		else if ( abs(m.alpha) <= ALPHA_MIN )
		{
		    cerr <<"\n---- Alpha is less than 1e-14."
			 <<" Setting it zero.---\n";
		    m.alpha = 0.0;
		}
		else if ( (m.bend.wg.defaultneffmin() > m.neff.re) ||   (m.bend.wg.defaultneffmax() < m.neff.re))  
		{
		    cerr<< "n_eff.re : " << m.neff.re << "\nWrong n_eff. \n"
			<< "n_min : " << m.bend.wg.defaultneffmin() << "\t"
			<< "n_max : " << m.bend.wg.defaultneffmax() << "\n";
		}
		else
		{
		    correct = true;
		}

      cerr<<"----------Info from secant routine ----------"
      <<"\nPolarization : " << pol
      <<"\nSWG, n_eff   : " << smode(0).neff
      <<"\nSWG, beta    : " << smode(0).beta
      <<"\nR            : " << m.bend.R
      <<"\n"
      <<"\nsuccess      : " << success
      <<"\niter_counter : " << iter_counter
      <<"\nexit_status  : " << exit_status
      <<"\n"
      <<"\nGamma        : " << m.gamma.re << ", " << m.gamma.im
      <<"\nN_eff        : " << m.neff.re  << ", " << m.neff.im
      <<"\n---------------------------------------------\n"
      << endl;		

    marr.add(m);
		return 1; 
	    }
	}
	else 
	{
	    // wish : use slabmodeerror() for error display :-)
	    cerr << "Unknown polarization \n";
	    return 1;
	}


	return marr.num;
}

/* complete mode analysis, returns number of found modes */
int bend_mode_analysis(
    BSlabWaveguide bend,  // the analysed structure
    Polarization pol,     // polarization type: TE, TM
    double nmin,          // interval investigated for
    double nmax,          //   effective mode indices 
                          //   nmin = 0, nmax = 0: 
                          //   use default values
    BSLAMS_Parameters par, // further SLAMS parameters
    BSlabModeArray& marr)  // found modes 
{

	BSlabMode m(bend, pol);
	Complex g;
	char pc = ' ';

	if(nmin <= 0.5) nmin = m.bend.wg.defaultneffmin();
	if(nmax <= 0.5)	nmax = m.bend.wg.defaultneffmax();

	fprintf(stderr, 
		"\n------------- BSLAMS - MA --------------------- '03 ---\n");
	switch(m.pol)
	{
	case TE: fprintf(stderr, "TE  "); pc = 'E'; break;
	case TM: fprintf(stderr, "TM  "); pc = 'M'; break;
	}
	fprintf(stderr, "Btol: %g  ", par.btol);
	fprintf(stderr, "n_eff: ( %.10g, %.10g )\n", nmin, nmax);
	fprintf(stderr, 
		"------------------------------------------------------\n");
	fprintf(stderr, "    R : %.10g\n", m.bend.R);
	fprintf(stderr, "    Nx: %d\n", m.bend.wg.nx);
	fprintf(stderr, "    Hx: ");
	for(int j=0; j<=m.bend.wg.nx; ++j) 
		fprintf(stderr, "%6.10g  ", m.bend.wg.hx(j));
	fprintf(stderr, "\n");
	fprintf(stderr, "     N: ");
	for(int j=0; j<=m.bend.wg.nx+1; ++j) 
	{
		fprintf(stderr, "%6.10g  ", m.bend.wg.n(j));
	}
	fprintf(stderr, "\n");
	fprintf(stderr, "Lambda: %.10g, K0: %g\n", m.bend.wg.lambda, m.k0);
	fprintf(stderr, 
		"----------------------------------------------------\n");
	fprintf(stderr, "\n");

	if(nmin >= (nmax-par.btol)) return 0; 
	if(pol == TM) bslamserror("TM mode analysis is not implemented");


	// ... mode solver routines ... 
	g = Complex(m.k0*(nmax*0.8+nmin*0.2), 1.0e-5);
	// ... mode solver routines ... 

	m.gamma = g;
	m.beta = g.re;
	m.alpha = g.im;
	m.neff = m.gamma/m.k0;

	marr.add(m);
	fprintf(stderr, ".");

	// ... mode solver routines ... 
	g = Complex(m.k0*(nmax*0.7+nmin*0.3), 1.0e-5);
	// ... mode solver routines ... 

	m.gamma = g;
	m.beta = g.re;
	m.alpha = g.im;
	m.neff = m.gamma/m.k0;

	marr.add(m);
	fprintf(stderr, ".");

	// ... mode solver routines ... 
	g = Complex(m.k0*(nmax*0.6+nmin*0.4), 1.0e-5);
	// ... mode solver routines ... 

	m.gamma = g;
	m.beta = g.re;
	m.alpha = g.im;
	m.neff = m.gamma/m.k0;

	marr.add(m);
	fprintf(stderr, ".");

	fprintf(stderr, "\n--> %d T%c modes.\n", marr.num, pc); 
	fprintf(stderr, "\n");

	return marr.num;
}

/* a domain in the complex plan defined by lower left point (b0, a0) 
   and upper right point (b1, a1)
*/
int bend_mode_analysis(
  BSlabWaveguide bend,   // the analysed structure
  Polarization pol,  // polarization type: TE, TM
  double b0, double b1, int Nb, 
  double a0, double a1, int Na, 
  BSLAMS_Parameters par, // further SLAMS parameters
  BSlabModeArray& marr)  // found modes 
{

    BSlabMode m(bend, pol);
    char pc = ' ';

    fprintf(stderr, 
	    "\n------------- BSLAMS - MA --------------------- '03 ---\n");
	switch(m.pol)
	{
	case TE: fprintf(stderr, "TE  "); pc = 'E'; break;
	case TM: fprintf(stderr, "TM  "); pc = 'M'; break;
	}
	fprintf(stderr, "Btol: %g  ", par.btol);
	fprintf(stderr, 
		"\n------------------------------------------------------\n");
	fprintf(stderr, "    R : %.10g\n", m.bend.R);
	fprintf(stderr, "    Nx: %d\n", m.bend.wg.nx);
	fprintf(stderr, "    Hx: ");
	for(int j=0; j<=m.bend.wg.nx; ++j) 
		fprintf(stderr, "%6.10g  ", m.bend.wg.hx(j));
	fprintf(stderr, "\n");
	fprintf(stderr, "     N: ");
	for(int j=0; j<=m.bend.wg.nx+1; ++j) 
	{
		fprintf(stderr, "%6.10g  ", m.bend.wg.n(j));
	}
	fprintf(stderr, "\n");
	fprintf(stderr, "Lambda: %.10g, K0: %g\n", m.bend.wg.lambda, m.k0);
	cerr <<  "Search domain: beta=[ " << b0 << ", " << b1 << "] "
	     << "alpha = [ " << a0 << ", " << a1 << "]\n"; 
	fprintf(stderr, 
		"----------------------------------------------------\n");
	fprintf(stderr, "\n");

	double da = (a1 - a0)/(Na -1), db = (b1 - b0)/(Nb - 1);
	complex <double> cpp_gamma;
	complex <double> g[(Na - 1)*(Nb - 1)];

	int g_t = 0; //total gamma
	int g_c = 0; // gamma with correct alpha and beta 
	int g_d = 0; // no. of distinct roots;

// find roots
	for (int i_a=0; i_a < (Na -1); i_a++)
	{
	    for (int i_b=0; i_b < (Nb -1); i_b++)
	    {
		complex <double> g0( b0 + i_b*db, a0 + i_a*da);
		complex <double> g1( b0 + (i_b + 1)*db, a0 + (i_a+1)*da);
		int  success,iter_counter, exit_status;

		success = m.secant(g0*bend.R, g1*bend.R, par.btol, par.iter, 
				   &cpp_gamma, &iter_counter, &exit_status);
		
		g_t++;
		
// store the roots correct sign for alpha
		if ( (success != 2 ) && // root is correct 
		     ( real(cpp_gamma) > 0) && (imag(cpp_gamma) < 0 ) )
		{
		    g_c++;

		    complex <double> root = cpp_gamma / bend.R;
		    
		    if ( g_d == 0) 
		    { // store the root
			g[g_d] = root;
			g_d++;
		    }
		    else
		    {
			int old_g_d = g_d;
			bool new_root = true;

			for (int j=0; j< old_g_d; j++)
			{
			    if ( (abs( real(root)- real(g[j])) <= 5e-6) && 
				 (abs( imag(root)- imag(g[j])) <= 5e-6))
			    { new_root = false;}
			    
			}
			
			if (new_root == true)
			{ 
			    g[g_d] = root;
			    g_d++;
			}
		    }
		}
		else
		{
//*		    cerr << " wrong alpha.\n";
		}
	    }
	}
	
	cerr << "Total roots                  : " << g_t << "\n"
	     << "Roots with correct alpha/beta: " << g_c << "\n"
	     << "Distinct roots               : " << g_d << "\n";

// sorting the roots with correct sign for alpha asending order of real part
// methos is based on shell sorting. See Numerical Recepies (some error?).

  int i, j, inc;
  complex <double> temp;

  inc = 3;
  while (inc > 0)
  {
      for (i=0; i < g_d; i++)  //note i < g_d is important
      {
	  j = i;
	  temp = g[i];
	  while ((j >= inc) && ( real(g[j-inc]) <= real(temp)))
	  {
	      g[j] = g[j - inc];
	      j = j - inc;
	  }
	  g[j] = temp;
      }
      if (inc/2 != 0)
	  inc = inc/2;
      else if (inc == 1)
	  inc = 0;
      else
	  inc = 1;
  }

// sorting finished

  cerr << "Sorted roots [gamma] | [gamma/k] :\n";

  for (int i=0; i < g_d; i++) 
  {
      //transforming results in class variables
      m.gamma = Complex(real(g[i]), imag(g[i]));
      m.alpha = -m.gamma.im; 
      m.beta  =  m.gamma.re;
      m.neff  =  m.gamma / m.k0;

      cerr << m.gamma.re << "\t" << m.gamma.im << "\t" 
	   << m.neff.re <<"\t" << m.neff.im <<"\n";
      
      marr.add(m);
  }
  
  marr.num = g_d;


  fprintf(stderr, "\n--> %d T%c modes.\n", marr.num, pc); 
  fprintf(stderr, "\n");

  return marr.num;
}

//-------------------------------------------------------------
// interpolated_gamms()=Gives approx. propagation consts 
// by quadratic /linear interpolation for wavelengths between nodal wavelengths 
// by diving it into Nl points. 
// Nodal values of gammas are given in the file "input_gamma" with
// Data structure: first line: no. of BWG modes 
// then for other lines: first entry wavelength then real and imag part of gamma
// from fundamental mode to higher order modes

Dmatrix interpolated_gamma(char *input_gamma,
			    char *output_file,
			    char F,      // interpolation option
			    int Nl,      // No. of points
			    int storage_flag)   // where to store
{
    ifstream infile(input_gamma, ios::in);

    if (!infile)
    {
	cerr <<"--*--interpolated_gamma(). "
	    <<"Unable to open file " << input_gamma <<".--*--\n";
	exit(1);
    }

    int tm; // no. of bent modes
    double Rad; // radius of the bent waveguide

    infile >> Rad >> tm;

    tm = tm*2;

    Dvector l;
    Dmatrix g;

    if ( F == 'Q')
    {
	l = Dvector(3);
	g = Dmatrix(3, tm);
    }
    else if ( F == 'L')
    {
	l = Dvector(2);
	g = Dmatrix(2, tm);
    }
    

    // fill l and g 
    for (int i=0; i < l.nel; i++)
    {
	infile >> l(i);
	for (int j=0; j < tm; j++) infile >> g(i,j);
    }

    Dmatrix lg(Nl, tm +1); // matrix of lambda and beta, alpha values
//-------------------------------------------------------------
// interpolating polynomial:a*x^2 + b*x + c or a*x + b
// interpolat each beta and alpha seperately for each mode

    if ( F == 'Q')  
    {
	double a[g.c], b[g.c], c[g.c]; //unknown coefficients
	Dvector y(l.nel);
	
	for (int i=0; i < g.c; i++)
	{
	    for (int j=0; j< l.nel; j++) 
	    {
		y(j) = g(j,i);
//		cerr << y(j) << "\t";
	    }
//	    cerr << "\n";
	    
	Dvector Qans = Qfit(l, y);
	
	a[i] = Qans(0);
	b[i] = Qans(1);
	c[i] = Qans(2);
	}

	double hl = (l(2) - l(0))/(Nl - 1);

	for (int i =0; i < Nl; i++)
	{
	    lg(i,0) = l(0) + i*hl;

	    for (int j=0; j< g.c; j++)
		lg(i,j+1) = a[j]*lg(i,0)*lg(i,0) + b[j]*lg(i,0) + c[j];
	}
    }
    else if ( F == 'L')
    {
	double a[g.c], b[g.c]; //unknown coefficients
	Dvector y(l.nel);
	
	for (int i=0; i < g.c; i++)
	{
	    for (int j=0; j< l.nel; j++) y(j) = g(j,i);
	    
	Dvector Lans = Lfit(l, y);
	
	a[i] = Lans(0);
	b[i] = Lans(1);
	}
	
	double hl = (l(1) - l(0))/(Nl - 1);

	for (int i =0; i < Nl; i++)
	{
	    lg(i,0) = l(0) + i*hl;
	    for (int j=0; j< g.c; j++) lg(i,j+1) = a[j]*lg(i,0) + b[j];
	}
    }

    if (storage_flag == 0)   
    {// don't write to file. Return values.
	return lg;
    }
    else if (storage_flag == 1) 
    {//  write it to file only
	ofstream outfile(output_file, ios::out);
 	outfile << Rad << "\n" << tm/2;

	for(int i=0; i < lg.r; i++)
	{
	    outfile << "\n" << lg(i,0);
	    for (int j=1; j < lg.c; j++)  outfile << "\t" << lg(i,j);
	}
	
	outfile.close();
	
	cerr << "--*-- From interpolated_gamma():"
	    << output_file <<" file created.--*--\n"; 
	
	Dmatrix tmp(1,1);
	tmp(0,0) = 0.0;
	
	return tmp;
    }

    
    // write to file and also return values.

    ofstream outfile(output_file, ios::out);
    outfile << Rad << "\n" << tm/2;
    
    for(int i=0; i < lg.r; i++)
    {
	outfile << "\n"<< lg(i,0);
	for (int j=0; j < lg.c; j++)  outfile << "\t" << lg(i,j) ;
    }
    
    outfile.close();
    
    cerr << "--*-- From interpolated_gamma():"
	 << output_file <<" file created.--*--\n"; 
    
    return lg;
}
//-------------------------------------------------------------
// Qfit() : Gives coefficients for quadratic interpolation polynomial
// interpolation polynomial: a*x^2 + b*x + c
// return Dvector=[a b c]
Dvector Qfit(Dvector x, Dvector y)
{
    Dmatrix A(3,3);

    for(int i=0; i<3; i++)
    {
	A(i,0) = x(i)*x(i);
	A(i,1) = x(i);
	A(i,2) = 1.0;
    }

    A.inverse();

    return mult(A,y);

    
}

//----------------------------------------------------------------
// Lfit(): Gives coefficients for linear interpolation polynomial 
// interpolation polynomial: a*x + b
// return Dvector=[a b]
Dvector Lfit(Dvector x, Dvector y)
{
    Dmatrix A(2,2);

    for(int i=0; i<2; i++)
    {
	A(i,0) = x(i);
	A(i,1) = 1.0;
    }

    A.inverse();

    return mult(A,y);
    
}
//------------------------------------------------------------------

// interpolated_reduced_S()=Gives approx. reduced scattering matrix 
// by quadratic /linear interpolation for wavelengths between nodal wavelengths 
// by diving it into Nl points. 
// Nodal values of reduced S are given in the file "input_Sr" with
// Data structure: first line: no. of BWG modes + SWG modes
// then for other lines: first entry wavelength then real and imag part of Sr

Dmatrix interpolated_reduced_S(char *input_Sr,
			       char *output_file,
			       char F,      // interpolation option
			       int Nl,      // No. of points
			       int storage_flag)   // where to store
{

    ifstream infile(input_Sr, ios::in);

    if (!infile)
    {
	cerr <<"--*--interpolated_reduced_S(). "
	    <<"Unable to open file " << input_Sr <<".--*--\n";
	exit(1);
    }

    int Nbm; // number of BWG modes
    int Nsm; // number of SWG modes
    int tm; // no. BWG modes + SWG modes

    double z0, z1;

    infile >> Nbm >> Nsm;
    infile >> z0 >> z1;

    tm = (Nbm + Nsm)*(Nbm + Nsm)*2;

    Dvector l;
    Dmatrix Sr;

    if ( F == 'Q')
    {
	l = Dvector(3);
	Sr = Dmatrix(3, tm);
    }
    else if ( F == 'L')
    {
	l = Dvector(2);
	Sr = Dmatrix(2, tm);
    }

    // fill l and Sr 

    for (int i=0; i < l.nel; i++)
    {
	infile >> l(i);
	for (int j=0; j < tm; j++) infile >> Sr(i,j);
    }

    Dmatrix lSr(Nl, tm +1); // matrix of lambda and Sr values

//-------------------------------------------------------------
// interpolating polynomial:a*x^2 + b*x + c or a*x + b
// interpolat each beta and alpha seperately for each mode

    if ( F == 'Q')  
    {
	double a[Sr.c], b[Sr.c], c[Sr.c]; //unknown coefficients
	Dvector y(l.nel);
	
	for (int i=0; i < Sr.c; i++)
	{
	    for (int j=0; j< l.nel; j++) 
	    {
		y(j) = Sr(j,i);
//		cerr << y(j) << "\t";
	    }
//	    cerr << "\n";
	    
	Dvector Qans = Qfit(l, y);
	
	a[i] = Qans(0);
	b[i] = Qans(1);
	c[i] = Qans(2);
	}

	double hl = (l(2) - l(0))/(Nl - 1);

	for (int i =0; i < Nl; i++)
	{
	    lSr(i,0) = l(0) + i*hl;

	    for (int j=0; j< Sr.c; j++)
		lSr(i,j+1) = a[j]*lSr(i,0)*lSr(i,0) + b[j]*lSr(i,0) + c[j];
	}
    }
    else if ( F == 'L')
    {
	double a[Sr.c], b[Sr.c]; //unknown coefficients
	Dvector y(l.nel);
	
	for (int i=0; i < Sr.c; i++)
	{
	    for (int j=0; j< l.nel; j++) 
	    {
		y(j) = Sr(j,i);
//		cerr << y(j) << "\t";
	    }
//	    cerr << "\n";
	    
	Dvector Lans = Lfit(l, y);
	
	a[i] = Lans(0);
	b[i] = Lans(1);
	}
	
	double hl = (l(1) - l(0))/(Nl - 1);

	for (int i =0; i < Nl; i++)
	{
	    lSr(i,0) = l(0) + i*hl;
	    for (int j=0; j< Sr.c; j++) lSr(i,j+1) = a[j]*lSr(i,0) + b[j];
	}
    }


    if (storage_flag == 0)   
    {// don't write to file. Return values.
	return lSr;
    }
    else if (storage_flag == 1) 
    {//  write it to file only
	ofstream outfile(output_file, ios::out);

	outfile << Nbm << "\t" << Nsm <<"\n"
		<< z0 << "\t" << z1;
	
	for(int i=0; i < lSr.r; i++)
	{
	    outfile << "\n" << lSr(i,0);
	    for (int j=1; j < lSr.c; j++)  outfile << "\t" << lSr(i,j) ;
	}
	
	outfile.close();
	
	cerr << "--*-- From interpolated_reduced_S(): "
	     << output_file << " file created.--*--\n"; 
	
	Dmatrix tmp(1,1);
	tmp(0,0) = 0.0;
	
	return tmp;
    }

    // write to file and also return values.
    ofstream outfile(output_file, ios::out);
    
    outfile << Nbm << "\n" << Nsm <<"\n"
	    << z0 << "\t" << z1;
    
    for(int i=0; i < lSr.r; i++)
    {
	outfile << "\n" << lSr(i,0);
	for (int j=1; j < lSr.c; j++)  outfile << "\t" << lSr(i,j);
    }
    
    outfile.close();
    infile.close();

    cerr << "--*-- From interpolated_reduced_S():"
	 << output_file << " file created.--*--\n"; 
    
    return lSr;
}

//-------------------------------------------------------------
// interpolated_beta()=Gives approx. propagation consts for Straight waveguide
// by quadratic /linear interpolation for wavelengths between nodal 
// wavelengths  by diving it into Nl points. 
// Nodal values of betas are given in the file '*input_beta' with
// Data structure: first line: no. of SWG modes
// 2'nd line: Initial amps of modes (Needed for microresonator calculations) 
// then for other lines: first entry wavelength then beta values(s)
// from fundamental mode to higher order modes

Dmatrix interpolated_beta(char *input_beta,
			  char *output_file,
			  char F,      // interpolation option
			  int Nl,      // No. of points
			  int storage_flag)   // where to store
{
    ifstream infile(input_beta, ios::in);

    if (!infile)
    {
	cerr <<"--*--interpolated_beta(). "
	    <<"Unable to open file " << input_beta <<".--*--\n";
	exit(1);
    }

    int tm; // no. of bent modes

    infile >> tm;

    Cvector Iamp(tm);

    // fill the inital amps
    for (int i=0; i < tm; i++) infile >> Iamp(i).re >> Iamp(i).im;

    Dvector l;
    Dmatrix g;

    if ( F == 'Q')
    {
	l = Dvector(3);
	g = Dmatrix(3, tm);
    }
    else if ( F == 'L')
    {
	l = Dvector(2);
	g = Dmatrix(2, tm);
    }
    

    // fill l and g 
    for (int i=0; i < l.nel; i++)
    {
	infile >> l(i);
	for (int j=0; j < tm; j++) infile >> g(i,j);
    }

    Dmatrix lg(Nl, tm +1); // matrix of lambda and beta values
//-------------------------------------------------------------
// interpolating polynomial:a*x^2 + b*x + c or a*x + b
// interpolat each beta and alpha seperately for each mode

    if ( F == 'Q')  
    {
	double a[g.c], b[g.c], c[g.c]; //unknown coefficients
	Dvector y(l.nel);
	
	for (int i=0; i < g.c; i++)
	{
	    for (int j=0; j< l.nel; j++) 
	    {
		y(j) = g(j,i);
	    }
	    
	Dvector Qans = Qfit(l, y);
	
	a[i] = Qans(0);
	b[i] = Qans(1);
	c[i] = Qans(2);
	}

	double hl = (l(2) - l(0))/(Nl - 1);

	for (int i =0; i < Nl; i++)
	{
	    lg(i,0) = l(0) + i*hl;

	    for (int j=0; j< g.c; j++)
		lg(i,j+1) = a[j]*lg(i,0)*lg(i,0) + b[j]*lg(i,0) + c[j];
	}
    }
    else if ( F == 'L')
    {
	double a[g.c], b[g.c]; //unknown coefficients
	Dvector y(l.nel);
	
	for (int i=0; i < g.c; i++)
	{
	    for (int j=0; j< l.nel; j++) 
	    {
		y(j) = g(j,i);
//		cerr << y(j) << "\t";
	    }
//	    cerr << "\n";
	    
	Dvector Lans = Lfit(l, y);
	
	a[i] = Lans(0);
	b[i] = Lans(1);
	}
	
	double hl = (l(1) - l(0))/(Nl - 1);

	for (int i =0; i < Nl; i++)
	{
	    lg(i,0) = l(0) + i*hl;
	    for (int j=0; j< g.c; j++) lg(i,j+1) = a[j]*lg(i,0) + b[j];
	}
    }

    if (storage_flag == 0)   
    {// don't write to file. Return values.
	return lg;
    }
    else if (storage_flag == 1) 
    {//  write it to file only
	ofstream outfile(output_file, ios::out);

	outfile << tm <<"\n";
	
	// fill the inital amps
	for (int i=0; i < tm; i++) 
	    outfile << Iamp(i).re  <<"\t" << Iamp(i).im <<"\t";

	for(int i=0; i < lg.r; i++)
	{
	    outfile << "\n" << lg(i,0);
	    for (int j=1; j < lg.c; j++)  outfile << "\t" << lg(i,j);
	}
	
	outfile.close();
	
	cerr << "--*-- From interpolated_beta():"
	    << output_file <<" file created.--*--\n"; 
	
	Dmatrix tmp(1,1);
	tmp(0,0) = 0.0;
	
	return tmp;
    }

    // write to file and also return values.
    ofstream outfile(output_file, ios::out);
    outfile << tm <<"\n";
    
    // fill the inital amps
    for (int i=0; i < tm; i++) 
	outfile << Iamp(i).re  <<"\t" << Iamp(i).im <<"\t";
    
    
    for(int i=0; i < lg.r; i++)
    {
	outfile << "\n"<< lg(i,0);
	for (int j=0; j < lg.c; j++)  outfile << "\t" << lg(i,j) ;
    }
    
    outfile.close();
    
    cerr << "--*-- From interpolated_beta():"
	 << output_file <<" file created.--*--\n"; 
    
    return lg;
}
//-------------------------------------------------------------
