/*
 * Circurs - Circular optical microresonators simulation tools
 * Kirankumar R. Hiremath (k.r.hiremath@ieee.org)
 * University of Twente, Department of Applied Mathematics 
 * P.O. Box 217, 7500AE Enschede, The Netherlands
 * (2005)
 */

/*
 * bslams.h
 * bend slab mode solver 
 */


#ifndef BSLAMS_H
#define BSLAMS_H


#include"bslawaveg.h"
#include"bslamode.h"
#include"bslamarr.h"


/* For straight waveguide related stuff */
#include "metric.h"

/* mode solver options */
class BSLAMS_Parameters 
{
public:
    // relative tolerance for the fixing the propagation constants
    double btol;  // tolerance
    int iter;     //maximum no. of iterations
    
    // set default values
    BSLAMS_Parameters();
};

/* ................................................................... */
/* find the fundamental mode of the bend waveguide */
int BSLAMS_findfundmode(BSlabWaveguide bend,   // the analysed structure 
			Polarization pol,  // polarization type: TE, TM
			BSLAMS_Parameters par,// further mode solver parameters
			BSlabModeArray& mode); // found modes 


/* complete mode analysis, returns number of found modes */
int BSLAMS_modeanalysis(BSlabWaveguide bend,   // the analysed structure 
			Polarization pol,  // polarization type: TE, TM
			double nmin,           // interval investigated for
			double nmax,           //   effective mode indices 
		                             //   nmin = 0, nmax = 0:
                                             //   use default values      
			BSLAMS_Parameters par, // further mode solver parameters
			BSlabModeArray& mode); // found modes 

/* a domain in the complex plane defined by lower left point (b0, a0) 
   and upper right point (b1, a1).
*/
int bend_mode_analysis(
  BSlabWaveguide bend,   // the analysed structure
  Polarization pol,  // polarization type: TE, TM
  double b0, double b1, int Nb, 
  double a0, double a1, int Na, 
  BSLAMS_Parameters par, // further BSLAMS parameters
  BSlabModeArray& marr);  // found modes 


// Qfit() : Gives coefficients for quadratic interpolation polynomial 
Dvector Qfit(Dvector x, Dvector y);

// Lfit(): Gives coefficients for linear interpolation polynomial 
Dvector Lfit(Dvector x, Dvector y);

// interpolated_gammas()=Gives approx. propagation consts by quadratic /linear 
// interpolation for wavelengths between nodal wavelengths by diving it 
// into Nl points. 
// nodal values are REQUIRED in the ascending order!
Dmatrix interpolated_gamma(char *input_gamma,
			    char *output_file,
			    char F,      // interpolation option
			    int Nl,      // No. of points
			    int storage_flag);   // where to store


// interpolated_reduced_S()=Gives approx. reduced scattering matrix by
// quadratic /linear interpolation for wavelengths between nodal wavelengths 
// by diving it into Nl points. 
// Nodal values of reduced S are given in the file "input_Sr" with
// Data structure: first line: no. of BWG modes + SWG modes
// then for other lines: first entry wavelength then real and imag part of Sr
Dmatrix interpolated_reduced_S(char *input_Sr,
			       char *output_file,
			       char F,      // interpolation option
			       int Nl,      // No. of points
			       int storage_flag);   // where to store


// interpolated_beta()=Gives approx. propagation consts for Straight waveguide
// by quadratic /linear interpolation for wavelengths between nodal 
// wavelengths  by diving it into Nl points. 
// Nodal values of betas are given in the file '*input_beta' with
// Data structure: first line: no. of SWG modes
// 2'nd line: Initial amps of modes (Needed for microresonator calculations) 
// then for other lines: first entry wavelength then beta values(s)
// from fundamental mode to higher order modes
Dmatrix interpolated_beta(char *input_beta,
			  char *output_file,
			  char F,      // interpolation option
			  int Nl,      // No. of points
			  int storage_flag);   // where to store
#endif // BSLAMS_H
