/*
 * Circurs - Circular optical microresonators simulation tools
 * Kirankumar R. Hiremath (k.r.hiremath@ieee.org)
 * University of Twente, Department of Applied Mathematics 
 * P.O. Box 217, 7500AE Enschede, The Netherlands
 * (2005)
 */

/*
 * mr.c
 * circular microresonator 
 */


#include "mr.h"
#include "bslamvis.h"

MR::MR(
    BSCoupler &bsc_i_, 
    BSCoupler &bsc_o_, 
    double xc0_, double xc1_, int Nxc_, 
    double zc0_, double zc1_, int Nzc_,
    Cvector MRA_in_, Cvector MRA_add_)
{
    bsc_i = bsc_i_;
    bsc_o=bsc_o_;

    cerr << "--*-- MR::MR() --*--\n" 
	 << "\tInput coupler no. of BWG modes  =" << bsc_i.bma.num <<"\t"
	 << "Input coupler no. of SWG modes  =" << bsc_i.sma.num << "\n"
	 << "\tOutput coupler no. of BWG modes =" << bsc_o.bma.num <<"\t"
	 << "Output coupler no. of SWG modes =" << bsc_o.sma.num << "\n";

  
// Check whether two coupler are compatible with each other for polarisation,
// radius of cavity, wavelength.
    if ( coupler_check() != 0)
    {
	cerr << "--*--MR::MR() Quitting.--*--\n\n";
	exit(1);
    }
    else
    {
	//  set the values for radius, wavelength and polarisation.
	R = bsc_i.bma(0).bend.R;
	pol = bsc_i.bspol;
	lambda = bsc_i.bma(0).bend.wg.lambda; // can be done in elegent way :-(
    }

    cerr << "--*-- Pol=T" <<polCHR(pol) <<"--*--\n";

// check input port amplitudes are proper.
    if (MRA_in_.nel != bsc_i.sma.num)
    {
	cerr <<"\n--*-- MR::MR(): error --*--"
	     <<"\nInput port amplitudes are not proper."
	     <<"\nRequired: no. of rows= " <<  bsc_i.sma.num
	     <<"\t\tInput: no. of rows= " << MRA_in_.nel
	     <<". \nQuiting.\n\n";
	    exit(1);
    }
    else
    {
	cerr << "--*-- MR input port amplitudes are proper.\n";
	Ap = Cvector(bsc_i.sma.num);
	for(int i=0; i< bsc_i.sma.num; i++)
	    Ap(i)  = MRA_in_(i)*exp( -CCI*bsc_i.sma(i).beta*(bsc_i.z0 - zc0_));	
    }

// check add port amplitudes are proper.
    if (MRA_add_.nel != bsc_o.sma.num)
    {
	cerr <<"\n--*-- MR::MR(): error --*--"
	     <<"\nAdd port amplitudes are not proper."
	     <<"\nRequired: no. of rows= " <<  bsc_o.sma.num
	     <<"\tInput: no. of rows= " << MRA_add_.nel
	     <<". \nQuiting.\n\n";
	    exit(1);
    }
    else
    {
	cerr << "--*-- MR add port amplitudes are proper.\n";
	Cm = Cvector(bsc_o.sma.num);
	for(int i=0; i< bsc_o.sma.num; i++)
	    Cm(i)  = MRA_add_(i)*exp( -CCI*bsc_o.sma(i).beta*(bsc_o.z0 - zc0_));	
    }

    xc0 = xc0_;
    xc1 = xc1_;
    Nxc = Nxc_;
    hxc = (xc1 - xc0)/((double)(Nxc - 1));

    zc0 = zc0_;
    zc1 = zc1_;
    Nzc = Nzc_;
    hzc = (zc1 - zc0)/((double)(Nzc - 1));

    set_amp(1, 0.0);

    
// maximum radial distance on which we evaluate bend fields
    double x_max, z_max, r_max;
    x_max = max(abs(xc0), abs(xc1));
    z_max = max(abs(zc0), abs(zc1));
    r_max = sqrt(x_max*x_max + z_max*z_max);
    int N_max = 2*max(Nxc, Nzc);

    // Interval on which we need discretised bend field
    Interval rd(0.0, r_max);

    cerr <<"--*-- MR::MR()--- Discretization interval = ["
	 << rd.x0 <<", " << rd.x1 <<"] with " << N_max << " points.\n";

    for(int i=0; i < bsc_i.bma.num; i++) bsc_i.bma(i).set_d_field(rd, N_max,0);
    for(int i=0; i < bsc_o.bma.num; i++) bsc_o.bma(i).set_d_field(rd, N_max,0);
//*/
    cerr << "--*-- MR::MR() leaving.\n\n";
}

//---------------------------------------------------------
int MR::coupler_check(void)const
{
// checking for wavelength compatibility
    if (bsc_i.bma(0).bend.wg.lambda != bsc_o.bma(0).bend.wg.lambda)
    {
	cerr <<"\n--*-- MR::coupler_check(): error --*--"
	     <<"\nTwo BS couplers don't have same wavelenght."
	     <<"\nbsc_i.bma(0).bend.wg.lambda=" << bsc_i.bma(0).bend.wg.lambda
	     << "\tbsc_o.bma(0).bend.wg.lambda=" << bsc_o.bma(0).bend.wg.lambda
	     <<". \nQuiting.";
	return 1;
    }
//checking for polarsation compatibility
     if (bsc_i.bspol != bsc_o.bspol)
     {
 	cerr <<"\n--*-- MR::coupler_check(): error --*--"
 	     <<"\nTwo BS couplers don't have same polarisation."
 	     <<"bsc_i.bspol=" << bsc_i.bspol
 	     << "\tbsc_o.bspol=" << bsc_o.bspol
 	     <<". \nQuiting.";
	return 1;
     }
//checking for compatibility of cavity elements of the two couplers
     if (bsc_i.bma(0).bend.R != bsc_o.bma(0).bend.R)
     {
 	cerr <<"\n--*-- MR::coupler_check(): error --*--"
 	     <<"\nTwo BS couplers don't have same radius."
 	     <<"R=" <<R 
 	     <<"\tR=" <<bsc_o.bma(0).bend.R 	    
 	     <<". \nQuiting.";
	return 1;
     }

    cerr << "--*-- MR::coupler_check(): Cavity checking done . --*--\n";
    return 0;
}


//-------------------------------------------------------------------------

//MR::~MR(){}

//------------------------------------------------------------------------
// complex valued MR field at a point (x,z) 
// cp:  ER, EY, EP, HR, HY, HP, SR, SY, SP
Complex MR::field(Fcomp cp, double x, double z)const
{
    Complex ans;

    if ( (z > -bsc_o.z1) && (z <= -bsc_o.z0) && 
	 ( x <= -bsc_o.x0) && ( x > -bsc_o.x1))
    {
	if ( (cp != EY) && (cp != HY)  )  
	    ans = bsc_o.mirror_field(cp, -x,-z);
	else
	    ans = bsc_o.field(cp, -x,-z); 

    }
    else if ( (z > bsc_i.z0) && (z <= bsc_i.z1) && 
	      ( x >= bsc_i.x0) && ( x <  bsc_i.x1))
    {
	ans = bsc_i.field(cp, x,z);
    }
    else if ( (z <= -bsc_o.z1 ) && ( x < 0.0))
    {
	for (int i=0; i < bsc_o.bma.num; i++)
	    ans = ans + ap(i)*bsc_i.bma(i).field_cart(cp, x,z)
		        * exp(CCI*bsc_i.bma(i).gamma*R*bsc_i.theta(bsc_i.z0));
	    
	for (int i=0; i < bsc_o.sma.num; i++)
	    ans = ans + Dm(i)
		        *bsc_o.sma(i).cfield(cp, FORW, -x, -z + bsc_o.z1);
    }
    else if ( (z <= bsc_i.z0) && ( x >= 0.0))
    {
	for (int i=0; i < bsc_i.bma.num; i++)
	    ans = ans + ap(i)*bsc_i.bma(i).field_cart(cp, x,z) 
		      * exp(CCI*bsc_i.bma(i).gamma*R* bsc_i.theta(bsc_i.z0));
	    
	for (int i=0; i < bsc_i.sma.num; i++)
	    ans = ans + Ap(i)
		*bsc_i.sma(i).cfield(cp, FORW, x,z - bsc_i.z0); 
    }
    else if ( (z > -bsc_o.z0 ) && ( x < 0.0))
    { 
	if ( (cp != EY) && (cp != HY)  )  
	{
	    for (int i=0; i < bsc_o.bma.num; i++)
		ans = ans - cm(i)*bsc_o.bma(i).field_cart(cp, -x,-z)
		    * exp(CCI*bsc_o.bma(i).gamma*R* (bsc_o.theta(bsc_o.z0))); //!
	}
	else
	{
	    for (int i=0; i < bsc_o.bma.num; i++)
		ans = ans + cm(i)*bsc_o.bma(i).field_cart(cp, -x,-z)
		    * exp(CCI*bsc_o.bma(i).gamma*R* (bsc_o.theta(bsc_o.z0))); //!
	}

	for (int i=0; i < bsc_o.sma.num; i++)
	    ans = ans + 
		  Cm(i)*bsc_o.sma(i).cfield(cp, FORW, -x, -z + bsc_o.z0);
	
    } 
    else if ( (z > bsc_i.z1 ) && ( x >= 0.0))
    {
	for (int i=0; i < bsc_i.bma.num; i++)
	    {ans = ans + bp(i)*bsc_i.bma(i).field_cart(cp, x,z)
		        *exp(CCI*bsc_i.bma(i).gamma*R*bsc_i.theta(bsc_i.z1));
	    }


	for (int i=0; i < bsc_i.sma.num; i++)
	{
	    ans = ans + Bp(i)*bsc_i.sma(i).cfield(cp, FORW, x, z - bsc_i.z1);
	}
    }
    else if( ( z >= 0.0) && ( z < bsc_i.z1) && ( x >=0.0 ) && ( x < bsc_i.x0))
    {
	for (int i=0; i < bsc_i.bma.num; i++)
		ans = ans + bp(i)*bsc_i.bma(i).field_cart(cp, x,z)
		    *exp(CCI*bsc_i.bma(i).gamma*R*bsc_i.theta(bsc_i.z1));
    }
    else // to make sure all regions are taken into account
    {
	cerr << "( " << x <<", " << z << ") is in missing region.\n";
	ans =0.0;
    }

    return ans;

}

//--------------------------------------------------------------------
// real valued MR field at a point (x,z) 
// cp:  ER, EY, EP, HR, HY, HP, SR, SY, SP
// foa: MOD, ORG, REP, IMP, SQR 
double MR::field(Fcomp cp, Afo foa, double x, double z)const
{
    Complex fc;
    fc = field(cp, x, z);
    switch(foa)
    {
	case REP: 
	    return fc.re;
	case IMP: 
	    return fc.im;
	case MOD: case ORG:
	    return fc.abs();
	case SQR:
	    return fc.sqabs();
    }
    return fc.re;
}


//-------------------------------------------------------------------
//-------------------------------------------------------------------

/* the optical field, discretized on a regular rectangular mesh,  
   npx x npz points on a display window dwx x dwz
   cp: one of E?, H?, S? ... ?
   foa: ORG, MOD, SQR, REP, IMP (ORG = REP) */
Cmatrix MR::field(Fcomp cp, Interval dwx, int npx, 
                            Interval dwz, int npz) const
{
	Cmatrix fld(npx, npz);
	double dx = dwx.len()/((double)(npx-1));
	double dz = dwz.len()/((double)(npz-1));
	double x, z;

	z = dwz.x0;
	for(int j=0; j<=npz-1; ++j)
	{
		x = dwx.x0;
		for(int i=0; i<=npx-1; ++i) 
		{
			fld(i, j) = field(cp, x, z);
			x += dx;
		}
		z += dz;
	}
	return fld;
}

//----------------------------------------------------------------------
Dmatrix MR::field(Fcomp cp, Afo foa, Interval dwx, int npx, 
                                     Interval dwz, int npz) const
{
	Dmatrix fld(npx, npz);
	double dx = dwx.len()/((double)(npx-1));
	double dz = dwz.len()/((double)(npz-1));
	double x, z;
	Complex f;

	z = dwz.x0;
	for(int j=0; j<=npz-1; ++j)
	{
		x = dwx.x0;
		for(int i=0; i<=npx-1; ++i) 
		{
			f = field(cp, x, z);
			switch(foa)
			{
			case MOD:           fld(i, j) = f.abs();   break;
			case SQR:           fld(i, j) = f.sqabs(); break;
			case ORG: case REP: fld(i, j) = f.re;      break;
			case IMP:           fld(i, j) = f.im;      break;
			default:            fld(i, j) = f.re;      break;
			}
			x += dx;
		}
		z += dz;
	}
	return fld;
}

//-------------------------------------------------------------------------

// ... determine the maximum absolute level fmax
Cmatrix MR::field(Fcomp cp, Interval dwx, int npx, 
                            Interval dwz, int npz, double& fmax) const
{
	Cmatrix fld(npx, npz);
	Complex f;
	double dx = dwx.len()/((double)(npx-1));
	double dz = dwz.len()/((double)(npz-1));
	double x, z;

	fmax = 0.0;
	z = dwz.x0;
	for(int j=0; j<=npz-1; ++j)
	{
		x = dwx.x0;
		for(int i=0; i<=npx-1; ++i) 
		{
			f = field(cp, x, z);
			fld(i, j) = f;
			if(f.abs() > fmax) fmax = f.abs();
			x += dx;
		}
		z += dz;
	}
	return fld;
}

/* ------------------------------------------------------------------- */
/* visualization of the light propagation, output: MATLAB scripts */
	
/* image of component cp
   cp: one of E?, H?, S? ... ?
   foa: MOD, SQR, REP, IMP, ORG = REP 
   xbeg, xend, zbeg, zend: x resp. z extensions of the plot window   
   npx, npz: number of plot points in the x and z directions
   ext0, ext1: filename id characters for the m.file */
void MR::plot(Fcomp cp, Afo foa, 
              double xbeg, double xend, double zbeg, double zend,
              int npx, int npz, char ext0, char ext1) const
{

	FILE *dat;
	char name[13] = "pl_____I.m";
	name[2] = afochr(foa);
        name[3] = fldchr(cp);
        name[4] = cpchr(cp);      
	name[5] = ext0;
	name[6] = ext1;
	fprintf(stderr, "plot([%g, %g] x [%g, %g]) >> %s\n", 
	                xbeg, xend, zbeg, zend, name);
	dat = fopen(name, "w+");
	mlout_title(dat, name, "Circurs, microresonator field");
	mlout_micresgeo(dat, bsc_i.bma(0).bend, bsc_i.sma(0).wg, 
	                                        bsc_o.sma(0).wg);
	mlout_meshxz(dat, xbeg, xend, npx, zbeg, zend, npz);
	Dmatrix fld = field(cp, foa, Interval(xbeg, xend), npx, 
	                             Interval(zbeg, zend), npz);
	mlout_fld(dat, npx, npz, cp, fld);
	name[8] = 0; 
	char desc[10];
	afocpdesc(foa, cp, desc);
	mlout_micresgenimage(cp, name, desc, dat); 
	if(foa == MOD || foa == SQR) mlout_lincolormap(dat);
	else                         mlout_magcolormap(dat);
	mlout_print(dat, name, 'e'); 
	fclose(dat);
	return;
}

/* animation of the light propagation
   the frames show the EY component (TE) resp. HY component (TM), at
   ntfr equidistant times over one time period
   xbeg, xend, zbeg, zend: x resp. z extensions of the plot window   
   npx, npz: number of plot points in the x and z directions
   ext0, ext1: filename id characters for the m.file */
void MR::movie(double xbeg, double xend, double zbeg, double zend,
               int npx, int npz, int ntfr, char ext0, char ext1) const
{
	FILE *dat;
	char name[13] = "an____M.m";
	double tmax, dt;
	Fcomp cp = principalcomp(pol);
        name[2] = fldchr(cp);
        name[3] = cpchr(cp);      
	name[4] = ext0;
	name[5] = ext1;
	tmax = period_T(lambda);
	dt = tmax/ntfr;
	fprintf(stderr, 
	   "movie(%d x [%g, %g] x [%g, %g]), T=%g fs, dt=%g fs >> %s\n", 
	   ntfr, xbeg, xend, zbeg, zend, tmax, dt, name);
	dat = fopen(name, "w+");
	mlout_title(dat, name, "Circurs, field animation");
	mlout_micresgeo(dat, bsc_i.bma(0).bend, bsc_i.sma(0).wg, 
	                                        bsc_o.sma(0).wg);
	mlout_meshxz(dat, xbeg, xend, npx, zbeg, zend, npz);
	double famp;
	Cmatrix fld = field(cp, Interval(xbeg, xend), npx, 
	                        Interval(zbeg, zend), npz, famp); 
	mlout_fld(dat, npx, npz, cp, fld.re());
	mlout_fldtore(dat, cp);
	mlout_fld(dat, npx, npz, cp, fld.im());
	mlout_fldtoim(dat, cp);
	name[7] = 0; 
	char desc[10];
	afocpdesc(ORG, cp, desc);
	mlout_micresmovie(cp, name, desc, dat, ntfr, dt, omega(lambda), famp); 
	fclose(dat);
	return;
}
		
/* export full field data ("all") into a viewer m-file 
   xbeg, xend, zbeg, zend: x resp. z extensions of the plot window   
   npx, npz: number of plot points in the x and z directions
   ext0, ext1: filename id characters */
void MR::viewer(double xbeg, double xend, double zbeg, double zend,
                int npx, int npz, char ext0, char ext1) const
{
	FILE *dat;
	char name[13] = "fld__A.m";
	name[3] = ext0;
	name[4] = ext1;
	fprintf(stderr, 
	   "viewer([%g (%d) %g] x [%g (%d) %g]) >> %s\n", 
	   xbeg, npx, xend, zbeg, npz, zend, name);
	dat = fopen(name, "w+");
	name[6] = 0; 
	mlout_micresviewertop(dat, name, pol, lambda);
	mlout_micresgeo(dat, bsc_i.bma(0).bend, bsc_i.sma(0).wg, 
	                                        bsc_o.sma(0).wg);
	mlout_meshxz(dat, xbeg, xend, npx, zbeg, zend, npz);
	Cmatrix f;
	Fcomp cp;
	if(pol == TE)
	{
		cp = EX;
		mlout_0fld(dat, cp);
		mlout_fldtore(dat, cp);
		mlout_0fld(dat, cp);
		mlout_fldtoim(dat, cp);
		cp = EY;
		f = field(cp, Interval(xbeg, xend), npx, 
			      Interval(zbeg, zend), npz); 
		mlout_fld(dat, npx, npz, cp, f.re());
		mlout_fldtore(dat, cp);
		mlout_fld(dat, npx, npz, cp, f.im());
		mlout_fldtoim(dat, cp);
		cp = EZ;
		mlout_0fld(dat, cp);
		mlout_fldtore(dat, cp);
		mlout_0fld(dat, cp);
		mlout_fldtoim(dat, cp);
		cp = HX;
		f = field(cp, Interval(xbeg, xend), npx, 
			      Interval(zbeg, zend), npz); 
		mlout_fld(dat, npx, npz, cp, f.re());
		mlout_fldtore(dat, cp);
		mlout_fld(dat, npx, npz, cp, f.im());
		mlout_fldtoim(dat, cp);
		cp = HY;
		mlout_0fld(dat, cp);
		mlout_fldtore(dat, cp);
		mlout_0fld(dat, cp);
		mlout_fldtoim(dat, cp);
		cp = HZ;
		f = field(cp, Interval(xbeg, xend), npx, 
			      Interval(zbeg, zend), npz); 
		mlout_fld(dat, npx, npz, cp, f.re());
		mlout_fldtore(dat, cp);
		mlout_fld(dat, npx, npz, cp, f.im());
		mlout_fldtoim(dat, cp);
	}
	else
	{
		cp = EX;
		f = field(cp, Interval(xbeg, xend), npx, 
			      Interval(zbeg, zend), npz); 
		mlout_fld(dat, npx, npz, cp, f.re());
		mlout_fldtore(dat, cp);
		mlout_fld(dat, npx, npz, cp, f.im());
		mlout_fldtoim(dat, cp);
		cp = EY;
		mlout_0fld(dat, cp);
		mlout_fldtore(dat, cp);
		mlout_0fld(dat, cp);
		mlout_fldtoim(dat, cp);
		cp = EZ;
		f = field(cp, Interval(xbeg, xend), npx, 
			      Interval(zbeg, zend), npz); 
		mlout_fld(dat, npx, npz, cp, f.re());
		mlout_fldtore(dat, cp);
		mlout_fld(dat, npx, npz, cp, f.im());
		mlout_fldtoim(dat, cp);
		cp = HX;
		mlout_0fld(dat, cp);
		mlout_fldtore(dat, cp);
		mlout_0fld(dat, cp);
		mlout_fldtoim(dat, cp);
		cp = HY;
		f = field(cp, Interval(xbeg, xend), npx, 
			      Interval(zbeg, zend), npz); 
		mlout_fld(dat, npx, npz, cp, f.re());
		mlout_fldtore(dat, cp);
		mlout_fld(dat, npx, npz, cp, f.im());
		mlout_fldtoim(dat, cp);
		cp = HZ;
		mlout_0fld(dat, cp);
		mlout_fldtore(dat, cp);
		mlout_0fld(dat, cp);
		mlout_fldtoim(dat, cp);
	}
	Dmatrix n(npx, npz);
	BSlabWaveguide b = bsc_i.bma(0).bend;
	Waveguide si = bsc_i.sma(0).wg;
	Waveguide so = bsc_o.sma(0).wg;
	double rc = b.R+b.wg.hx(1);
	double ri;
	if(   fabs(b.wg.hx(1)-b.wg.hx(0))>COMPTOL_HX
	   && fabs(b.wg.n(0)-b.wg.n(1))>COMPTOL_N)
		ri = b.R+b.wg.hx(0);
	else
		ri = b.R+b.wg.hx(1);
	double wi = si.hx(1)-si.hx(0); 
	double gi = si.hx(0)-rc;
	double wo = so.hx(1)-so.hx(0); 
	double go = so.hx(0)-rc;
	double ni = b.wg.n(0);
	double nc = b.wg.n(1);
	double nb = b.wg.n(2);
	double nsi = si.n(1);
	double nso = so.n(1);
	double nei = si.n(2);
	double neo = so.n(2);
	double dx = (xend-xbeg)/((double)(npx-1));
	double dz = (zend-zbeg)/((double)(npz-1));
	double x, z, r;
	z = zbeg;
	for(int j=0; j<=npz-1; ++j)
	{
		x = xbeg;
		for(int i=0; i<=npx-1; ++i) 
		{ 
			if(x<=-rc-gi-wi)
			{
				n(i, j) = nei; 
				goto fin;
			}
			if(x<=-rc-gi)
			{
				n(i, j) = nsi; 
				goto fin;
			}
			if(x>=rc+go+wo)
			{
				n(i, j) = neo; 
				goto fin;
			}
			if(x>=rc+go)
			{
				n(i, j) = nso; 
				goto fin;
			}
			r = sqrt(x*x+z*z);
			if(r>=rc)
			{
				n(i, j) = nb; 
				goto fin;
			}
			if(r<=ri)
			{
				n(i, j) = ni; 
				goto fin;
			}
			n(i, j) = nc; 
fin: 			;;
			x += dx; 
		}
		z += dz;
	}
	mlout_n(dat, npx, npz, n);
	mlout_micresfldviewer(dat, name);
	fclose(dat);
	return;
}

// gives power dropped
double MR::get_pd(int mode_index)const
{
    if ((mode_index < 0) || (mode_index >= bsc_o.sma.num))
    {
	cerr <<"--*--From MR::get_pd(mode_index) -- error: Wrong mode index"
	     <<"mode_index should be between 0 and " << (bsc_o.sma.num -1)
	     << ".\nInput mode index is " << mode_index << ".\n";
	exit(1);
    }
    else
	return Dm(mode_index).sqabs();
}

//-------------------------------------------------------------------
// gives power transmitted
double MR::get_pt(int mode_index)const
{
    if ((mode_index < 0) || (mode_index >= bsc_i.sma.num))
    {
	cerr <<"--*--From MR::get_pt(mode_index) -- error: Wrong mode index"
	     <<"mode_index should be between 0 and " << (bsc_i.sma.num -1)
	     << ".\nInput mode index is " << mode_index << ".\n";
	exit(1);
    }
    else
	return Bp(mode_index).sqabs();
}

//-----------------------------------------------------------------------
// Direct method of amplitude computation
void MR::set_amp(int proj, double z)
{
    cerr <<"--*-- MR::set_amp() called.\n";
    
    int Nbm_i, Nsm_i, Nbm_o, Nsm_o;
    Nbm_i = bsc_i.bma.num;
    Nsm_i = bsc_i.sma.num;
    Nbm_o = bsc_o.bma.num;
    Nsm_o = bsc_o.sma.num;

    double l[4];

    Cmatrix S_i(bsc_i.S.r, bsc_i.S.c), Oi_i(bsc_i.Oi.r, bsc_i.Oi.c);
    Cmatrix S_o(bsc_o.S.r, bsc_o.S.c), Oi_o(bsc_o.Oi.r, bsc_o.Oi.c);

    if (proj == 0)
    { //use field matching setting
	cerr << "--*-- Field match setting is used. --*--\n";
	
    }
    else
    { // use projection setting
	cerr << "--*-- Projection setting is used. --*--\n";
	
	bsc_i.correction_matrix(bsc_i.Z.x0, Oi_i);
	bsc_o.correction_matrix(bsc_o.Z.x0, Oi_o);

// For in port coupler

	if (z != 0.0)
	{ // use S matrix at input z level

	    cerr << "S matrix near z=" << z << " is used.\n";

	    double i_z = (z - bsc_i.Z.x0)/bsc_i.hz;
	    
	    int nf = (int)(floor(i_z));
	    
	    cerr << "nf=" << nf << "\n";

	    //  input z lies between zf < z
	    double zf = bsc_i.Z.x0 + nf*bsc_i.hz; 

	    cerr << "Using S matrix at z="<< zf << "\n";	

	    Cmatrix S_nf=bsc_i.Sz.submatrix(1, (Nbm_i + Nsm_i)*(Nbm_i + Nsm_i), nf, 0); 

	    int ic = 0;

	    for (int i=0; i < bsc_i.bma.num + bsc_i.sma.num; i++)
	    {
		for (int j=0; j < bsc_i.bma.num + bsc_i.sma.num; j++)
		{
		    S_i(i,j) = S_nf(0,ic);
		    ic++;
		}
	    }
	    
	    l[0] = abs(bsc_i.theta(bsc_i.z0)) * R; 
	    l[1] = abs(bsc_i.theta(zf)) * R;

	}
	else
	{
	    S_i = bsc_i.S;
	    l[0] = abs(bsc_i.theta(bsc_i.z0)) * R; 
	    l[1] = abs(bsc_i.theta(bsc_i.z1)) * R;
    
	}

	
	// for add port coupler
	if (z != 0.0)
	{ // S matrix at input z level is used

	    cerr << "S matrix near z=" << z << " is used. \n";

	    double i_z = (z - bsc_o.Z.x0)/bsc_o.hz;
	    
	    int nf = (int)(floor(i_z));
	    
	    cerr << "nf=" << nf << "\n";

	    //  input z lies between zf < z 
	    double zf= bsc_o.Z.x0 + nf*bsc_o.hz; 

	    cerr << "Using S matrix at z="<< zf << "\n";	

	    Cmatrix S_nf=bsc_o.Sz.submatrix(1, (Nbm_o + Nsm_o)*(Nbm_o + Nsm_o), nf, 0); 

	    int ic =0;

	    for (int i=0; i < bsc_o.bma.num + bsc_o.sma.num; i++)
	    {
		for (int j=0; j < bsc_o.bma.num + bsc_o.sma.num; j++)
		{
		    S_o(i,j) = S_nf(0,ic);
		    ic++;
		}
	    }
	    
	    l[2] = abs(bsc_o.theta(bsc_o.z0)) * R;
	    l[3] = abs(bsc_o.theta(zf)) * R;
	    
	}
	else
	{
	    S_o = bsc_o.S;
	    l[2] = abs(bsc_o.theta(bsc_o.z0)) * R;
	    l[3] = abs(bsc_o.theta(bsc_o.z1)) * R;
	}

	
	cerr << "\n";


    }

// - - - - - - 
    
    cerr  << " l[0] = " << l[0] << "\t"
          << " l[1] = " << l[1] << "\t"
	  << " l[2] = " << l[2] << "\t"                         
	  << " l[3] = " << l[3] << "\n";

    L1 = PI*R - l[1] - l[2];
    L2 = Pi*R - l[0] - l[3];
  
    cerr << "--*-- L1= " << L1 << "\t" << "L2= " << L2 << "\n";

    Oi_i.inverse();
    Oi_o.inverse();


    // if scatterin matrix at input z level is used, then for bent waveguide 
    // field matching outside the coupler, original bp, Bpm dm, Dm values are 
    // required. Store these values for later use.

    Cvector ori_bp( Nbm_i), ori_Bp(Nsm_i), ori_dm( Nbm_o), ori_Dm(Nsm_o);

    if ( z != 0)
    {
	ori_bp = bp;
	ori_Bp = Bp;
	ori_dm = dm;
	ori_Dm = Dm;
    }


    Cmatrix LHS(4*Nbm_i + Nsm_i + Nsm_o, 4*Nbm_i + Nsm_i + Nsm_o);

    Cvector RHS(4*Nbm_i + Nsm_i + Nsm_o);

    Bp = Cvector(Nsm_i);
    Bm = Cvector(Nsm_i);
    Am = Cvector(Nsm_i);
    ap = Cvector(Nbm_i);
    am = Cvector(Nbm_i);
    bp = Cvector(Nbm_i);
    bm = Cvector(Nbm_i);

    Cp = Cvector(Nsm_o);
    Dp = Cvector(Nsm_o);
    Dm = Cvector(Nsm_o);

    cp = Cvector(Nbm_o);
    cm = Cvector(Nbm_o);
    dp = Cvector(Nbm_o);
    dm = Cvector(Nbm_o);


    int r0, r1, c0, c1;

// set all diagonal elements of LHS to -1.0

    for(int i=0; i< LHS.r; i++) LHS(i,i) = -1.0;

// first block
    r0 = 0;
    r1 = Nbm_i;

    for(int r=r0; r< r1; r++)
	LHS(r, 3*Nbm_i + Nsm_i + r)= exp( -CCI*bsc_i.bma(r).gamma * L2);

//second block
    r0 = Nbm_i;
    r1 = 2*Nbm_i + Nsm_i;
    c0 = 0;
    c1 = Nbm_i;

    for(int r = r0; r < r1;  r++)
	for (int c=c0; c <c1 ; c++)  LHS(r,c) = S_i( r - r0,c);

// third block
    r0 = 2*Nbm_i + Nsm_i;
    r1 = 3*Nbm_i + Nsm_i;

    for(int r=r0; r<r1; r++)
	LHS(r, r - Nbm_i - Nsm_i) =  exp(-CCI*bsc_i.bma(r - r0).gamma*L1);

//forth block
    r0 = 3*Nbm_i + Nsm_i;
    r1 = 4*Nbm_i + Nsm_i + Nsm_o;
    c0 = 2*Nbm_i + Nsm_i;
    c1 = 3*Nbm_i + Nsm_i;

    for(int r = r0; r < r1;  r++)
	for (int c=c0; c< c1; c++) LHS(r,c) = S_o( r - r0, c - c0);


// filling up RHS matrix (RHS)

    for(int r= Nbm_i; r < 2*Nbm_i + Nsm_i; r++)
	for(int c= 0 ; c < Nsm_i; c++)
	    RHS(r) -= S_i( r - Nbm_i, Nbm_i + c) * Ap(c);
    

    for(int r= 3*Nbm_i + Nsm_i; r < 4*Nbm_i + Nsm_i + Nsm_o; r++)
    {
	for(int c= 0 ; c < Nsm_o; c++)
	    RHS(r) -= S_o( r - 3*Nbm_i - Nsm_i, Nbm_i + c) * Cm(c);
    }
    
    LHS.inverse();
   
    Cvector ap_bp_Bp_cm_dm_Dm = mult(LHS,RHS);

// extract the amps 

    ap = ap_bp_Bp_cm_dm_Dm.subvector(Nbm_i,0);
    bp = ap_bp_Bp_cm_dm_Dm.subvector(Nbm_i, Nbm_i);
    Bp = ap_bp_Bp_cm_dm_Dm.subvector(Nsm_i, 2*Nbm_i);
    cm = ap_bp_Bp_cm_dm_Dm.subvector(Nbm_i, 2*Nbm_i + Nsm_i);
    dm = ap_bp_Bp_cm_dm_Dm.subvector(Nbm_i, 3*Nbm_i + Nsm_i);
    Dm = ap_bp_Bp_cm_dm_Dm.subvector(Nsm_o, 4*Nbm_i + Nsm_i);


// display the results
    cerr <<"--*-- MR::set_amp() results. (indexing: fundamental mode to higher order mode) --*--\n";

    cerr <<"Ap: ";
    for (int i=0; i< Nsm_i; i++) 
	cerr <<"(" << Ap(i).re <<", "<< Ap(i).im <<")\t";  
    cerr <<"\n";

    cerr <<"ap: ";
    for (int i=0; i< Nbm_i; i++) 
	cerr <<"(" << ap(i).re <<", "<< ap(i).im <<")\t";  
    cerr <<"\n";

    cerr <<"Bp: ";
    for (int i=0; i< Nsm_i; i++) 
	cerr <<"(" << Bp(i).re <<", "<< Bp(i).im <<")\t";  
    cerr <<"\n";

    cerr <<"bp: ";
    for (int i=0; i< Nbm_i; i++) 
	cerr <<"(" << bp(i).re <<", "<< bp(i).im <<")\t";  
    cerr <<"\n";

    cerr <<"Cm: ";
    for (int i=0; i< Nsm_o; i++) 
	cerr <<"(" << Cm(i).re <<", "<< Cm(i).im <<")\t";  
    cerr <<"\n";

    cerr <<"cm: ";
    for (int i=0; i< Nbm_o; i++) 
	cerr <<"(" << cm(i).re <<", "<< cm(i).im <<")\t";  
    cerr <<"\n";

    cerr <<"Dm: ";
    for (int i=0; i< Nsm_o; i++) 
	cerr <<"(" << Dm(i).re <<", "<< Dm(i).im <<")\t";  
    cerr <<"\n";

    cerr <<"dm: ";
    for (int i=0; i< Nbm_o; i++) 
	cerr <<"(" << dm(i).re <<", "<< dm(i).im <<")\t";  

    cerr <<"\n--*------------------------------------------------------------------------------*--\n";

// now MR amplitudes are set.
    set_amp_flag = true;

    Cvector aA( Nbm_i + Nsm_i);
    Cvector cC( Nbm_i + Nsm_i);

    // fill aA, cC

    for (int i=0; i< Nbm_i; i++) 
    {
	aA(i) = ap(i);
	cC(i) = cm(i);
    }
    for (int i=0; i< Nsm_i; i++) aA(Nbm_i + i) = Ap(i);
    for (int i=0; i< Nsm_o; i++) cC(Nbm_o + i) = Cm(i);


    // to set input amplitudes for input coupler
    BSAmpbsc_i = mult(Oi_i, aA);

    // set the corresponding coupled mode amplitudes of input coupler
    cerr << "--*-- For input coupler  :";
    bsc_i.set_amp(BSAmpbsc_i);

    // to set input amplitudes for output coupler
    BSAmpbsc_o = mult(Oi_o, cC);

    // set the corresponding coupled mode amplitudes of output coupler
    cerr << "--*-- For output coupler :";
    bsc_o.set_amp(BSAmpbsc_o);
    

    // if scatterin matrix at input z level is used, then for bent waveguide 
    // field matching outside the coupler, original bp, Bpm dm, Dm values are 
    // required. These values for original computational windows are stored.
    // Restore these values.

    cerr << "--*-- MR::set_amp() is leaving. --*--\n";   
	
}



//------------------------------------------------------------------
//  interpolated power spectrum for microresonator
// Output file with data structure: wavelength - Through power - Drop power
void interpolated_power_spectrum(
    char *gf,    // file of BWG gamma values
    char *bif,   // file of Inport SWG beta values
    char *baf,   // file of Addport SWG beta values
    char *Sif,   // file of Inport S reduced matrix
    char *Saf,   // file of Addport S reduced matrix
    char *output_file) 
{
    cerr <<"--*-- MR::interpolated_power_spectrum() called.--*--\n";


// propagation constants of cavity modes for individual wavelength are 
// available in the file "gf".
// Data structure: first line: bent waveguide radius
// second line: no. of BWG modes
// then for each line first entry wavelength then real and imag part of gamma

    ifstream bgamma(gf, ios::in);
    
    if (!bgamma)
    {
	cerr <<"--*--MR::interpolated_power_spectrum(). "
	    <<"Unable to open file " << gf <<".--*--\n";
	exit(1);
    }

// propagation constants of Inport straight waveguide modes for each 
// wavelength are given in the file "bif" and those of Addport straight 
// waveguide modes are given in the file "baf". 
// Data structure: first line: no. of SWG modes
// second line: Initial amplitude for each mode
// then for each line first entry wavelength then beta value(s).

    ifstream sibeta(bif, ios::in);
    
    if (!sibeta)
    {
	cerr <<"--*--MR::interpolated_power_spectrum(). "
	    <<"Unable to open file " << bif <<".--*--\n";
	exit(1);
    }

    ifstream sabeta(baf, ios::in);
    
    if (!sabeta)
    {
	cerr <<"--*--MR::interpolated_power_spectrum(). "
	    <<"Unable to open file " << baf <<".--*--\n";
	exit(1);
    }

 
// S_reduced matrix of couplers for individual wavelength are available in 
// the file "Sif" (for In port coupler) and "Saf" (for Add port coupler).
// Data structure: 
// first line: no. of BWG modes and  no. of SWG modes
// second line: End points of Z computation window (Z.x0, Z.x1)
// then for other lines: first entry wavelength then real and imag part of Sr

    ifstream Sr_i(Sif, ios::in);
    ifstream Sr_a(Saf, ios::in);
    
    if (!Sr_i)
    {
	cerr <<"--*--MR::interpolated_power_spectrum(). "
	    <<"Unable to open file " << Sif <<".--*--\n";
	exit(1);
    }

    if (!Sr_a)
    {
	cerr <<"--*--MR::interpolated_power_spectrum(). "
	    <<"Unable to open file " << Saf <<".--*--\n";
	exit(1);
    }

    int Nbm_i, Nsm_i, Nbm_a, Nsm_a, tmp; //no. of modes 
    double Rad; // radius of cavity

    bgamma >> Rad;

    bgamma >> tmp;  //no. of bent modes
    Sr_i >> Nbm_i;  // no. of BWG modes for Inport coupler
    Sr_a >> Nbm_a;  // no. of BWG modes for Addport coupler

    if ( (tmp != Nbm_i) || (tmp != Nbm_a) )
    {
	cerr << "No. of bent modes given in file " << gf
	     <<" is not same as that of given in file " << Sif
	     <<" or in file " << Saf <<".\n";
	exit(1);
    }

    sibeta >> tmp;
    Sr_i >> Nsm_i;  // no. of SWG modes for Inport coupler
    if (tmp != Nsm_i)
    {
	cerr << "No. of straight waveguide modes given in file " << bif
	     <<" is not same as that of given in file " << Sif <<".\n";
	exit(1);
    }

    sabeta >> tmp;
    Sr_a >> Nsm_a;  // no. of SWG modes for Addport coupler
    if (tmp != Nsm_a)
    {
	cerr << "No. of straight waveguide modes given in file " << baf
	     <<" is not same as that of given in file " << Saf <<".\n";
	exit(1);
    }


    // output file   = interpolatedimate power spectrum data
    ofstream ps(output_file, ios::out);

    Cvector gamma(Nbm_i);   // propagation const
    Dvector ibeta(Nsm_i);   // Inport propagation constant
    Dvector abeta(Nsm_a);   // Addport propagation constant
    Cmatrix tSi(Nbm_i + Nsm_i,Nbm_i + Nsm_i); // S (reduced) for Inport coupler
    Cmatrix tSa(Nbm_a + Nsm_a,Nbm_a + Nsm_a); // S (reduced) for Addport coupler

    double l;               // wavelength

    double Lcav2 = PI*Rad; // Lcav by 2

    double z0_i, z1_i, z0_a, z1_a;

    Sr_i >> z0_i >> z1_i;
    Sr_a >> z0_a >> z1_a;

    Cvector tap(Nbm_i), tAp(Nsm_i);
    Cvector tbp(Nbm_i), tBp(Nsm_i);
    Cvector tcm(Nbm_a), tCm(Nsm_a);
    Cvector tdm(Nbm_a), tDm(Nsm_a);

    //initial amps of Inport modes and Addport modes
    Cvector IAp(Nsm_i), ICm(Nsm_a); 

    for(int i=0; i < Nsm_i; i++) sibeta >> IAp(i).re >> IAp(i).im;
    for(int i=0; i < Nsm_a; i++) sabeta >> ICm(i).re >> ICm(i).im;

    int r0, r1, c0, c1;

    cerr <<"Radius= " << Rad <<"\n"
	 <<"Inport BWG mode=" << Nbm_i <<"\n"
	 <<"Addport BWG mode=" << Nbm_a <<"\n"
	 <<"Inport SWG mode=" << Nsm_i <<"\n"
	 <<"Addport SWG mode=" << Nsm_a <<"\n"
	 <<"Inport z-levels=" << z0_i <<"\t" << z1_i <<"\n"
	 <<"Addport z-levels=" << z0_a <<"\t" << z1_a <<"\n"
	 <<"Inport initial amps";

    for(int i=0; i < Nsm_i; i++) 
	cerr << "\t" << IAp(i).re <<"\t" << IAp(i).im;
    
    cerr <<"\n";

    cerr << "Addport initial amps";
    for(int i=0; i < Nsm_a; i++) 
	cerr << "\t" << ICm(i).re <<"\t" << ICm(i).im;
    cerr <<"\n";
	
    //   exit(1);

    // start computationg interpolated power    
    while(!bgamma.eof())
    {
	bgamma >> l;

	// get gamma values
	for (int i=0; i < (Nbm_i); i++) bgamma >> gamma(i).re  >> gamma(i).im;

	sibeta >> l;

	// get Inport waveguide beta values
	for (int i=0; i < (Nsm_i); i++)  sibeta >> ibeta(i);

	sabeta >> l;

	// get Addport waveguide beta values
	for (int i=0; i < (Nsm_a); i++)  sabeta >> abeta(i);

	Sr_i >> l;

	// get Sr matrix for Inport coupler.
	for (int i=0; i < Nbm_i + Nsm_i; i++)
	{
	    for (int j=0; j < Nbm_i + Nsm_i; j++) 
		Sr_i >> tSi(i,j).re >> tSi(i,j).im;
	}

	Sr_a >> l;

	// get Sr matrix for Addport coupler.
	for (int i=0; i < Nbm_a + Nsm_a; i++)
	{
	    for (int j=0; j < Nbm_a + Nsm_a; j++)
		Sr_a >> tSa(i,j).re >> tSa(i,j).im;
	}


	// filling matrices start
	
	for (int i=0; i< Nsm_i; i++)
	    tAp(i) = IAp(i)* exp(-CCI*ibeta(i)*(-z0_i));
	
	for (int i=0; i< Nsm_a; i++)
	    tCm(i) = ICm(i)*exp(-CCI*abeta(i)*(-z0_a));

	// define tRHS matrix

	Cvector tRHS(4*Nbm_i + Nsm_i + Nsm_a);

	// filling up RHS matrix (RHS)
	
	for(int r= Nbm_i; r < 2*Nbm_i + Nsm_i; r++)
	{
	    for(int c= 0 ; c < Nsm_i; c++)
		tRHS(r) -= tSi( r - Nbm_i, Nbm_i + c) * tAp(c);
	}
	
	for(int r= 3*Nbm_i + Nsm_i; 
	    r < 4*Nbm_i + Nsm_i + Nsm_a; r++)
	{
	    for(int c= 0 ; c < Nsm_a; c++)
		tRHS(r) -= tSa( r - 3*Nbm_i - Nsm_i, Nbm_i + c)  * tCm(c);
	}
	
	
	Cmatrix tLHS(4*Nbm_i + Nsm_i + Nsm_a, 4*Nbm_i + Nsm_i + Nsm_a);
	
	// set all diagonal elements of tLHS to -1.0
	
	for(int i=0; i< tLHS.r; i++) tLHS(i,i) = -1.0;
	
	// first block
	r0 = 0;
	r1 = Nbm_i;
	
	for(int r=r0; r< r1; r++)
	tLHS(r, 3*Nbm_i + Nsm_i + r)=exp(-CCI*gamma(r)*Lcav2);
	
	// second block
	r0 = Nbm_i;
	r1 = 2*Nbm_i + Nsm_i;
	c0 = 0;
	c1 = Nbm_i;
	
	for(int r = r0; r < r1;  r++)
	    for (int c=c0; c <c1 ; c++)  tLHS(r,c) = tSi( r - r0,c);
	
	// third block
	r0 = 2*Nbm_i + Nsm_i;
	r1 = 3*Nbm_i + Nsm_i;
	
	for(int r=r0; r<r1; r++)
	    tLHS(r,r -Nbm_i - Nsm_i)=exp(-CCI*gamma(r - r0)*Lcav2);
	
	
	//forth block
	r0 = 3*Nbm_i + Nsm_i;
	r1 = 4*Nbm_i + Nsm_i + Nsm_a;
	c0 = 2*Nbm_i + Nsm_i;
	c1 = 3*Nbm_i + Nsm_i;
	
	for(int r = r0; r < r1;  r++)
	    for (int c=c0; c< c1; c++) tLHS(r,c) = tSa( r - r0, c - c0);
	
	
	tLHS.inverse();
	
	Cvector ap_bp_Bp_cm_dm_Dm = mult(tLHS,tRHS);
	
	// extract the amps 
	
	tap = ap_bp_Bp_cm_dm_Dm.subvector(Nbm_i,0);
	tbp = ap_bp_Bp_cm_dm_Dm.subvector(Nbm_i, Nbm_i);
	tBp = ap_bp_Bp_cm_dm_Dm.subvector(Nsm_i, 2*Nbm_i);
	tcm = ap_bp_Bp_cm_dm_Dm.subvector(Nbm_i, 2*Nbm_i + Nsm_i);
	tdm = ap_bp_Bp_cm_dm_Dm.subvector(Nbm_i, 3*Nbm_i + Nsm_i);
	tDm = ap_bp_Bp_cm_dm_Dm.subvector(Nsm_a, 4*Nbm_i + Nsm_i);
	
	// write result to file
	
	ps << l; 

//      To characterise the resonances
//	for (int i=0; i< Nbm_i; i++) ps <<  "\t" << tbp(i).sqabs();
	
	for (int i=0; i< Nsm_i; i++) ps <<  "\t" << tBp(i).sqabs();
	for (int i=0; i< Nsm_a; i++) ps << "\t" << tDm(i).sqabs();
	
	ps <<"\n";
    };
    
    ps.close();
    bgamma.close();
    sibeta.close();
    sabeta.close();
    Sr_i.close();
    Sr_a.close();

    cerr << "--*-- " << output_file << " file created. --*--\n"
	 << "--*-- MR::interpolated_power_spectrum() is leaving. --*--\n";   
	
}

//------------------------------------------------------------------

