/*
 * Circurs - Circular optical microresonators simulation tools
 * Kirankumar R. Hiremath (k.r.hiremath@ieee.org)
 * University of Twente, Department of Applied Mathematics 
 * P.O. Box 217, 7500AE Enschede, The Netherlands
 * (2005)
 */

/*
 * mr.h
 * circular microresonator 
 */


#ifndef MR_H
#define MR_H

#include "bscoupler.h"
#include <fstream>

class MR
{
 public: 

// bsc_i = input coupler, bsc_o = addport coupler
    BSCoupler bsc_i, bsc_o; 

// input amplitudes for input coupler
    Cvector BSAmpbsc_i;

// input amplitudes for output coupler
    Cvector BSAmpbsc_o;

// microresonator computational window 
// In x direction = [xc0, xc1]  and in z direction =[zc0, zc1]
    double xc0, xc1, zc0, zc1;

// discretisation points : in x direction = Nxc, in z direction = Nzc
    int Nxc, Nzc;

// discretisation step size : in x direction = hxc, in z direction = hzc
    double hxc, hzc;

// radius of the cavity
    double R;

// wavelenght of the MR
    double lambda;

// polarisation 
    Polarization pol;

// constructor    
    MR(BSCoupler &bsc_i_, 
       BSCoupler &bsc_o_, 
       double xc0_, double xc1_, int Nxc_, 
       double zc0_, double zc1_, int Nzc_,
       Cvector MRA_in_, Cvector MRA_add_);

// Check whether two coupler are compatible with each other    
    int coupler_check(void) const;

// set amplitudes (direct method)
    void set_amp(int proj, double z);

// complex valued MR field at a point (x,z) 
// cp:  ER, EY, EP, HR, HY, HP, SR, SY, SP
    Complex field(Fcomp cp, double x, double z)const;

// real valued MR field at a point (x,z) 
// cp:  ER, EY, EP, HR, HY, HP, SR, SY, SP
// foa: MOD, ORG, REP, IMP, SQR  
    double field(Fcomp cp, Afo foa, double x, double z)const;

    // ---------------------------------------------------------------
    // the optical field, discretized on a regular rectangular mesh,
    // npx x npz points on a display window dwx x dwz
    // cp: one of E?, H?, S? ... ?
    // foa: ORG, MOD, SQR, REP, IMP (ORG = REP)
    Cmatrix field(Fcomp cp, Interval dwx, int npx,
		  Interval dwz, int npz) const;
    Dmatrix field(Fcomp cp, Afo foa, Interval dwx, int npx,
		  Interval dwz, int npz) const;
    // ... determine the maximum absolute level fmax
    Cmatrix field(Fcomp cp, Interval dwx, int npx,
		  Interval dwz, int npz, double& fmax) const;
    
    // ---------------------------------------------------------------
    // visualization: output to MATLAB .m files 
    // evaluation of the mode interference pattern in the display window
    // xbeg <= x <= xend, zbeg <= z <= zend
    // npx, npz: number of plot points in the output mesh
    // ext0, ext1: filename id characters for the m.file
    // assumption: uniform polarization wavelength of all included modes 
    
    // image plot corresponding to field component cp
    // cp: one of E?, H?, S? ... ?
    // foa: ORG, MOD, SQR, REP, IMP (ORG = REP)
    void plot(Fcomp cp, Afo foa, 
	      double xbeg, double xend, double zbeg, double zend,
	      int npx, int npz, char ext0, char ext1) const; 
    
    // animation of the light propagation
    // the frames show images of EY (TE) resp. HY (TM), at
    // ntfr equidistant times over one time period
    void movie(double xbeg, double xend, double zbeg, double zend,
	       int npx, int npz, int ntfr, char ext0, char ext1) const;
    
    // export full field data ("everything") into a viewer MATLAB file
    void viewer(double xbeg, double xend, double zbeg, double zend,
		int npx, int npz, char ext0, char ext1) const;
    
   
// Approximate power spectrum with constant scattering matrix 
// and wavelenght dependent propagation constsnt
//    void approx_power_spectrum( double l0, double l1, int Nl);
				
// gives power dropped for mode with index=mode_index
    double get_pd(int mode_index)const;

// gives power transmitted mode with index=mode_index
    double get_pt(int mode_index)const;
    
//	~MR();

    
// Following variables (and their notations) are based on the model of MR
// presented by Manfred in the NAIS document dated Sept. 12'th, 2002.
    
// size of these matrices is determined by no. of modes of bend waveguide
// and straight waveguide. See constractor() for details. p/m= plus / minus
    Cvector Ap, Bp, Cp, Dp, Am, Bm, Cm, Dm; // for straight waveguide ports
    Cvector ap, bp, cp, dp, am, bm, cm, dm; // for bent waveguide ports
    
// length of the bend segments which is not included in the couplers
// L1 = Throught port to Add port
// L2 = In port to drop port
    double L1, L2;

    bool set_amp_flag;

};

// Gives power drop and through spectrum for interpolation method 
// Output file with data structure: wavelength - Through power - Drop power 
void interpolated_power_spectrum(
    char *gf,    // file of bent waveguide gamma values
    char *bif,   // file of Inport straight waveguide beta values
    char *baf,   // file of Addport straight waveguide beta values
    char *Sif,   // file of Inport S reduced matrix
    char *Saf,   // file of Addport S reduced matrix
    char *output_file); // output file 

#endif
